dofile_once( "data/scripts/lib/utilities.lua" )

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

function split_string(inputstr, sep)
  sep = sep or "%s"
  local t= {}
  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
    table.insert(t, str)
  end
  return t
end

function get_material_damage(entity_id)
  local damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
  if damage_model_component then
    local materials_that_damage = ComponentGetValue2(damage_model_component, "materials_that_damage") --"water,acid,blood,whatever"
    local materials_how_much_damage = ComponentGetValue2(damage_model_component, "materials_how_much_damage")--"1,-1,0.03,0.2"
    materials_that_damage = split_string(materials_that_damage, ",")
    materials_how_much_damage = split_string(materials_how_much_damage, ",")
    local material_damages = {}
    for i, v in ipairs(materials_that_damage) do
      material_damages[v] = tonumber(materials_how_much_damage[i])
    end
    return material_damages
  end
end

local entities = EntityGetInRadiusWithTag(x, y, 9999999, "mortal")

for i, entity in ipairs(entities) do

	local material_damages = get_material_damage(entity)

	if material_damages and material_damages.acid and material_damages.acid > 0 then

		local entity_damage_model_component = EntityGetFirstComponent(entity, "DamageModelComponent")
		
		if entity_damage_model_component then
		
			EntitySetDamageFromMaterial( entity, "cloud_acid", material_damages.acid )
			
		end
		
	end

end

function get_state()
  local entity_id = GetUpdatedEntityID()
  state = state or {}
  state[entity_id] = state[entity_id] or {}
  return state[entity_id]
end

local number_of_frames = 7
local state = get_state()
state.counter = ((state.counter or 0) % number_of_frames + 1)

function get_particle_emitter_components(entity_id)
  local all_components = EntityGetAllComponents(entity_id)
  local t = {}
  for i, comp in ipairs(all_components) do
    if ComponentGetTypeName(comp) == "ParticleEmitterComponent" then
      table.insert(t, comp)
    end
  end
  return t
end

local particle_emitter_components = get_particle_emitter_components(entity_id)
local current = state.counter - 1
if current <= 0 then
  current = current + number_of_frames
end
local next = (current % number_of_frames) + 1

EntitySetComponentIsEnabled(entity_id, particle_emitter_components[current], false)
EntitySetComponentIsEnabled(entity_id, particle_emitter_components[next], true)

--disable Ghost component if player is in holy mountain:

function get_state2()
  state2 = state2 or {}
  state2[entity_id] = state2[entity_id] or {
    last_frame_hit = 0
  }
  return state2[entity_id]
end

local state2 = get_state2()
local frames_active_after_getting_hit = 600

function damage_received()
  state2.last_frame_hit = GameGetFrameNum()
end

local is_angry = state2.last_frame_hit + frames_active_after_getting_hit > GameGetFrameNum()

local players = EntityGetWithTag("player_unit")
local ghost_component = EntityGetFirstComponentIncludingDisabled(entity_id, "GhostComponent")

if not is_angry and BiomeMapGetName() == "$biome_holymountain" and #players > 0 then
	EntitySetComponentIsEnabled( entity_id, ghost_component, false )
elseif is_angry and BiomeMapGetName() == "$biome_holymountain" and #players > 0 then
	EntitySetComponentIsEnabled( entity_id, ghost_component, true )
else
	EntitySetComponentIsEnabled( entity_id, ghost_component, true )
end