function approx_rolling_average(avg, new_sample, n)
  avg = avg - avg / n;
  avg = avg + new_sample / n;
  return avg
end
function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function get_direction( x1, y1, x2, y2 )
	return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

local entity_id = GetUpdatedEntityID()
local x, y, rot, scale_x  = EntityGetTransform(entity_id)

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local hands = 0

--Getting all my children
for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do

	--Getting Game Effects and removing them
	local game_effect_component = EntityGetFirstComponentIncludingDisabled(child, "GameEffectComponent")
	
	if ( game_effect_component ~= nil ) and ( game_effect_component > 0 ) and ( EntityGetName(child) ~= "important" ) then
	
		EntityKill(child)
		
	end

  if EntityGetName(child) == "controls" then
	pathfinding_component = EntityGetFirstComponentIncludingDisabled(child, "PathFindingComponent")
  elseif EntityGetName(child) == "torso" then
	torso_id = child
	tx, ty, trot = EntityGetTransform(torso_id)
	for i, child_of_child in ipairs(EntityGetAllChildren(torso_id)) do
		if EntityGetName(child_of_child) == "neck" then
			neck_id = child_of_child
			nx, ny = EntityGetTransform(neck_id)
		end
	end
  elseif EntityGetName(child) == "hand" then
	hands = hands + 1
  end
end

if hands == 0 then 
	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", true)
end

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 5 then
	EntitySetTransform(torso_id, x, y)
else
	EntitySetComponentsWithTagEnabled(torso_id, "enabled_in_world", true)
	
	--If physics pixel count goes below 50, remove physics body 2 and shift to physics body 1
	local physbod2_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PhysicsBody2Component")
	local physimgshape_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PhysicsImageShapeComponent")
	if physbod2_component then
		pixcount = ComponentGetValue2(physbod2_component, "mPixelCount")
		
		if pixcount < 50 then
			local imgphyscomp = EntityGetFirstComponentIncludingDisabled(entity_id, "PhysicsImageShapeComponent")
			EntityRemoveComponent( entity_id, imgphyscomp )
			EntityRemoveComponent( entity_id, physbod2_component )
			-- ComponentSetValue2( physcomp, "update_entity_transform", false )
			-- local x, y = EntityGetTransform(entity_id)
			-- EntitySetTransform(entity_id, x, y, 0)
			EntitySetComponentsWithTagEnabled(entity_id, "shoot_pos", true)
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", false)
			
			EntityAddComponent(entity_id, "PhysicsBodyComponent", {
				allow_sleep="1",
				angular_damping="2",
				fixed_rotation="0",
				is_bullet="0",
				auto_clean="0",
				linear_damping="2"
			})
		end
	end

	--Use Controls' AI to move physics body 2 like physics ai would on physics body 1
	local targ_x, targ_y = ComponentGetValueVector2(pathfinding_component, "path_next_node_vector_to")
	if physbod2_component == nil then
		PhysicsApplyForce( entity_id, targ_x, targ_y)
		PhysicsApplyForce( entity_id, 0, -20 )
		PhysicsApplyTorque(entity_id, -rot * 20)
	else
		PhysicsApplyForce( entity_id, ( targ_x * pixcount/486 ), ( targ_y * pixcount/486 ))
		PhysicsApplyForce( entity_id, 0, -pixcount * 0.07 )
		PhysicsApplyTorque(entity_id, -rot * 20 * pixcount/486)
	end


	--Move torso near head
	function constrain(entity, x2, y2, max_dist)
		local chardatacomp = EntityGetFirstComponent(entity, "CharacterDataComponent")
		local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
		local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity1, "PhysicsBody2Component") or EntityGetFirstComponent(entity1, "SimplePhysicsComponent")
		if ( chardatacomp ~= nil ) and ( chardatacomp > 0 ) then
			local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
		end
		if ( projcomp ~= nil ) and ( projcomp > 0 ) then
			local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		end
		local x1, y1 = EntityGetTransform(entity)
		if (not x1) or (not x2) then return nil end
			
		local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
		local target_x = x2 + (((x1-x2)/dist)*max_dist)
		local target_y = y2 + (((y1-y2)/dist)*max_dist)
		local target_vel_x = target_x - x1
		local target_vel_y = target_y - y1

		if dist > max_dist then
			if ( ( physcomp ~= nil ) and ( physcomp > 0 ) ) or not proj_vel_x then
				PhysicsApplyForce(entity, target_vel_x*10, target_vel_y*10)
				local direction = get_direction(tx, ty, target_x, target_y)
				PhysicsApplyTorque(torso_id, direction * -1 )
			end
			if ( chardatacomp ~= nil ) and ( chardatacomp > 0 ) then
				vel_y = vel_y - 60
				local target_mag = math.sqrt((target_vel_x^2) + (target_vel_y^2))
				local mag = math.sqrt((vel_x^2) + (vel_y^2)) / 40
				local force_x = ((target_vel_x/target_mag)*mag)
				local force_y = ((target_vel_y/target_mag)*mag)

				vel_x = ((vel_x + force_x)*1.025) + (target_vel_x*dist*0.8)
				vel_y = ((vel_y + force_y)*1.025) + (target_vel_y*dist*0.8)
				EntitySetTransform(entity, target_x, target_y)
				if ComponentGetValue2(chardatacomp, "is_on_ground") == false then vel_x = vel_x*1.16 end
				ComponentSetValue2(chardatacomp, "mVelocity", vel_x, vel_y+60)
			elseif ( projcomp ~= nil ) and ( projcomp > 0 ) then
				ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*100), proj_vel_y+(target_vel_y*100))
				EntitySetTransform(entity, target_x, target_y)
			end
		end
	end

	constrain(torso_id, x, y, 15)

	-- local distance = get_distance(tx, ty, x, y)
	-- local max_distance = 15
	-- if distance > max_distance then     
		-- local direction = get_direction(x, y, tx, ty)
		-- local tx = x - math.cos(direction) * max_distance
		-- local ty = y - math.sin(direction) * max_distance
		-- PhysicsApplyTorque(torso_id, ( 5 * direction ) )
		
		-- local lerp_factor = 0.1 -- Adjust this value to control the speed of interpolation
		-- local new_x = tx + (tx - x) * lerp_factor
		-- local new_y = ty + (ty - y) * lerp_factor
		
		-- EntitySetTransform(torso_id, new_x,new_y)
	-- end

	PhysicsApplyForce(torso_id, 0, -20)
	PhysicsApplyTorque(torso_id, -trot * 20)

end

--Control Neck limb

local angle = get_direction(nx, ny, x, y)
local distance = get_distance(nx, ny, x, y)

local sprite_width = 17
local scale = math.max(0, distance) / sprite_width

EntitySetTransform(neck_id, tx, ty, angle, scale, 1)

--Blood Gravity Suck Effects

dofile_once("data/scripts/lib/utilities.lua")
local distance_full = 300

function calculate_force_at(body_x, body_y)
	local distance = math.sqrt( ( x - body_x ) ^ 2 + ( y - body_y ) ^ 2 )
	local direction = 0 - math.atan2( ( y - body_y ), ( x - body_x ) )

	local gravity_percent = ( distance_full - distance ) / distance_full
	local gravity_coeff = 100
	
	local fx = math.cos( direction ) * ( gravity_coeff * gravity_percent )
	local fy = -math.sin( direction ) * ( gravity_coeff * gravity_percent )

	return fx,fy
end

-- attract projectiles
local entities = EntityGetInRadiusWithTag(x, y, distance_full, "selective_hole")
for _,id in ipairs(entities) do	

	local px, py = EntityGetTransform( id )

	local velocitycomp = EntityGetFirstComponent( id, "VelocityComponent" )
	if ( velocitycomp ~= nil ) then
		local fx, fy = calculate_force_at(px, py)
		edit_component( id, "VelocityComponent", function(comp,vars)
			local vel_x,vel_y = ComponentGetValue2( comp, "mVelocity")
			
			vel_x = vel_x + fx
			vel_y = vel_y + fy

			ComponentSetValue2( comp, "mVelocity", vel_x, vel_y)
		end)
	end
end
	
for i=1, Random(6, 20) do
	local rand_dir = Randomf(-math.pi, math.pi)
	local inner_radius = 20
	local outer_radius = 150
	local x1 = x + math.cos(rand_dir) * inner_radius
	local y1 = y + math.sin(rand_dir) * inner_radius
	local x2 = x + math.cos(rand_dir) * outer_radius
	local y2 = y + math.sin(rand_dir) * outer_radius
	local did_hit, hit_x, hit_y = RaytraceSurfaces(x1, y1, x2, y2)
	if did_hit then
		local particle_checker = EntityLoad( "mods/new_enemies/files/selective_hole/selective_hole_material_checker.xml", hit_x, hit_y )
		EntityAddChild(entity_id, particle_checker)
	end
end