function to_string( value )
	if value == true  then return "1" end
	if value == false then return "0" end
	return tostring(value)
end

function edit_component( entity_id, type_name, do_what )
	if not is_valid_entity( entity_id ) then
		return  
	end

	local comp = EntityGetFirstComponent( entity_id, type_name )
	if comp ~= nil then
		local modified_vars = { }
		do_what( comp, modified_vars )
		for key,value in pairs( modified_vars ) do 
			ComponentSetValue( comp, key, to_string(value) )
		end
	end
end

function get_variable_storage_component( entity_id, name )
	if not is_valid_entity( entity_id ) then  
		return nil
	end

	local components = EntityGetComponent( entity_id, "VariableStorageComponent" )
	if( components ~= nil ) then
		for _,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == name) then
				return comp_id
			end
		end
	end

	return nil
end

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function is_valid_entity( entity_id )
	return entity_id ~= nil and entity_id ~= 0
end

function component_get_value_int( entity_id, type_name, value_name, default )
	if not is_valid_entity( entity_id ) then  
		return default
	end

	local comp = EntityGetFirstComponent( entity_id, type_name )
	if comp ~= nil then
		return ComponentGetValueInt( comp, value_name )
	end

	return default
end

function get_herd_id( entity_id )
	return component_get_value_int( entity_id, "GenomeDataComponent", "herd_id", -1 )
end

function shoot_projectile( who_shot, entity_file, x, y, vel_x, vel_y, send_message )
	local entity_id = EntityLoad( entity_file, x, y )
	local herd_id   = get_herd_id( who_shot )
	if( send_message == nil ) then send_message = true end

	GameShootProjectile( who_shot, x, y, x+vel_x, y+vel_y, entity_id, send_message )

	edit_component( entity_id, "ProjectileComponent", function(comp,vars)
		vars.mWhoShot       = who_shot
		vars.mShooterHerdId = herd_id
	end)

	edit_component( entity_id, "VelocityComponent", function(comp,vars)
		ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y )
	end)

	return entity_id
end

local entity_id = GetUpdatedEntityID()
local x, y, rot, scale_x = EntityGetTransform(entity_id)

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
function check_floor(x1, x2, y1, y2)
    for i = x1, x2, 5 do
        local did_hit, _, __ = RaytracePlatforms(i - 45, y1, i + 45, y2)
        if did_hit then
            return true
        end
    end
    return false
end

local character_platforming_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
local velocity_component = EntityGetFirstComponentIncludingDisabled(entity_id, "VelocityComponent")
local vel_x, vel_y = ComponentGetValue2(velocity_component, "mVelocity")

local is_standing = check_floor(x - 90, x + 90, y - 10, y + 3)
if is_standing == true then
	ComponentSetValue2(velocity_component, "mVelocity", vel_x , 0 )
end

vel_x, vel_y = ComponentGetValue2(velocity_component, "mVelocity")

local state = GetValueNumber("state", 1)
local state_prev = GetValueNumber("state_prev", 1)
local state_delay = GetValueNumber("state_delay", 350)
local state_delay2 = GetValueNumber("state_delay2", 0)

--State1 = standing
--State2 = moving towards player
--State3 = Idle

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

local players = EntityGetWithTag("player_unit")

if #players > 0 then
	enemy_x, enemy_y = EntityGetTransform(players[1])
end

if state == 1 then
	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
	ComponentSetValue2(velocity_component, "mVelocity", 0 , vel_y )
	
	local animal_ai_component = EntityGetFirstComponentIncludingDisabled(entity_id, "AnimalAIComponent")
	local state2 = ComponentGetValue2(animal_ai_component, "ai_state")
	local difference = math.abs(15 - state2)
	
	--animation
	-- if not ( difference < 0.0001 ) then
		-- local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		-- for i, comp in ipairs(sprite_comps) do

			-- ComponentSetValue2(comp, "rect_animation", "stand")
			-- ComponentSetValue2(comp, "next_rect_animation", "stand")
			
		-- end
	-- end
	if enemy_x then
		ComponentSetValue2( animal_ai_component, "ai_state", 15 ) --set ai state to prepare an attack
		ComponentSetValue2( animal_ai_component, "mGreatestPrey", players[1] ) --set greatest prey to player
		ComponentSetValue2( animal_ai_component, "mHasFoundPrey", true ) --prevent enemy from jumping 40 meters into the sky (no idea why it happens)
		
		distance = get_distance( x, y, enemy_x, enemy_y )
		if distance > 2400 then
			SetValueNumber("state", 3)
		end
	end
	if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % state_delay == 0 then
		-- if not ( difference < 0.0001 ) then
			SetValueNumber("state", 2)
			SetValueNumber("state_delay", Random(350,650))
		-- end
	end
	
elseif state == 2 then
	
	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
	
	local vel_x, vel_y = ComponentGetValue2(velocity_component, "mVelocity")
	ComponentSetValue2(velocity_component, "mVelocity", scale_x * 60 , vel_y )
	
	--animation
	local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
	for i, comp in ipairs(sprite_comps) do

		ComponentSetValue2(comp, "rect_animation", "walk")
		ComponentSetValue2(comp, "next_rect_animation", "walk")
		
	end
	
	if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % state_delay == 0 then
		SetValueNumber("state", 1)
		SetValueNumber("state_delay", Random(250,400))
		--animation
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		for i, comp in ipairs(sprite_comps) do

			ComponentSetValue2(comp, "rect_animation", "stand")
			ComponentSetValue2(comp, "next_rect_animation", "stand")
			
		end
	end
	
	if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 30 == 0 then
		GameScreenshake(2)
	end
elseif state == 3 then

	ComponentSetValue2(velocity_component, "mVelocity", 0 , vel_y )
	
	if enemy_x then
		distance = get_distance( x, y, enemy_x, enemy_y )
		if distance < 2400 then
			SetValueNumber("state", 1)
		end
	end

end

SetValueNumber("state_prev", state)

if #players > 0 then
	enemy_x, enemy_y = EntityGetTransform(players[1])
	distance = get_distance( x, y, enemy_x, enemy_y )
	if distance > 400 then
		SetValueNumber("state_delay2", state_delay2 + 1)
		if state_delay2 > 200 then
			SetValueNumber("state_delay2", 0)
			shoot_projectile( entity_id, "data/entities/projectiles/boss_fungus_shot.xml", x , y, 0, 0)
		end
	end
end