dofile_once("data/scripts/lib/coroutines.lua")
dofile_once("data/scripts/lib/utilities.lua")
-- dofile( "data/scripts/game_helpers.lua" )
async_loop(function()

	wait(2)
	
	local entity_id = GetUpdatedEntityID()

	local projectile_component = EntityGetFirstComponentIncludingDisabled(entity_id, "ProjectileComponent")
	local shooter_id = ComponentGetValue2(projectile_component, "mWhoShot")   
	ComponentSetValue2( projectile_component, "mWhoShot", shooter_id )

	EntityAddChild(shooter_id, entity_id)

	local entity_id = GetUpdatedEntityID()
	local root_id = EntityGetRootEntity( entity_id )
	local child_entities = EntityGetAllChildren(entity_id) or {}
	for i, v in ipairs(child_entities) do
	  if EntityGetName(v) == "tongue_tip" then
		tongue_tip_entity = v
		break
	  end
	end
	
	local var_comp = EntityGetFirstComponent(entity_id, "VariableStorageComponent")

	local x, y, rot = EntityGetTransform(entity_id)
	-- Calculate the target position using raycasting
	local max_raycast_distance = 500
	local did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x + math.cos(rot) * max_raycast_distance, y + math.sin(rot) * max_raycast_distance)

	if did_hit then
		-- Calculate the direction towards the target
		local dir_x = hit_x - x
		local dir_y = hit_y - y
		local dir_length = math.sqrt(dir_x * dir_x + dir_y * dir_y)

		-- Calculate the number of explosions to spawn
		local explosion_interval = 3  -- Adjust the interval between explosions
		local num_explosions = math.floor(dir_length / explosion_interval)

		-- Calculate the step size for each explosion
		local step_x = dir_x / num_explosions
		local step_y = dir_y / num_explosions

		-- Spawn explosions along the trajectory
		for i = 1, num_explosions do
			eoffset_x = step_x * i
			eoffset_y = step_y * i
			
			EntityAddComponent2(entity_id, "SpriteComponent", {
				image_file="data/enemies_gfx/chameleon_tongue_sprite.png",
				offset_x=-eoffset_x,
				offset_y=-eoffset_y,
				has_special_scale=true,
				z_index= 0.61 - (i * 0.0001)
			})
			
			local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
			for i2, comp in ipairs(sprite_comps) do
				ComponentSetValue2(comp, "special_scale_x", 1 - (i2 * 0.001))
				ComponentSetValue2(comp, "special_scale_y", 1 - (i2 * 0.001))
				EntityRefreshSprite( entity_id, comp )
			end
			
			local range = 15
			local pos_x = x + eoffset_x
			local pos_y = y + eoffset_y
			
			
			EntityApplyTransform(tongue_tip_entity, pos_x, pos_y)
			
			-- locate nearest enemy
			local enemy, enemy_x, enemy_y
			local min_dist = 15
			for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, range, "mortal")) do
				-- is target a valid enemy
				if EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(root_id, id) < 40 and ( id ~= entity_id ) and ( id ~= root_id ) then
					local x, y = EntityGetFirstHitboxCenter( id )
					local dist = get_distance(pos_x, pos_y, x, y)
					if dist < min_dist then
						min_dist = dist
						enemy = id
						enemy_x = x
						enemy_y = y
					end
				end
			end


			SetRandomSeed(pos_x + GameGetFrameNum(), pos_y)

			-- local distance = get_distance(pos_x, pos_y, enemy_x, enemy_y)
			-- local did_hit_yes = RaytracePlatforms(pos_x, pos_y, pos_x, pos_y)
			-- if did_hit_yes then
				-- break
			-- end

			wait(0.01)
		end
		
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		local sprite_comp_table = {}
		for i2, comp in ipairs(sprite_comps) do
		  sprite_comp_table[i2] = {
			comp = comp,
			z_index = ComponentGetValue2(comp, "z_index")
		  }
		end
		table.sort(sprite_comp_table, function(a, b) return a.z_index > b.z_index end)

		for i, v in ipairs(sprite_comp_table) do
			local comp = v.comp
			
			local pos_x = x + ComponentGetValue2(comp, "offset_x")
			local pos_y = y + ComponentGetValue2(comp, "offset_y")
			EntitySetTransform(tongue_tip_entity, pos_x, pos_y)
			EntityRemoveComponent( entity_id, comp )
			
			local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
			for i2, comp in ipairs(sprite_comps) do
				ComponentSetValue2(comp, "special_scale_x", 1 - (i2 * 0.001))
				ComponentSetValue2(comp, "special_scale_y", 1 - (i2 * 0.001))
				EntityRefreshSprite( entity_id, comp )
			end 
			EntityRefreshSprite( entity_id, comp )
			wait(0.01)
		end
		
		local range = 15
		
		-- locate nearest enemy
		local enemy, enemy_x, enemy_y
		local min_dist = 15
		for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, range, "mortal")) do
			-- is target a valid enemy
			if EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(root_id, id) < 40 and ( id ~= entity_id ) and ( id ~= root_id ) then
				local x, y = EntityGetFirstHitboxCenter( id )
				local dist = get_distance(pos_x, pos_y, x, y)
				if dist < min_dist then
					min_dist = dist
					enemy = id
					enemy_x = x
					enemy_y = y
				end
			end
		end

		local distance = get_distance(x, y, enemy_x, enemy_y)

		if distance < range and enemy > 0 then
			EntityInflictDamage( enemy, 0.5, "DAMAGE_PROJECTILE", "Swallow", "BLOOD_EXPLOSION", 0, 0, root_id, enemy_x, enemy_y, 0)
		end
		
		EntityKill(entity_id)
	end
end)