local times_executed = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")

function to_string( value )
	if value == true  then return "1" end
	if value == false then return "0" end
	return tostring(value)
end

function edit_component( entity_id, type_name, do_what )
	if not is_valid_entity( entity_id ) then
		return  
	end

	local comp = EntityGetFirstComponent( entity_id, type_name )
	if comp ~= nil then
		local modified_vars = { }
		do_what( comp, modified_vars )
		for key,value in pairs( modified_vars ) do 
			ComponentSetValue( comp, key, to_string(value) )
		end
	end
end

function get_variable_storage_component( entity_id, name )
	if not is_valid_entity( entity_id ) then  
		return nil
	end

	local components = EntityGetComponent( entity_id, "VariableStorageComponent" )
	if( components ~= nil ) then
		for _,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == name) then
				return comp_id
			end
		end
	end

	return nil
end

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function is_valid_entity( entity_id )
	return entity_id ~= nil and entity_id ~= 0
end

function component_get_value_int( entity_id, type_name, value_name, default )
	if not is_valid_entity( entity_id ) then  
		return default
	end

	local comp = EntityGetFirstComponent( entity_id, type_name )
	if comp ~= nil then
		return ComponentGetValueInt( comp, value_name )
	end

	return default
end

function get_herd_id( entity_id )
	return component_get_value_int( entity_id, "GenomeDataComponent", "herd_id", -1 )
end

function shoot_projectile( who_shot, entity_file, x, y, vel_x, vel_y, send_message )
	local entity_id = EntityLoad( entity_file, x, y )
	local herd_id   = get_herd_id( who_shot )
	if( send_message == nil ) then send_message = true end

	GameShootProjectile( who_shot, x, y, x+vel_x, y+vel_y, entity_id, send_message )

	edit_component( entity_id, "ProjectileComponent", function(comp,vars)
		vars.mWhoShot       = who_shot
		vars.mShooterHerdId = herd_id
	end)

	edit_component( entity_id, "VelocityComponent", function(comp,vars)
		ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y )
	end)

	return entity_id
end

-- Function to calculate the normalized direction vector and velocity
function calculateDirectionAndVelocity(startX, startY, targetX, targetY, velocity)
    local directionX = targetX - startX
    local directionY = targetY - startY

    local length = math.sqrt(directionX^2 + directionY^2)

    -- Normalize the direction vector
    local normalizedDirectionX = directionX / length
    local normalizedDirectionY = directionY / length

    -- Calculate the final velocity vector
    local velocityX = normalizedDirectionX * velocity
    local velocityY = normalizedDirectionY * velocity

    return velocityX, velocityY
end

local range = 450

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local this_comp = GetUpdatedComponentID()

local target = EntityGetClosestWithTag(x, y, "player_unit")

if target > 0 then
else
	target = EntityGetClosestWithTag(x, y, "polymorphed_player")
	if target > 0 then
	else
		target = entity_id
	end
end

local tx, ty = EntityGetTransform(target)
local dist = get_distance(tx, ty, x, y)

local animal_ai_component = EntityGetFirstComponentIncludingDisabled(entity_id, "AnimalAIComponent")

local did_hit = RaytracePlatforms(x, y, tx, ty)

local orbcount = GameGetOrbCountThisRun()

local max_count_reduction = math.min(30, orbcount)
if ( ( did_hit or (dist > 200) ) or orbcount > 10 ) and ( times_executed % 4 == 0 ) then
	
	local max_count = math.min(33, orbcount)
	for i=1,Random(6, 22) + max_count do
		shoot_projectile( entity_id, "data/entities/projectiles/cyclops_slime_shot.xml", x , y, 0, 0)
		
		GamePlaySound("data/audio/Desktop/animals.bank", "animals/boss_centipede/spawn_minion", x, y)
		
		--animation
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		for i, comp in ipairs(sprite_comps) do

			ComponentSetValue2(comp, "rect_animation", "attack_ranged")
			ComponentSetValue2(comp, "next_rect_animation", "stand")
			
		end
	end
elseif ( did_hit or (dist > 200) ) and ( times_executed % 40 == 0 ) then
	GamePlaySound("data/audio/Desktop/animals.bank", "animals/boss_centipede/prebattle_return_to_work", x, y)
end

if dist > range then

	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", false)

	if times_executed % 5 == 0 then
		shoot_projectile( entity_id, "data/entities/animals/jungle_worm_toxic.xml", x, y, 0, 0 )
	end

	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
	
	for i, child in ipairs(EntityGetAllChildren(entity_id)  or {}) do
		if child > 0 and EntityGetName(child) ~= "important" then
			EntityKill(child)
		end
	end
	
	local var_comp = get_variable_storage_component(entity_id, "initialised")
	ComponentSetValue2(var_comp, "value_bool", false)
	
	local phys_comp = EntityGetFirstComponent(entity_id, "PhysicsAIComponent")
	ComponentSetValue2(phys_comp, "force_coeff", 20)
	
else
	
	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", true)

	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
	
	local phys_comp = EntityGetFirstComponent(entity_id, "PhysicsAIComponent")
	ComponentSetValue2(phys_comp, "force_coeff", 2)

end

if times_executed == 1 then
	local sprite_comp = EntityGetFirstComponent(entity_id, "SpriteComponent")
	ComponentSetValue2(sprite_comp, "z_index", -1)
end

if times_executed % 2 == 0 then
	PhysicsApplyTorque(entity_id, -400)
end

if dist < 300 then
	if times_executed % 10 == 0 then
		shoot_projectile( entity_id, "data/entities/animals/jungle_worm_toxic.xml", x, y, 0, 0 )
	end

	if times_executed % 2 == 0 then
		local vel_x, vel_y = calculateDirectionAndVelocity(x, y, tx, ty, Random(200, 250))
		shoot_projectile( entity_id, "data/entities/projectiles/orb_mat_radioactive_scaled.xml", x, y, vel_x, vel_y )
	end
end