dofile_once( "data/scripts/lib/utilities.lua" )

local entity_id = GetUpdatedEntityID()

--expand the material converter 
local components = EntityGetComponent(entity_id, "MagicConvertMaterialComponent")
if components then
	for k, component in pairs(components)do
		local old_radius = ComponentGetValue2(component, "radius")
		if(old_radius < 45)then
			ComponentSetValue2(component, "radius", old_radius + 1)
			ComponentSetValue2(component, "min_radius", old_radius)
		else
			ComponentSetValue2(component, "radius", 1)
			ComponentSetValue2(component, "min_radius", 1)
		end
	end
end

local x, y, rot = EntityGetTransform( entity_id )

--Smoothen out rotation
function approx_rolling_average(avg, new_sample, n)
  avg = avg - avg / n;
  avg = avg + new_sample / n;
  return avg
end

local old_average = GetValueNumber("average_rotation", 0)
local new_average = approx_rolling_average(old_average, rot, 4)
if math.abs(new_average - rot) > math.pi then
	SetValueNumber("average_rotation", rot)
else
	EntitySetTransform(entity_id, x, y, new_average)
	SetValueNumber("average_rotation", new_average)
end

--Get Jaw Position
local mandible = { x1 = 32, y1 = 0 }
local mandible_x, mandible_y = vec_rotate(mandible.x1, mandible.y1, rot)
mandible_x = x + mandible_x
mandible_y = y + mandible_y

-- locate nearest enemy
local enemy, enemy_x, enemy_y
local mandible_range = 25
local detection_range = 300
for _,id in pairs(EntityGetInRadiusWithTag(mandible_x, mandible_y, detection_range, "mortal") or {}) do
	-- is target a valid enemy
	if ( id > 0 ) and ( id ~= nil ) then
		if ( EntityGetComponent(id, "GenomeDataComponent") ~= nil ) and ( EntityGetHerdRelation(entity_id, id) < 40 ) then
			local x, y = EntityGetFirstHitboxCenter( id )
			local dist = get_distance(mandible_x, mandible_y, x, y)
			if dist < mandible_range then
				min_dist = dist
				enemy = id
				enemy_x = x
				enemy_y = y
				SetValueBool("is_attacking", true)
				in_melee_range = true
			else
				min_dist = dist
				enemy = id
				enemy_x = x
				enemy_y = y
				in_melee_range = false
			end
		end
	end
end

--moving towards target
if (enemy ~= nil) and (enemy > 0) and (EntityGetHerdRelation(entity_id, enemy) < 40) then

	--getting the direction to go to
	local dir = get_direction(mandible_x, mandible_y, enemy_x, enemy_y)
	local force = 15
	local force_x = math.cos(dir) * force
	local force_y = math.sin(dir) * force
	
	local cdc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterDataComponent")
	local velocity_x, velocity_y = ComponentGetValue2(cdc_id, "mVelocity")

	local entity_in_water, hit_x, hit_y = RaytraceSurfacesAndLiquiform( x, y, x, y )
	local entity_in_solid, hit_x2, hit_y2 = RaytracePlatforms( x, y, x, y )
	local player_in_water, hit_x3, hit_y3 = RaytraceSurfacesAndLiquiform( enemy_x, enemy_y, enemy_x, enemy_y )
	local player_in_solid, hit_x4, hit_y4 = RaytracePlatforms( enemy_x, enemy_y, enemy_x, enemy_y )
	local path_not_clear, hit_x5, hit_y5 = RaytracePlatforms( x, y, enemy_x, enemy_y )

	if entity_in_water and not entity_in_solid and player_in_water and not player_in_solid and not path_not_clear then
		ComponentSetValueVector2(cdc_id, "mVelocity", velocity_x - force_x, velocity_y + force_y)
		local fish_ai_component = EntityGetFirstComponentIncludingDisabled( entity_id, "AdvancedFishAIComponent")
	  
		if fish_ai_component then
			ComponentSetValue2( fish_ai_component, "mTargetPos", enemy_x, enemy_y )
		end
	end
end

local number = GetValueNumber("number", 0)
local is_attacking = GetValueBool("is_attacking", false)

local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
local animation = ComponentGetValue2( sprite_component, "rect_animation")
		
if ( is_attacking == true ) then

	SetValueNumber("number", number + 1)

	if number == 0 then

		if animation ~= "prepare" then
			
			ComponentSetValue2(sprite_component, "rect_animation", "prepare")
			
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/giant/attack_shoot", mandible_x, mandible_y)
			
		end
		
	elseif number == 60 then
	
		if animation ~= "smash" then
		
			ComponentSetValue2(sprite_component, "rect_animation", "smash")
			if ( enemy > 0 ) and ( enemy ~= nil ) and ( in_melee_range == true ) then
				EntityInflictDamage( enemy, 40, "DAMAGE_PHYSICS_HIT", "", "BLOOD_EXPLOSION", 0, 0, entity_id, x, y )
			end
			
			GamePlaySound("data/audio/Desktop/animals.bank", "explosions/barrel_oil", mandible_x, mandible_y)
			GamePlaySound("data/audio/Desktop/animals.bank", "explosions/cocktail", mandible_x, mandible_y)
			EntityLoad( "data/entities/misc/devourer_vapour.xml", mandible_x , mandible_y )
			
		end
	
	elseif number >= 170 then
	
		if animation ~= "open" then
		
			ComponentSetValue2(sprite_component, "rect_animation", "open")
			GamePlaySound("data/audio/Desktop/animals.bank", "collision/metalhollow", mandible_x, mandible_y)
			SetValueBool("is_attacking", false)
			SetValueNumber("number", 0)
		end
		
	end
	
end

----------------------------------------------------------------------------

local entity = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity)

function clamp(value, min, max)
    if value > max then return max elseif value < min then return min else return value end
end
function Distance(dx, dy, dx2, dy2)
    return math.sqrt(((dx-dx2)^2) + ((dy-dy2)^2))
end
function rotatePoint(x,y,cx,cy,rads)
    --local angle = radToDeg(rads)
    local sin = math.sin(rads)
    local cos = math.cos(rads)
    x = x - cx
    y = y - cy
    local new_x = (x * cos) - (y * sin)
    local new_y = (x * sin) - (y * cos)
    x = new_x + cx
    y = new_y + cy
    return x, y
end

local prev_x, prev_y = GetValueNumber("prev_x", x), GetValueNumber("prev_y", y)
local vel_x, vel_y = x-prev_x, y-prev_y

local targ_x, targ_y = GetValueNumber("targ_x", 0), GetValueNumber("targ_y", 0)
local targ_dist = Distance(x, y, targ_x, targ_y)

local targ_entity = EntityGetWithTag("player_unit")[1]--GetValueNumber("targ_entity", EntityGetWithTag("player_unit")[1])
local px, py = EntityGetTransform(targ_entity)

local state = GetValueNumber("state", 0)
local verlet_physics_component = EntityGetFirstComponent(entity, "VerletPhysicsComponent")
local positions = ComponentGetValue2(verlet_physics_component, "positions")
local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")


local number_of_segments = 49
local force_amount = 0.01
local upward_force_amount = 0.01
local damping = 49

--WIGGLE AROUND:
for i=(number_of_segments*2)-3,1,-2 do
    positions_prev[i+1] = ((positions_prev[i+1]*17) + (positions_prev[i+3])) / 18
    positions_prev[i] = (((positions_prev[i]*17) + (positions_prev[i+2])) / 18) + upward_force_amount
end

--move toward target:
positions_prev[3] = positions_prev[3] - (((targ_x-positions[3])/targ_dist))
positions_prev[4] = positions_prev[4] - (((targ_y-positions[4])/targ_dist))

ComponentSetValue2(verlet_physics_component, "positions_prev", positions_prev)
ComponentSetValue2(verlet_physics_component, "positions", positions)


if state == 0 then -- Fly around seeking prey:
    local path_obstructed, block_x, block_y = RaytracePlatforms(x, y, targ_x, targ_y)
    
    if not path_obstructed and targ_dist < 20 and math.random(20) == 1 then
        SetValueNumber("targ_x", targ_x+math.random(-70,70))
        SetValueNumber("targ_y", targ_y+math.random(-70,70))
    elseif math.random(20) == 1 then
        SetValueNumber("targ_x", targ_x+math.random(-5,5))
        SetValueNumber("targ_y", targ_y+math.random(-5,5))
    end

    --find prey within line of sight:
    local newtarg = nil
    for i, creature in ipairs(EntityGetInRadiusWithTag(x,y,100, "hittable")) do
        local px, py = EntityGetTransform(creature)
        local dmc2 = EntityGetFirstComponent(creature, "DamageModelComponent")
        local path_obstructed = RaytracePlatforms(x, y, px, py)
        if not path_obstructed and dmc2 and creature ~= entity then
            SetValueNumber("targ_entity", creature)
            SetValueNumber("state", 1)
            break
        end
    end
    --if nothing living, find a random object to inspect instead:
    if not newtarg then
        for i, creature in ipairs(EntityGetInRadius(x,y,100)) do
            local px, py = EntityGetTransform(creature)
            local path_obstructed = RaytracePlatforms(x, y, px, py)
            if not path_obstructed and creature ~= entity then
                SetValueNumber("targ_entity", creature)
                SetValueNumber("state", 1)
                break
            end
        end
    end
elseif state == 1 then --Stalk potential prey:
    local path_obstructed, block_x, block_y = RaytracePlatforms(x, y, px, py) --are we in view of target entity?
    local path2_obstructed, block2_x, block2_y = RaytracePlatforms(targ_x, targ_y, px, py) --is our target position in view of target entity?
    
    --pathfind around obstacles:
    if path_obstructed and path2_obstructed then
        for i=1,10 do
            local x2, y2 = x+math.random(-60,60), y+math.random(-60,60)
            local path2_obstructed, block2_x, block2_y = RaytracePlatforms(x, y, x2, y2)
            if path2_obstructed then x2, y2 = block2_x, block2_y end
            if not RaytracePlatforms(x2, y2, px, py) then
                SetValueNumber("targ_x", x2)
                SetValueNumber("targ_y", y2)
            end
        end
    elseif math.random(100) == 1 and not path_obstructed then
        local px2, py2 = px+math.random(-60,60), py+math.random(-60,60)
        local path2_obstructed, block2_x, block2_y = RaytracePlatforms(x, y, px2, py2)
        if not path2_obstructed then
            SetValueNumber("targ_x", px2)
            SetValueNumber("targ_y", py2)
        end
    end

elseif state == 2 then --Strike prey:
    local path_obstructed, block_x, block_y = RaytracePlatforms(x, y, px, py)
    if not path_obstructed then
        SetValueNumber("targ_x", px+math.random(-30,30))
        SetValueNumber("targ_y", py+math.random(-30,30))
    else
        SetValueNumber("state", 0)
    end
end

-- EntitySetTransform(entity, positions[1], positions[2])

------------------------------------------------------------------------------
--Segment Hitboxes

local entity = GetUpdatedEntityID()

local _, _, day, hour, minute, second = GameGetDateAndTimeLocal()
SetRandomSeed(day + hour + second, minute + second + entity)

local verlet_data = {point_count = 0, positions = {}}

local x, y = EntityGetTransform(entity)

local verlet_entities = {}
local verlet_files = {}
local empty_percentage = 0

component_read( EntityGetFirstComponent( entity, "VerletPhysicsComponent" ), { positions = {} }, function(comp)

    verlet_data.point_count = ((#comp.positions) / 2)

	--manually setting the number of segments
    for i = 1, 30, 2 do
        local point = {x = 0, y = 0}
		point.x, point.y = comp.positions[i], comp.positions[i + 1]
		table.insert(verlet_data.positions, point) 
    end
end)

function set_verlet_position(verlet, x, y)
    component_readwrite( EntityGetFirstComponent( verlet, "VerletPhysicsComponent" ), { positions = {} }, function(comp)
        comp.positions[1] = x
        comp.positions[2] = y
    end)
end

function delimiter_split(input, delimiter)
    local word_table = {}
    for word in string.gmatch(input, '([^'..delimiter..']+)') do
        table.insert(word_table, word)
    end
    return word_table
end

function tablelength(T)
    local count = 0
    for _ in pairs(T) do count = count + 1 end
    return count
end

local holder = nil

local variable_storage = EntityGetComponent(entity, "VariableStorageComponent")

for k, v in pairs(variable_storage)do
    local name = ComponentGetValue2(v, "name")

    if(name == "verlet_entity")then
        table.insert(verlet_entities, tonumber(ComponentGetValue2(v, "value_string")))     
    elseif(name == "verlet_files")then
        verlet_files = delimiter_split(ComponentGetValue2(v, "value_string"), ',')
    elseif(name == "holder")then
        holder = tonumber(ComponentGetValue2(v, "value_string"))
    end
end

function get_direction2( x1, y1, x2, y2 )
    local result = math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
    return result
end

for k, position in pairs(verlet_data.positions)do
    if(verlet_entities[1] ~= nil)then
        if(verlet_entities[k] ~= 0)then
            if(EntityGetIsAlive(verlet_entities[k]))then
				
				local hx, hy = EntityGetTransform(verlet_entities[k])
				local angle = get_direction2( hx, hy, position.x, position.y )
				
                -- EntitySetTransform(verlet_entities[k], position.x, position.y, angle - 90)
                -- set_verlet_position(verlet_entities[k], position.x, position.y)
            end
        end
    else
        if(0 == empty_percentage) and k > 1 and k % 2 == 0 then

			local verlet_file = verlet_files[Random(1, #verlet_files)]
			local verlet = EntityLoad(verlet_file, x, y)
			EntityAddChild(entity, verlet)
			if(verlet ~= nil)then
				
				EntityAddComponent(entity, "VariableStorageComponent", {
					name="verlet_entity",
					value_string=tostring(verlet),
				})
			   -- GamePrint("holder = "..holder)
				if(holder ~= 0)then
					EntityAddComponent(holder, "VariableStorageComponent", {
						name="verlet_entity",
						value_string=tostring(verlet),
					})
				end

			end
        else
            EntityAddComponent(entity, "VariableStorageComponent", {
                name="verlet_entity",
                value_string="0",
            })           
        end
    end
end

------------------------------------------------------------------------------

--fins:
local num_points = 19
local fin_index = 5
local fin_side = 1
local fin_spacing = 2.2
local fin_spacing_falloff = 0.9
for i, fin in pairs(EntityGetAllChildren(entity)) do
    if EntityGetName(fin) == "fin" then
        local verlet_physics_component2 = EntityGetFirstComponent(fin, "VerletPhysicsComponent")
        local statecomp = EntityGetFirstComponent(fin, "VariableStorageComponent")
        local aicomp = EntityGetFirstComponent(entity, "AdvancedFishAIComponent")
        local targ_x, targ_y = ComponentGetValue2(aicomp, "mTargetPos")
        local move_x, move_y = (targ_x-x), (targ_y-y)

        local positions2 = ComponentGetValue2(verlet_physics_component2, "positions")
        local positions2_prev = ComponentGetValue2(verlet_physics_component2, "positions_prev")
        local state = 0
		if ( statecomp ~= nil ) and ( statecomp > 0 ) then
			state = ComponentGetValue2(statecomp, "value_int")
		end
        local spine_x, spine_y = positions[fin_index], positions[fin_index+1]
        local vec_x, vec_y = spine_x-positions[fin_index-2], spine_y-positions[fin_index-1]
        local vec2_x, vec2_y = move_x, move_y
        local mag = math.sqrt(vec_x^2 + vec_y^2)
        local mag2 = math.sqrt(vec2_x^2 + vec2_y^2)
        vec_x, vec_y = (vec_y/mag)*fin_side*-8, (vec_x/mag)*fin_side*8
        vec2_x, vec2_y = (vec2_y/mag2)*-8, (vec2_x/mag2)*8
        local alignment = ((vec_x/8)*(vec2_y/-8)) + ((vec_y/8)*(vec2_x/8))--math.min(((vec_x/8)*(vec2_y/-8)) + ((vec_y/8)*(vec2_x/8))+0.7, 1)
        alignment = alignment / math.abs(alignment)

        local socket_x, socket_y = spine_x+(vec_x*fin_spacing), spine_y+(vec_y*fin_spacing)
        local reference_x, reference_y = socket_x-(vec_x*2), socket_y-(vec_y*2)

        local fin_reach = Distance(socket_x, socket_y, positions2[37], positions2[38])
        local fin_vec_x, fin_vec_y = (positions2[37]-socket_x)/fin_reach, (positions2[38]-socket_y)/fin_reach
        local dp1 = ((vec_x/8)*fin_vec_x) + ((vec_y/8)*fin_vec_y)
        local dp2 = ((-vec_y/8)*fin_vec_x*fin_side) + ((vec_x/8)*fin_vec_y*fin_side)
        local dp = math.min(math.abs(dp1) + math.max(dp2, 0), 1)

        local force = math.min(math.max((Distance(x, y, targ_x, targ_y)*0.01), (math.sqrt(vel_x^2 + vel_y^2))), 1) / 3
        if alignment < 0 then force = 0 end
        local flap_force = 0.007*dp*fin_reach*fin_side*force
        

        --FLAP:
        local extend_x = ((socket_x+vec_x)-positions2[3])
        local extend_y = ((socket_y+vec_y)-positions2[4])
        local retract_x = ((socket_x-(vec_y*fin_side))-positions2[3])
        local retract_y = ((socket_y+(vec_x*fin_side))-positions2[4])
        positions2_prev[3] = (positions2_prev[3] + (-vec_y*flap_force)) - (((extend_x*dp) + (retract_x*(1-dp))) * force)
        positions2_prev[4] = (positions2_prev[4] + (vec_x*flap_force)) - (((extend_y*dp) + (retract_y*(1-dp))) * force)

        for j=5,37,2 do
            local p1_x, p1_y = positions2[j-4], positions2[j-3]
            local p2_x, p2_y = positions2[j-2], positions2[j-1]
            local p3_x, p3_y = positions2[j], positions2[j+1]
            local p1_prev_x, p1_prev_y = positions2_prev[j-4], positions2_prev[j-3]
            local p2_prev_x, p2_prev_y = positions2_prev[j-2], positions2_prev[j-1]
            local p3_prev_x, p3_prev_y = positions2_prev[j], positions2_prev[j+1]

            local extend_x = (positions2[3]-socket_x)
            local extend_y = (positions2[4]-socket_y)
            local retract_x = ((-vec_y*fin_side) - (extend_x/2))*2
            local retract_y = ((vec_x*fin_side) - (extend_y/2))*2

            
            positions2_prev[j] = (p3_prev_x + (-vec_y*flap_force)) - (((extend_x*dp) + (retract_x*(1-dp))) * force)
            positions2_prev[j+1] = (p3_prev_y + (vec_x*flap_force)) - (((extend_y*dp) + (retract_y*(1-dp))) * force)

            if j < 37 then
                positions2_prev[j] = (((positions2_prev[j]*5) + (positions2_prev[j+2])) / 6)
                positions2_prev[j+1] = (((positions2_prev[j+1]*5) + (positions2_prev[j+3])) / 6)
            end

            if not ((positions2_prev[j] > 0) or (positions2_prev[j] < 0)) then 
                positions2_prev[j] = p3_prev_x
                positions2_prev[j+1] = p3_prev_y
                GamePrint("nan")
            end
        end
		
        if not encountered_stupid_nan then ComponentSetValue2(verlet_physics_component2, "positions_prev", positions2_prev); end

        EntitySetTransform(fin, socket_x, socket_y)
        fin_index = fin_index + 2
        fin_side = fin_side * -1
        fin_spacing = fin_spacing * fin_spacing_falloff
	end
end

-- --fins:
-- local fin_index = 5
-- local fin_side = 1
-- local fin_spacing = 2.2
-- local fin_spacing_falloff = 0.9
-- local fin_resting_distance = 8
-- local force_divider = 30
-- local fin_curling_multiplier = 0.2
-- for i, fin in pairs(EntityGetAllChildren(entity)) do
    -- if EntityGetName(fin) == "fin" then
        -- local verlet_physics_component2 = EntityGetFirstComponent(fin, "VerletPhysicsComponent")
        -- local statecomp = EntityGetFirstComponent(fin, "VariableStorageComponent")
        -- local aicomp = EntityGetFirstComponent(entity, "AdvancedFishAIComponent")
        -- local targ_x, targ_y = ComponentGetValue2(aicomp, "mTargetPos")
		-- local move_x, move_y = (targ_x-x), (targ_y-y)
        -- local positions2 = ComponentGetValue2(verlet_physics_component2, "positions")
        -- local positions2_prev = ComponentGetValue2(verlet_physics_component2, "positions_prev")
        -- local spine_x, spine_y = positions[fin_index], positions[fin_index+1]
        -- local vec_x, vec_y = spine_x-positions[fin_index-2], spine_y-positions[fin_index-1]
        -- local vec2_x, vec2_y = move_x, move_y
        -- local mag = math.sqrt(vec_x^2 + vec_y^2)
        -- local mag2 = math.sqrt(vec2_x^2 + vec2_y^2)
        -- vec_x, vec_y = (vec_y/mag)*fin_side*(-1 * fin_resting_distance), (vec_x/mag)*fin_side*fin_resting_distance
        -- vec2_x, vec2_y = (vec2_y/mag2)*(-1 * fin_resting_distance), (vec2_x/mag2)*fin_resting_distance
        -- local alignment = math.min(((vec_x*0.1)*(vec2_y*-0.1)) + ((vec_y*0.1)*(vec2_x*0.1))+0.7, 1)
        -- alignment = alignment / math.abs(alignment)

        -- local socket_x, socket_y = spine_x+(vec_x*fin_spacing), spine_y+(vec_y*fin_spacing)
        -- local reference_x, reference_y = socket_x-(vec2_x), socket_y-(vec2_y)

        -- local fin_reach = Distance(reference_x, reference_y, positions2[37], positions2[38])
        -- local fin_vec_x, fin_vec_y = (positions2[37]-reference_x)/fin_reach, (positions2[38]-reference_y)/fin_reach
        -- local dp = math.abs(((vec_x*fin_curling_multiplier)*fin_vec_x) + ((vec_y*fin_curling_multiplier)*fin_vec_y))
        -- local force = math.min((Distance(x, y, targ_x, targ_y)*0.008)+0.1, 0.7) / force_divider
        -- if alignment < 0 then force = force*0.5 end
        -- local flap_force = 0.03*dp*fin_reach*fin_side*force*alignment

        -- --FLAP:
        -- local encountered_stupid_nan = false --i hate nans i hate nans i hate nans i hate nans i hate nans
        -- local extend_x = ((socket_x+vec_x)-positions2[3])
        -- local extend_y = ((socket_y+vec_y)-positions2[4])
        -- local retract_x = ((socket_x-(vec_y*fin_side*alignment))-positions2[3])
        -- local retract_y = ((socket_y+(vec_x*fin_side*alignment))-positions2[4])
        -- positions2_prev[3] = (positions2_prev[3] + (-vec_y*flap_force)) - (((extend_x*dp) + (retract_x*(1-dp))) * force)
        -- positions2_prev[4] = (positions2_prev[4] + (vec_x*flap_force)) - (((extend_y*dp) + (retract_y*(1-dp))) * force)
        -- for j=5,37,2 do
            -- local p1_x, p1_y = positions2[j-4], positions2[j-3]
            -- local p2_x, p2_y = positions2[j-2], positions2[j-1]
            -- local p3_x, p3_y = positions2[j], positions2[j+1]
            -- local p1_prev_x, p1_prev_y = positions2_prev[j-4], positions2_prev[j-3]
            -- local p2_prev_x, p2_prev_y = positions2_prev[j-2], positions2_prev[j-1]
            -- local p3_prev_x, p3_prev_y = positions2_prev[j], positions2_prev[j+1]

            -- local extend_x = ((p2_x+(p2_x-p1_x))-p3_x)
            -- local extend_y = ((p2_y+(p2_y-p1_y))-p3_y)
            -- local retract_x = (socket_x+vec_x)-p3_x
            -- local retract_y = (socket_y+vec_y)-p3_y

            -- positions2_prev[j] = (p3_prev_x + (-vec_y*flap_force)) - (((extend_x*dp) + (retract_x*(1-dp))) * force)
            -- positions2_prev[j+1] = (p3_prev_y + (vec_x*flap_force)) - (((extend_y*dp) + (retract_y*(1-dp))) * force)
            -- if not ((positions2_prev[j] > 0) or (positions2_prev[j] < 0)) then encountered_stupid_nan = true end
        -- end
        
        -- if not encountered_stupid_nan then ComponentSetValue2(verlet_physics_component2, "positions_prev", positions2_prev); end

        -- EntitySetTransform(fin, socket_x, socket_y)
        -- fin_index = fin_index + 2
        -- fin_side = fin_side * -1
        -- fin_spacing = fin_spacing * fin_spacing_falloff
    -- end
-- end