dofile_once( "data/scripts/lib/utilities.lua" )
dofile_once( "data/scripts/game_helpers.lua" )

local players = EntityGetWithTag("player_unit")

local entity_id = GetUpdatedEntityID()

local x, y = EntityGetTransform( entity_id )

function split_string(inputstr, sep)
  sep = sep or "%s"
  local t= {}
  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
    table.insert(t, str)
  end
  return t
end

function get_material_damage(entity_id)
  local damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
  if damage_model_component then
    local materials_that_damage = ComponentGetValue2(damage_model_component, "materials_that_damage") --"water,acid,blood,whatever"
    local materials_how_much_damage = ComponentGetValue2(damage_model_component, "materials_how_much_damage")--"1,-1,0.03,0.2"
    materials_that_damage = split_string(materials_that_damage, ",")
    materials_how_much_damage = split_string(materials_how_much_damage, ",")
    local material_damages = {}
    for i, v in ipairs(materials_that_damage) do
      material_damages[v] = tonumber(materials_how_much_damage[i])
    end
    return material_damages
  end
end

local entities = EntityGetInRadiusWithTag(x, y, 9999999, "mortal")

for i, entity in ipairs(entities) do

	local material_damages = get_material_damage(entity)

	if material_damages and material_damages.acid and material_damages.acid > 0 then

		local entity_damage_model_component = EntityGetFirstComponent(entity, "DamageModelComponent")
		
		if entity_damage_model_component then
		
			EntitySetDamageFromMaterial( entity, "acid_weak", material_damages.acid * 0.15 )
			
		end
		
	end

end

if #players > 0 then
  
	local player_x, player_y = EntityGetTransform(players[1])
  
	if player_x > x - 20 and player_y < y + 300 and player_y > y and player_x < x + 20 then
	
		local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y + 5 , player_x, player_y)
		
		if not did_hit and (EntityGetHerdRelation(entity_id, players[1]) < 40) then
		
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
			
			play_animation( entity_id, "attack")
			
		end

	end
  
  local did_hit2, hit_x2, hit_y2 = RaytraceSurfaces(x, y + 1, x, y + 0)
  
  if not did_hit2 then
	
	local damage_model_component = EntityGetFirstComponent(entity_id, "DamageModelComponent")
		
	ComponentSetValue2(damage_model_component, "hp", 0)
	ComponentSetValue2(damage_model_component, "air_needed", true)
	ComponentSetValue2(damage_model_component, "air_in_lungs", 0)

  end
  
end

function damage_received(damage, message,entity_thats_responsible, is_fatal)

	local entity_id = GetUpdatedEntityID()
	EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
	
	play_animation( entity_id, "attack")

end