dofile_once("data/scripts/lib/utilities.lua")

function approx_rolling_average(avg, new_sample, n)
  avg = avg - avg / n;
  avg = avg + new_sample / n;
  return avg
end
function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function get_direction( x1, y1, x2, y2 )
	return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

function RadsToVec(rads)
    return math.cos(rads), math.sin(rads)
end

function rotDiff(to_angle, from_angle) 
    local end1 = math.pi
    local end2 = -end1

    if from_angle < end2 then 
        from_angle = end1 + (from_angle - end2)
    elseif from_angle > end1 then 
        from_angle = end2 + (from_angle - end1)
    end
    if to_angle < end2 then 
        to_angle = end1 + (to_angle - end2)
    elseif to_angle > end1 then 
        to_angle = end2 + (to_angle - end1)
    end


    local route1 = to_angle-from_angle
    local route2 = (end2-from_angle) + (to_angle-end1)
    local route3 = (end1-from_angle) + (to_angle-end2)

    if math.abs(route1) <= math.abs(route2) then
        if math.abs(route1) <= math.abs(route3) then
            return route1
        else
            return route3
        end
    else
        if math.abs(route2) <= math.abs(route3) then
            return route2
        else
            return route3
        end
    end
end--rotational difference
function rotatePoint(x,y,cx,cy,rads)
    local dist = get_distance(x,y,cx,cy)
    local angle = get_direction(cx,cy,x,y) + rads
    local add_x, add_y = RadsToVec(angle)
    local new_x, new_y = cx+(add_x*dist), cy+(add_y*dist)
    return new_x, new_y
end--> rotated x, y

local entity_id = GetUpdatedEntityID()
local root_id = EntityGetRootEntity( entity_id )
local x, y, rot, scale_x  = EntityGetTransform(entity_id)

--This is purely turning animation code
local last_scale_x = GetValueNumber("last_xscale", scale_x)

local var_comp = get_variable_storage_component( entity_id, "turning" )

if ComponentGetValue2(var_comp, "value_bool") == false then

	if last_scale_x ~= scale_x then
	
		ComponentSetValue2(var_comp, "value_bool", true)
		
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		
		for i, comp in ipairs(sprite_comps) do
			-- if ( ComponentGetValue2(comp, "image_file") == "data/enemies_gfx/platoon.xml" ) then
				--ComponentRemoveTag( comp, "character" )
			-- end
			
			if ( not ComponentHasTag( comp, "health_bar" ) ) then
				ComponentRemoveTag( comp, "character" )
				ComponentSetValue2( comp, "rect_animation", "turn_right" )
				ComponentSetValue2( comp, "next_rect_animation", "stand" )
			end
			
		end
		
		GamePlaySound("mods/new_enemies/data/sound/more_sounds/my_mod_audio.bank", "snd_mod/clamper_move", x, y)
		
		SetValueNumber("timeout_frames", 0)
		
	end
	
end

if ComponentGetValue2(var_comp, "value_bool") == true then
	SetValueNumber("timeout_frames", GetValueNumber("timeout_frames", 40) + 1)
	if ( GetValueNumber("timeout_frames", 40) > 40 ) then
		ComponentSetValue2(var_comp, "value_bool", false)
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		for i, comp in ipairs(sprite_comps) do
			-- if ( ComponentGetValue2(comp, "image_file") == "data/enemies_gfx/platoon.xml" ) then
				
			-- end
			
			if ( not ComponentHasTag( comp, "health_bar" ) ) then
				ComponentAddTag( comp, "character" )
			end
		end
	end
end

SetValueNumber("last_xscale", scale_x)

local animalcomp = EntityGetFirstComponent(entity_id, "AnimalAIComponent")
local greatest_prey_id = ComponentGetValue2(animalcomp, "mGreatestPrey")
local player = EntityGetClosestWithTag(x, y, "player_unit")
if greatest_prey_id > 0 then
	gprx, gpry  = EntityGetTransform(greatest_prey_id)
elseif player > 0 then
	gprx, gpry  = EntityGetTransform(player)
else
	gprx, gpry  = EntityGetTransform(entity_id)
end

local old_enemy_x = GetValueNumber("enemy_x", gprx)
local old_enemy_y = GetValueNumber("enemy_y", gpry)
enemy_x = approx_rolling_average(old_enemy_x, gprx, 50)
enemy_y = approx_rolling_average(old_enemy_y, gpry, 50)
SetValueNumber("enemy_x", enemy_x)
SetValueNumber("enemy_y", enemy_y)

-- local trunk_root = { x1 = 20 * scale_x, y1 = 23 }
-- local trootx, trooty = vec_rotate(trunk_root.x1, trunk_root.y1, rot)
-- troot_x = x + trootx
-- troot_y = y + trooty

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

--Getting Game Effects and removing them
for i, child_id in ipairs(EntityGetAllChildren(entity_id) or {}) do

	local game_effect_component = EntityGetFirstComponentIncludingDisabled(child_id, "GameEffectComponent")
	
	if game_effect_component and EntityGetName(child_id) ~= "important" then
		EntityKill(child_id)
	elseif EntityGetName(child_id) == "hand" then
		troot_x, troot_y = EntityGetTransform(child_id)
	end

end

--Constrain trunk segments
function constrain_and_elongate(entity, x2, y2, max_dist, y_scale)
	local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
	local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity, "PhysicsBody2Component") or EntityGetFirstComponent(entity, "SimplePhysicsComponent")
	local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
	local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		
	local x1, y1 = EntityGetTransform(entity)
	
	--Control Neck limb

	local angle = get_direction(x1, y1, x2, y2)
	local distance = get_distance(x1, y1, x2, y2)

	local sprite_width = 20
	local scale = math.max(0, distance) / sprite_width

	EntitySetTransform(entity, x1, y1, angle, scale, y_scale)

	if (not x1) or (not x2) then return nil end
	
	local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
	local target_x = x2 + (((x1-x2)/dist)*max_dist)
	local target_y = y2 + (((y1-y2)/dist)*max_dist)
	local target_vel_x = target_x - x1
	local target_vel_y = target_y - y1
	
		
	  local dx = x1 - x2
	  local dy = y1 - y2
	  local dist = math.sqrt(dx*dx + dy*dy)
	  if ( dist > max_dist ) or ( dist < max_dist ) and (dist~=0) then
		x1 = x2 + (dx/dist) * max_dist
		y1 = y2 + (dy/dist) * max_dist
	  end

	if dist > max_dist then
		if proj_vel_x then
			ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*5), proj_vel_y+(target_vel_y*5))
			EntitySetTransform(entity, x1, y1)
		end
	end
end

--Constrain body to a ring around the entity
function constrain_to_ring2(entity, x2, y2, max_dist, scale_of_y)
  local x1, y1, bla, blasx = EntityGetTransform(entity)
	--Control Neck limb

	local angle = get_direction(x1, y1, x2, y2)
	local distance = get_distance(x1, y1, x2, y2)

	local sprite_width = 20
	local scale = math.max(0, distance) / sprite_width
	
  local angle = get_direction(x1, y1, x2, y2)
  local dx = x1 - x2
  local dy = y1 - y2
  local dist = math.sqrt(dx*dx + dy*dy)
  if ( dist > max_dist ) or ( dist < max_dist ) and (dist~=0) then
    x1 = x2 + (dx/dist) * max_dist
    y1 = y2 + (dy/dist) * max_dist
  end
  EntitySetTransform(entity, x1, y1, angle, scale, scale_of_y)
end

--Constrain body to a ring around the entity
function constrain_to_ring(entity, x2, y2, max_dist, scale_of_y)
  local x1, y1, bla, blasx = EntityGetTransform(entity)
  local angle = get_direction(x1, y1, x2, y2)
  local dx = x1 - x2
  local dy = y1 - y2
  local dist = math.sqrt(dx*dx + dy*dy)
  if ( dist > max_dist ) or ( dist < max_dist ) and (dist~=0) then
    x1 = x2 + (dx/dist) * max_dist
    y1 = y2 + (dy/dist) * max_dist
  end
  if x1 < x2 then
	scale_calc = 1
  else
	scale_calc = -1
  end
  EntitySetTransform(entity, x1, y1, angle, blasx, scale_calc)
end

local ivalue = 0
--Getting all my children
local prevchild
for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do
  if EntityGetName(child) == "trunk" then
	ivalue = ivalue + 1
    local amount_of_children = 20
	local real_amount_of_children = 11
    local value = (amount_of_children - i + 1) / amount_of_children
	if i == 11 then
		y_value = 1
    else
		y_value = value
    end
    if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 5 then
        EntitySetTransform(child, x, y)
    elseif ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 10 then
        EntitySetComponentsWithTagEnabled(child, "enabled_in_world", true)
    end
    if not prevchild then
		local segment_x, segment_y, segment_rot = EntityGetTransform(child)
        -- constrain_and_elongate(child, troot_x, troot_y, 14, y_value)
		constrain_to_ring2(child, troot_x, troot_y, 14, y_value)
		
        --stuff Thatrius added
        local ratio1 = ivalue / real_amount_of_children
        local ratio2 = (real_amount_of_children - ivalue) / real_amount_of_children
        local angle_outward = get_direction(0, 0, scale_x * -1, 0)
        local angle_aim = get_direction(enemy_x, enemy_y, segment_x, segment_y)
        local torque = (rotDiff(angle_outward, segment_rot)*ratio2) + (rotDiff(angle_aim, segment_rot)*ratio1)
        local targ_x, targ_y = rotatePoint(segment_x, segment_y, troot_x, troot_y, torque)
		
		local projcomp = EntityGetFirstComponent(child, "VelocityComponent")
		local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		ComponentSetValue2(projcomp, "mVelocity", ( proj_vel_x * 0.5 ) + ((targ_x-segment_x) * 9), ( proj_vel_y * 0.5 ) + ((targ_y-segment_y) * 9))
    else
        local segment_x, segment_y, segment_rot = EntityGetTransform(child)
		local prevsegmentx, prevsegmenty = EntityGetTransform(prevchild)
        -- constrain_and_elongate(child, prevsegmentx, prevsegmenty, 14, y_value)
		constrain_to_ring2(child, prevsegmentx, prevsegmenty, 14, y_value)

        --stuff Thatrius added
        local ratio1 = ivalue / real_amount_of_children
        local ratio2 = (real_amount_of_children - ivalue) / real_amount_of_children
        local angle_outward = get_direction(0, 0, scale_x * -1, 0)
        local angle_aim = get_direction(enemy_x, enemy_y, segment_x, segment_y)
        local torque = (rotDiff(angle_outward, segment_rot)*ratio2) + (rotDiff(angle_aim, segment_rot)*ratio1)
        local targ_x, targ_y = rotatePoint(segment_x, segment_y, prevsegmentx, prevsegmenty, torque)
		
		local projcomp = EntityGetFirstComponent(child, "VelocityComponent")
		local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		ComponentSetValue2(projcomp, "mVelocity", ( proj_vel_x * 0.3 ) + ((targ_x-segment_x) * 10), ( proj_vel_y * 0.3 ) + ((targ_y-segment_y) * 10))
    end
    prevchild = child
	
	local trunk_end = EntityGetAllChildren(entity_id)[11]
	local segmentx, segmenty = EntityGetTransform(trunk_end)
	local targ_x, targ_y = x+GetValueNumber("trunk_end_x", x), y+GetValueNumber("trunk_end_y", y)
	if math.random(100)==1 then 
		SetValueNumber("trunk_end_x", math.random(50))
		SetValueNumber("trunk_end_y", math.random(50))
		local projcomp = EntityGetFirstComponent(trunk_end, "VelocityComponent")
		local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(targ_x-segmentx), proj_vel_y+(targ_y-segmenty))
	end
  elseif EntityGetName(child) == "body" then
    if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 5 then
        EntitySetTransform(child, x, y)
    end
	constrain_to_ring(child, x, y, 54, scale_x)
	
	--Adding detection for the surfaces
	-- surface_found = false
	-- surface_found_quantity = 0

	-- for _, child in ipairs(EntityGetAllChildren(entity_id) or {} ) do
		-- local surface_found_var_comp = EntityGetFirstComponentIncludingDisabled(child, "VariableStorageComponent")
		-- local surface_check_value = ComponentGetValue2(surface_found_var_comp, "value_bool")
		-- if surface_check_value == true then
		  -- surface_found_quantity = surface_found_quantity + 1
		-- end
	-- end

	-- if surface_found_quantity > 0 then
		-- EntitySetComponentsWithTagEnabled(root_id, "enabled_in_hand", true)
	-- else
		-- EntitySetComponentsWithTagEnabled(root_id, "enabled_in_hand", false)
	-- end
  end
end