dofile_once("data/scripts/lib/utilities.lua")

function collision_trigger(colliding_entity)
	local range = 250

	local lerp_amount = 0.99
	local bob_h = 10
	local bob_w = 10
	local bob_speed_y = 0.01
	local bob_speed_x = 0.01

	local entity_id = GetUpdatedEntityID()
	local pos_x, pos_y, rot, parent_scale_x = EntityGetTransform( entity_id )

	local child = (EntityGetAllChildren(entity_id) or {}) [1]
	if child then
	  local cx, cy, crot, cscale_x = EntityGetTransform(child)
	  EntitySetTransform(entity_id, pos_x, pos_y, rot, cscale_x)
	end

	--Check Nearest Enemy
	local enemy, enemy_x, enemy_y
	local min_dist = 9999
	for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, range, "mortal")) do
		-- is target a valid enemy
		if entity_id ~= id  and (EntityGetHerdRelation(entity_id, id) < 40) and EntityGetParent(id) ~= entity_id and (EntityGetComponent(id, "GenomeDataComponent") ~= nil)	then
			local x, y = EntityGetFirstHitboxCenter( id )
			local dist = get_distance(pos_x, pos_y, x, y)
			if dist < min_dist then
				min_dist = dist
				enemy = id
				enemy_x = x
				enemy_y = y
			end
		end
	end

	-- and (EntityGetHerdRelation(entity_id, id) < 40)

	if not enemy then return end

	-- ghost continously lerps towards a target that floats around the parent
	local target_x, target_y = EntityGetTransform(enemy)
	if target_x == nil then return end

	local time = GameGetFrameNum()
	local r = ProceduralRandomf(entity_id, 0, -1, 1)

	-- randomize times and speeds slightly so that multiple ghosts don't fly identically
	time = time + r * 10000
	bob_speed_y = bob_speed_y + (r * bob_speed_y * 0.1)
	bob_speed_x = bob_speed_x + (r * bob_speed_x * 0.1)
	lerp_amount = lerp_amount - (r * lerp_amount * 0.01)

	-- bob
	target_y = target_y + math.sin(time * bob_speed_y) * bob_h
	target_x = target_x + math.sin(time * bob_speed_x) * bob_w

	local dist_x = pos_x - target_x

	-- move towards target
	pos_x,pos_y = vec_lerp(pos_x, pos_y, target_x, target_y, lerp_amount)
	EntitySetTransform( entity_id, pos_x, pos_y, 0, 1, 1)

	-- animation state
	edit_component( entity_id, "SpriteComponent", function(comp,vars)
		local current_anim = ComponentGetValue( comp, "rect_animation")
		
		-- float when nearby and fly when further away
		local mode = "float_"
		if math.abs(dist_x) > 28 then mode = "fly_" end
		
		-- check if changing the animation is needed based on current animation and heading
		if dist_x < 2 and current_anim ~= mode.."right" then
			ComponentSetValue( comp, "rect_animation", mode.."right")
		elseif dist_x > 2 and current_anim ~= mode.."left" then
			ComponentSetValue( comp, "rect_animation", mode.."left")
		end
	end)
end