--Parallax
dofile_once( "data/scripts/lib/utilities.lua" )

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform( entity_id )

local var_comp = EntityGetFirstComponent(entity_id, "VariableStorageComponent")

if ComponentGetValue2(var_comp, "value_bool") == false then
	ComponentSetValue2(var_comp, "value_bool", true)
	
	local var_comp2 = get_variable_storage_component( entity_id, "origin" )
	ComponentSetValue2(var_comp2, "value_int", x)
	ComponentSetValue2(var_comp2, "value_string", y)
	
	SetValueNumber("origin_x", x)
	SetValueNumber("origin_y", y)
end

local ox = GetValueNumber("origin_x", x)
local oy = GetValueNumber("origin_y", y)

local cx, cy = GameGetCameraPos()

local parallaxFactor = 0.9

local x2 = (cx - ox) * parallaxFactor
local y2 = (cy - oy) * parallaxFactor

EntitySetTransform(entity_id, ox + x2, oy + y2)

function get_player()
  local players = get_players()
  if players ~= nil then
	return players[1]
  end

  -- TODO: What does this mean anyway? Player is dead..?
  return nil
end
	
local player_id = get_player()
local px, py = EntityGetTransform(player_id)

local dead_frames = GetValueNumber("dead_frames", 0)

starting_out = false
if not ( dead_frames > 0 ) then
	if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 500 then
		function set_alpha(sprite_id, alpha_change_speed)

			local alpha = ComponentGetValue2(sprite_id, "alpha")
			--0.5
			alpha = math.min(alpha+alpha_change_speed, 0.5)
			ComponentSetValue2(sprite_id, "alpha", alpha)
			
		end
		local sprite_comp_id = EntityGetFirstComponent( entity_id, "SpriteComponent" )
		set_alpha(sprite_comp_id, 0.001)
		starting_out = true
	else
		EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", true)
		--stomp when low health
		if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 300 == 0 then
			local damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
			local hp = ComponentGetValue2(damage_model_component, "hp")
			local max_hp = ComponentGetValue2(damage_model_component, "max_hp")
			
			if hp < max_hp * 0.5 then
				local sprite_offset_animator_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteOffsetAnimatorComponent")
				ComponentSetValue2( sprite_offset_animator_component, "y_amount", 10 )
				
				EntityLoad( "data/entities/projectiles/god_warrior_earthquake_spawner.xml", px , py )
				
				GamePlaySound("data/audio/Desktop/animals.bank", "explosions/barrel_oil", px, py)
				GamePlaySound("data/audio/Desktop/animals.bank", "explosions/cocktail", px, py)
				
				EntityAddComponent( entity_id, "LuaComponent", 
				{ 
					script_source_file="data/scripts/animals/god_warrior_remove_animator.lua",
					execute_every_n_frame = "16",
					remove_after_executed="1",
					execute_times = "1"
					
				} )
			end
		end

		--attack
		if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 3 == 0 then
			local range = 300
			local projectile_velocity = 50
			local scatter = 0

			local entity_id = GetUpdatedEntityID()
			local pos_x, pos_y = EntityGetFirstHitboxCenter( entity_id )

			SetRandomSeed( GameGetFrameNum(), pos_x + pos_y + entity_id )

			local it_is_attack_frame = GetValueNumber("is_attacking", 0)
			local previous_attack = GetValueNumber("previous_attack", 2)

			if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 150 == 0 then

				if it_is_attack_frame == 1 then
					SetValueNumber("is_attacking", 0)
					SetValueNumber("previous_attack", 1)
				elseif it_is_attack_frame == 2 then
					SetValueNumber("is_attacking", 0)
					SetValueNumber("previous_attack", 2)
				elseif it_is_attack_frame == 0 then
					if previous_attack == 1 then
						SetValueNumber("is_attacking", 2)
					elseif previous_attack == 2 then
						SetValueNumber("is_attacking", 1)
					else
						SetValueNumber("is_attacking", 1)
					end
				end
				
			end

			-- direction
			local vel_x, vel_y = vec_sub(px, py, pos_x, pos_y)
			vel_x,vel_y = vec_normalize(vel_x, vel_y)

			-- offset to shoot direction to avoid colliding with hand
			-- pos_x = pos_x + vel_x * 8
			-- pos_y = pos_y + vel_y * 8

			-- scatter
			vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

			-- apply velocity & shoot
			vel_x,vel_y = vec_mult(vel_x,vel_y, projectile_velocity)

			local distance = get_distance(pos_x, pos_y, px, py)

			function approx_rolling_average(avg, new_sample, n)
			  avg = avg - avg / n;
			  avg = avg + new_sample / n;
			  return avg
			end
			
			if distance < range then
				--attack
				if it_is_attack_frame == 1 then
					
					local projectile_id = shoot_projectile( entity_id, "data/entities/projectiles/god_warrior_laser.xml", pos_x , pos_y, vel_x, vel_y)
					local comp = EntityGetFirstComponent( projectile_id, "VariableStorageComponent" )
					if ( comp ~= nil ) then
						ComponentSetValue2( comp, "value_int", entity_id )
					end
				elseif ( it_is_attack_frame == 2 )  and ( ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 150 == 0 ) then
					for i=1,6 do
						local projectile_id = shoot_projectile( entity_id, "data/entities/projectiles/god_warrior_electrode_spawner.xml", pos_x , pos_y, vel_x, vel_y)
						local comp = EntityGetFirstComponent( projectile_id, "VariableStorageComponent" )
						if ( comp ~= nil ) then
							ComponentSetValue2( comp, "value_int", entity_id )
						end
						
						-- edit_component2( entity_id, "SpriteComponent", function(comp,vars)
						
							-- ComponentSetValue2( comp, "rect_animation", "attack")
							-- ComponentSetValue2( comp, "next_rect_animation", "stand")
						
						-- end)
						
					end
				end
			end
		end

	end
end

local comp = EntityGetFirstComponent( GetUpdatedEntityID(), "DamageModelComponent" )
if( comp ~= nil ) then
	--ComponentSetValue( comp, "hp", "-0.1" ) -- DEBUG: kill immediately
	local hp = ComponentGetValueFloat( comp, "hp" )
	if ( hp <= 0.0 ) then
		
		SetRandomSeed( GameGetFrameNum(), GameGetFrameNum() )

		local rand = function() return Random( -10, 10 ) end
		
		SetValueNumber("dead_frames", dead_frames + 1)

		-- kill
		if dead_frames >= 500 then
			comp = EntityGetFirstComponent( GetUpdatedEntityID(), "DamageModelComponent" )
			if( comp ~= nil ) then
				ComponentSetValue( comp, "kill_now", "1" )
			end
			StatsLogPlayerKill( GetUpdatedEntityID() )
		else
			local sprite_comp_id = EntityGetFirstComponent( entity_id, "SpriteComponent" )
			local alpha = ComponentGetValue2(sprite_comp_id, "alpha")
			if alpha > 0 then
				ComponentSetValue2(sprite_comp_id, "alpha", alpha - 0.001)
			end
			
			if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 20 == 0 then
				for i=1,Random(1,10) do
					local x = x + Random (-50, 50)
					local y = y + Random (-40, 300)
					GameScreenshake( i * 1, x, y )
					GamePlaySound( "data/audio/Desktop/misc.snd", "misc/chest_dark_open", x, y )
					EntityLoad("data/entities/particles/particle_explosion/main_small.xml", x, y )
				end
			end
		end
	end
end

if starting_out == false then

	--health bar
	local entity_id = GetUpdatedEntityID()
	-- local parent_id = EntityGetParent( entity_id )
	local players = EntityGetWithTag("player_unit")

	-- if parent_id > 0 or not (#players > 0) then

	local player_x, player_y = EntityGetTransform(players[1])
	-- EntitySetTransform( entity_id, player_x, player_y )

	local damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	local parent_damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
		
	local parent_max_health = ComponentGetValue2(parent_damage_model_component, "max_hp")
	local parent_health = ComponentGetValue2(parent_damage_model_component, "hp")


	--Cool Boss Health Gui Script
	gui = gui or GuiCreate()
	GuiStartFrame(gui)
	local screen_width, screen_height = GuiGetScreenDimensions(gui)
	local healthbar_width = (87 / 2)
	local healthbar_height = 8
	local healthbar_offset_y = screen_height * 0.213889 +25
	local hp_max = math.floor( ( parent_max_health * 25 ) + 0.5)
	local hp = math.floor( ( parent_health * 25 ) + 0.5) -- SET YOUR HP HEEEEEEEEEEEEEREEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
	if hp < 0 then
	  return
	end

	local percent_filled = hp / hp_max
	local inner_healthbar_width = (healthbar_width - 2) * percent_filled
	-- Outline / Background
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiColorSetForNextWidget(gui, 0.412, 0.243, 0.200, 1)
	GuiZSetForNextWidget(gui, -100)
	GuiImage(gui, 2, (screen_width - healthbar_width) / 2, screen_height - healthbar_offset_y, "data/debug/whitebox.png", 1, healthbar_width / 20, healthbar_height / 20) -- 20x20
	-- Actual inner bar
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiColorSetForNextWidget(gui, 0.529, 0.749, 0.110, 1)
	GuiZSetForNextWidget(gui, -101)
	GuiImage(gui, 3, (screen_width - healthbar_width) / 2 + 1, screen_height - healthbar_offset_y + 1, "data/debug/whitebox.png", 1, inner_healthbar_width / 20, (healthbar_height - 2) / 20) -- 20x20
	-- Health icon
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -102)
	GuiImage(gui, 4, (screen_width - healthbar_width) / 2 + healthbar_width + 2.5, screen_height - healthbar_offset_y - 1, "data/ui_gfx/hud/health.png", 1, 1, 1) -- 20x20
	-- Text
	local text_width = GuiGetTextDimensions(gui, hp, 1)
	GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
	GuiZSetForNextWidget(gui, -103)
	GuiText(gui, (screen_width - text_width) / 2, screen_height - healthbar_offset_y - 1, hp)
	-- "Player" string
	local player_string = GameTextGetTranslatedOrNot("$god_warrior")
	local text_width = GuiGetTextDimensions(gui, player_string, 1)
	GuiColorSetForNextWidget(gui, 0.8, 0.8, 0.8, 1)
	GuiZSetForNextWidget(gui, -103)
	GuiText(gui, (screen_width - text_width) / 2, screen_height - healthbar_offset_y - 13, player_string)

end