dofile_once( "data/scripts/lib/utilities.lua" )

---------------------------------------------------------------------------------
--Define Functions
---------------------------------------------------------------------------------
function distance(x1,y1,x2,y2)
    return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
end

function get_direction( x1, y1, x2, y2 )
  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

function dotProduct(x1, y1, x2, y2)
	return (x1*x2) + (y1*y2)
end

function approx_rolling_average(avg, target, n)
    avg = avg - avg / n;
    avg = avg + target / n;
    return avg
end
local smoothness_factor = 6

---------------------------------------------------------------------------------
--General Setup
---------------------------------------------------------------------------------
local entity = GetUpdatedEntityID()
local ex, ey, er, esx = EntityGetTransform(entity)

local parent = EntityGetParent(entity)
local px, py, rot, psx, psy = EntityGetTransform(parent)
local vel_x, vel_y = GameGetVelocityCompVelocity(parent)
local vel_mag = math.sqrt((vel_x)^2 + (vel_y)^2)
local futurex, futurey = ex+(vel_x*60), ey+(vel_y*60)

local lengthcomp = EntityGetFirstComponent(entity, "VariableStorageComponent", "length")
local target = {GetValueNumber("targ_x", ex), GetValueNumber("targ_y", ey)}
local length = ComponentGetValue2(lengthcomp, "value_int")
local time = GetValueNumber("time", 0)

local direction = GetValueNumber("direction", 1)

--These values are for bending the knees in the right direction

------------------
--Getting corners
------------------

local right_corner = { x1 = -54, y1 = 0 }
local right_x, right_y = vec_rotate(right_corner.x1, right_corner.y1, rot)
right_x = ex + right_x
right_y = ey + right_y

local left_corner = { x2 = 54, y2 = 0 }
local left_x, left_y = vec_rotate(left_corner.x2, left_corner.y2, rot)
left_x = ex + left_x
left_y = ey + left_y

local bottom_corner = { x2 = 0, y2 = 54 }
local bottom_x, bottom_y = vec_rotate(bottom_corner.x2, bottom_corner.y2, rot)
bottom_x = ex + bottom_x
bottom_y = ey + bottom_y

local top_corner = { x2 = 0, y2 = -50 }
local top_x, top_y = vec_rotate(top_corner.x2, top_corner.y2, rot)
top_x = ex + top_x
top_y = ey + top_y

-- local direction = GetValueNumber("direction", 0)

-- if angle_diff > 0 then
	-- rot_dir2 = 1
-- elseif angle_diff < 0 then
	-- rot_dir2 = 0
-- end
---------------------------------------------------------------------------------
--Inverse Kinematics
---------------------------------------------------------------------------------
--organize the segments and joints into neat little tables:
local legments = {} --"leg" + "segments"... haha funny word combo do you get
local joints = {}
local knees = {}
for i, child in ipairs(EntityGetAllChildren(entity)) do
    if not ( EntityGetName(child) == "knee" ) then
        local x, y = EntityGetTransform(child)
        if x == 0 then x = 1 end --safeguards against stupid "nan"s by making sure the code can never divide 0/0. Lost a piece of my soul in the frustration brought about by trying to track down a bug cause by this. Why does 0/0 have to equal "nan" in lua anyways and not just 0 or something, it serves no purpose except to create frustration, this is utterly retarted
        if y == 0 then y = 1 end
        table.insert(legments, child)
        if i == 1 then table.insert(joints, {ex, ey}) else table.insert(joints, {x, y}) end
    else
        table.insert(knees, child)
    end
end
table.insert(joints, target)


--First pass, starting at end of chain:
for i = #joints, 1, -1 do
    if i == #joints then
        --move joint directly to the target coords:
        joints[i] = target
    elseif i ~= 1 then
        --move joint within range of the previous one iterated:
        local jx, jy = joints[i][1], joints[i][2]
        local jx2, jy2 = joints[i+1][1], joints[i+1][2]
        if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end --more nan-avoiding bullcrap
        local joint_dist = distance(jx, jy, jx2, jy2)
        local jx = jx2 + (((jx-jx2) / joint_dist) * length)
        local jy = jy2 + (((jy-jy2) / joint_dist) * length)
        joints[i] = {jx, jy}
    end
end

--Second pass, starting at beginning of chain:
for i, joint in ipairs(joints) do
    if i ~= 1 then
        --move joint within range of the previous one iterated:
        local jx, jy = joints[i][1], joints[i][2]
        local jx2, jy2 = joints[i-1][1], joints[i-1][2]
        if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end
        local joint_dist = distance(jx, jy, jx2, jy2)
        local jx = jx2 + (((jx-jx2) / joint_dist) * length)
        local jy = jy2 + (((jy-jy2) / joint_dist) * length)
        joints[i] = {jx, jy}
    end
end

--make knees bend right way:
local reach = distance(target[1], target[2], ex, ey) --distance from socket to foot
local vec_x, vec_y = (target[1]-ex)/reach, (target[2]-ey)/reach --vector from socket to foot
local invec_x, invec_y = -vec_y, vec_x --vector perpendicular to the one above

local side_vec_x, side_vec_y = math.cos(rot), math.sin(rot) --convert the spider's rotation into a vector
local side = dotProduct(vec_x, vec_y, side_vec_x, side_vec_y) --figure out which side of the vector the foot is on
direction = ((direction*3) - (side/math.abs(side)))/4 --smoothly interpolate the "direction" variable towards "side"
if (side == 0) or (reach == 0) then direction = 1 end--safeguard against evil nans
SetValueNumber("direction", direction) --update direction variable for next run

if reach > 0 then
    for i, joint in ipairs(joints) do
        if i ~= 1 and i ~= #joints then
            local jx, jy = joints[i][1], joints[i][2]
            local dist = math.max(distance(jx, jy, ex, ey), 0.001)
            local vec2_x, vec2_y = (jx-ex)/dist, (jy-ey)/dist
            local dp = dotProduct(vec_x, vec_y, vec2_x, vec2_y)
            local cpol_x, cpol_y = ex+(vec_x*dist*dp), ey+(vec_y*dist*dp)--closest point on line between socket and foot
            local cpol_dist = distance(jx, jy, cpol_x, cpol_y)--initial dist from joint to cpol
            jx, jy = cpol_x, cpol_y--snap joint to cpol
            jx, jy = jx+(invec_x*direction*cpol_dist),jy+(invec_y*direction*cpol_dist)
            joints[i] = {jx, jy}
        end
    end
end

--Arrange all the legments to connect in between the joints:
for i, legment in ipairs(legments) do
    local jx, jy = math.floor(joints[i][1]), math.floor(joints[i][2])
    local jx2, jy2 = joints[i+1][1], joints[i+1][2]
    if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end
    local vec_x, vec_y = jx2-jx, jy2-jy
    local targ_rot = math.atan2(vec_y, vec_x)
	
    local joint_dist = distance(jx, jy, jx2, jy2)
    local div = joint_dist/length
    leg_sx = math.min(div, 1)
	
    if direction == 0 then direction = 1 end
    EntitySetTransform(legment, jx, jy, targ_rot, leg_sx, -(direction/math.abs(direction)))
    if knees[i] then EntitySetTransform(knees[i], jx2, jy2) end
end
	
---------------------------------------------------------------------------------
--AI:
---------------------------------------------------------------------------------
local total_leglength = length *#legments --combined length of all the segments in the leg
local length_tolerance = total_leglength --how far to let the end of the leg get before re-positioning it

--what the leg needs to point at:
local tx = GetValueNumber("tx", target[1])
local ty = GetValueNumber("ty", target[2])
local distance_to_target = distance(tx, ty, target[1], target[2])


--mix target position between ex, ey and tx, ty, based on time (picks up feet slightly when stepping):
local mult = smoothness_factor * (1+(2/3))
local inv = mult-time
local mix_x = (ex/mult)*time + (tx/mult)*inv
local mix_y = (ey/mult)*time + (ty/mult)*inv

--move feet toward target, unless already within range of it, in which case snap to the coordinates:
if distance_to_target > 10 then
    target[1] = approx_rolling_average(target[1], mix_x, smoothness_factor)
    target[2] = approx_rolling_average(target[2], mix_y, smoothness_factor)
elseif target[1] ~= tx and target[2] ~= ty then
    target[1] = tx
    target[2] = ty
    --[a stepping noise could go here]
end

--Stepping audio
local stepping = GetValueBool("stepping", false)
if distance_to_target < 5 then
	if stepping == false then
		SetValueBool("stepping", true)
		GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki_tiny/limb_attach", tx, ty)
	end
	
end

--if the leg stretches too far, find a new target:
if (distance(tx, ty, ex, ey) > (total_leglength) ) and distance(tx, ty, futurex, futurey) > (total_leglength) or distance(tx, ty, ex, ey) == 0 then 
    local surfaces_found = false
    --try to find nearby surfaces up to 3 times before resorting to stepping on air:
	
	local top_reference_point, trhx, trhy = RaytracePlatforms(ex, ey, top_x, top_y)
    for i=1,3 do
		-- local rx, ry = rad_to_vec( rot - math.rad(-90) )
        potential_x, potential_y = (vel_x*60)+math.random(-total_leglength*3, total_leglength*3), (vel_x*60)+math.random(-total_leglength*3, total_leglength*3)
        mag = math.sqrt(((trhx+potential_x)-trhx)^2 + ((trhy+potential_y)-trhy)^2)
        potential_x, potential_y = trhx + ((potential_x/mag)*(total_leglength*3)), trhy + ((potential_y/mag)*(total_leglength*3))
		if bottom_y > ey then
			if potential_y < ey then
				potential_y = ey + (ey - potential_y)
			end
		elseif bottom_y < ey then
			if potential_y > ey then
				potential_y = ey - (ey - potential_y)
			end
		end
		
        did_hit, hit_x, hit_y = RaytracePlatforms(ex, ey, potential_x, potential_y)
        if did_hit then
            tx = hit_x
            ty = hit_y
            surfaces_found = true
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki/limb_attach", tx, ty)
			SetValueBool("stepping", false)
            break
        end
    end
    --step on air:
    -- if not surfaces_found then
        -- tx = ex
        -- ty = ey
    -- end
    time = mult
end

--apply changes:
SetValueNumber("tx", tx)
SetValueNumber("ty", ty)
SetValueNumber("targ_x", target[1])
SetValueNumber("targ_y", target[2])
SetValueNumber("time", math.max(0,time-1))
-- SetValueNumber("direction", direction)


-- dofile_once( "data/scripts/lib/utilities.lua" )

-- ---------------------------------------------------------------------------------
-- --Define Functions
-- ---------------------------------------------------------------------------------
-- function distance(x1,y1,x2,y2)
    -- return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
-- end

-- function get_direction( x1, y1, x2, y2 )
  -- return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
-- end

-- function approx_rolling_average(avg, target, n)
    -- avg = avg - avg / n;
    -- avg = avg + target / n;
    -- return avg
-- end
-- local smoothness_factor = 6

-- ---------------------------------------------------------------------------------
-- --General Setup
-- ---------------------------------------------------------------------------------
-- local entity = GetUpdatedEntityID()
-- local ex, ey= EntityGetTransform(entity)

-- local parent = EntityGetParent(entity)
-- local px, py, rot, psx, psy = EntityGetTransform(parent)
-- local vel_x, vel_y = GameGetVelocityCompVelocity(parent)
-- local vel_mag = math.sqrt((vel_x)^2 + (vel_y)^2)
-- local futurex, futurey = ex+(vel_x*60), ey+(vel_y*60)

-- local lengthcomp = EntityGetFirstComponent(entity, "VariableStorageComponent", "length")
-- local target = {GetValueNumber("targ_x", ex), GetValueNumber("targ_y", ey)}
-- local length = ComponentGetValue2(lengthcomp, "value_int")
-- local time = GetValueNumber("time", 0)

-- ---------------------------------------------------------------------------------
-- --Inverse Kinematics
-- ---------------------------------------------------------------------------------
-- --organize the segments and joints into neat little tables:
-- local legments = {} --"leg" + "segments"... haha funny word combo do you get
-- local joints = {}
-- local knees = {}
-- for i, child in ipairs(EntityGetAllChildren(entity)) do
    -- if not ( EntityGetName(child) == "knee" ) then
        -- local x, y = EntityGetTransform(child)
        -- if x == 0 then x = 1 end --safeguards against stupid "nan"s by making sure the code can never divide 0/0. Lost a piece of my soul in the frustration brought about by trying to track down a bug cause by this. Why does 0/0 have to equal "nan" in lua anyways and not just 0 or something, it serves no purpose except to create frustration, this is utterly retarted
        -- if y == 0 then y = 1 end
        -- table.insert(legments, child)
        -- if i == 1 then table.insert(joints, {ex, ey}) else table.insert(joints, {x, y}) end
    -- else
        -- table.insert(knees, child)
    -- end
-- end
-- table.insert(joints, target)


-- --First pass, starting at end of chain:
-- for i = #joints, 1, -1 do
    -- if i == #joints then
        -- --move joint directly to the target coords:
        -- joints[i] = target
    -- elseif i ~= 1 then
        -- --move joint within range of the previous one iterated:
        -- local jx, jy = joints[i][1], joints[i][2]
        -- local jx2, jy2 = joints[i+1][1], joints[i+1][2]
        -- if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end --more nan-avoiding bullcrap
        -- local joint_dist = distance(jx, jy, jx2, jy2)
        -- local jx = jx2 + (((jx-jx2) / joint_dist) * length)
        -- local jy = jy2 + (((jy-jy2) / joint_dist) * length)
        -- joints[i] = {jx, jy}
    -- end
-- end

-- --Second pass, starting at beginning of chain:
-- for i, joint in ipairs(joints) do
    -- if i ~= 1 then
        -- --move joint within range of the previous one iterated:
        -- local jx, jy = joints[i][1], joints[i][2]
        -- local jx2, jy2 = joints[i-1][1], joints[i-1][2]
        -- if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end
        -- local joint_dist = distance(jx, jy, jx2, jy2)
        -- local jx = jx2 + (((jx-jx2) / joint_dist) * length)
        -- local jy = jy2 + (((jy-jy2) / joint_dist) * length)
        -- joints[i] = {jx, jy}
    -- end
-- end

-- --Arrange all the legments to connect in between the joints:
-- for i, legment in ipairs(legments) do
    -- local jx, jy = math.floor(joints[i][1]), math.floor(joints[i][2])
    -- local jx2, jy2 = joints[i+1][1], joints[i+1][2]
    -- if jx == jx2 or jy == jy2 then jx, jx = jx+1, jy+1 end
    -- local vec_x, vec_y = jx2-jx, jy2-jy
    -- local targ_rot = math.atan2(vec_y, vec_x)
    -- EntitySetTransform(legment, jx, jy, targ_rot)
    -- if knees[i] then EntitySetTransform(knees[i], jx2, jy2) end
-- end
-- ------------------
-- --Getting Left and right corners
-- ------------------

-- local right_corner = { x1 = -54, y1 = 0 }
-- local right_x, right_y = vec_rotate(right_corner.x1, right_corner.y1, rot)
-- right_x = ex + right_x
-- right_y = ey + right_y

-- local left_corner = { x2 = 54, y2 = 0 }
-- local left_x, left_y = vec_rotate(left_corner.x2, left_corner.y2, rot)
-- left_x = ex + left_x
-- left_y = ey + left_y

-- local bottom_corner = { x2 = 0, y2 = 54 }
-- local bottom_x, bottom_y = vec_rotate(bottom_corner.x2, bottom_corner.y2, rot)
-- bottom_x = ex + bottom_x
-- bottom_y = ey + bottom_y

-- local top_corner = { x2 = 0, y2 = -50 }
-- local top_x, top_y = vec_rotate(top_corner.x2, top_corner.y2, rot)
-- top_x = ex + top_x
-- top_y = ey + top_y
	
-- ---------------------------------------------------------------------------------
-- --AI:
-- ---------------------------------------------------------------------------------
-- local total_leglength = length *#legments --combined length of all the segments in the leg
-- local length_tolerance = total_leglength --how far to let the end of the leg get before re-positioning it

-- --what the leg needs to point at:
-- local tx = GetValueNumber("tx", target[1])
-- local ty = GetValueNumber("ty", target[2])
-- local distance_to_target = distance(tx, ty, target[1], target[2])


-- --mix target position between ex, ey and tx, ty, based on time (picks up feet slightly when stepping):
-- local mult = smoothness_factor * (1+(2/3))
-- local inv = mult-time
-- local mix_x = (ex/mult)*time + (tx/mult)*inv
-- local mix_y = (ey/mult)*time + (ty/mult)*inv

-- --move feet toward target, unless already within range of it, in which case snap to the coordinates:
-- if distance_to_target > 10 then
    -- target[1] = approx_rolling_average(target[1], mix_x, smoothness_factor)
    -- target[2] = approx_rolling_average(target[2], mix_y, smoothness_factor)
-- elseif target[1] ~= tx and target[2] ~= ty then
    -- target[1] = tx
    -- target[2] = ty
    -- --[a stepping noise could go here]
-- end

-- --Stepping audio
-- local stepping = GetValueBool("stepping", false)
-- if distance_to_target < 5 then
	-- if stepping == false then
		-- SetValueBool("stepping", true)
		-- GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki_tiny/limb_attach", tx, ty)
	-- end
	
-- end

-- --if the leg stretches too far, find a new target:
-- if (distance(tx, ty, ex, ey) > (total_leglength) ) and distance(tx, ty, futurex, futurey) > (total_leglength) or distance(tx, ty, ex, ey) == 0 then 
    -- local surfaces_found = false
    -- --try to find nearby surfaces up to 3 times before resorting to stepping on air:
	
	-- local top_reference_point, trhx, trhy = RaytracePlatforms(ex, ey, top_x, top_y)
    -- for i=1,3 do
		-- -- local rx, ry = rad_to_vec( rot - math.rad(-90) )
        -- potential_x, potential_y = (vel_x*60)+math.random(-total_leglength*3, total_leglength*3), (vel_x*60)+math.random(-total_leglength*3, total_leglength*3)
        -- mag = math.sqrt(((trhx+potential_x)-trhx)^2 + ((trhy+potential_y)-trhy)^2)
        -- potential_x, potential_y = trhx + ((potential_x/mag)*(total_leglength*3)), trhy + ((potential_y/mag)*(total_leglength*3))
		-- if bottom_y > ey then
			-- if potential_y < ey then
				-- potential_y = ey + (ey - potential_y)
			-- end
		-- elseif bottom_y < ey then
			-- if potential_y > ey then
				-- potential_y = ey - (ey - potential_y)
			-- end
		-- end
		
        -- did_hit, hit_x, hit_y = RaytracePlatforms(ex, ey, potential_x, potential_y)
        -- if did_hit then
            -- tx = hit_x
            -- ty = hit_y
            -- surfaces_found = true
			-- GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki/limb_attach", tx, ty)
			-- SetValueBool("stepping", false)
            -- break
        -- end
    -- end
    -- --step on air:
    -- -- if not surfaces_found then
        -- -- tx = ex
        -- -- ty = ey
    -- -- end
    -- time = mult
-- end

-- --apply changes:
-- SetValueNumber("tx", tx)
-- SetValueNumber("ty", ty)
-- SetValueNumber("targ_x", target[1])
-- SetValueNumber("targ_y", target[2])
-- SetValueNumber("time", math.max(0,time-1))
