dofile_once("data/scripts/lib/utilities.lua")

---------------------------------------------------------------------------------
--Define Functions
---------------------------------------------------------------------------------
function get_distance(x1,y1,x2,y2)
    return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
end

function get_direction( x1, y1, x2, y2 )
    return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

function approx_rolling_average(avg, target, n)
    avg = avg - avg / n;
    avg = avg + target / n;
    return avg
end
local smoothness_factor = 12

---------------------------------------------------------------------------------
--General Setup
---------------------------------------------------------------------------------
local entity = GetUpdatedEntityID()
local ex, ey, rotation = EntityGetTransform(entity)

local parent = EntityGetParent(entity)
local parent_x, parent_y, protatio, scale_x = EntityGetTransform(parent)
local px, py = EntityGetTransform(entity)

local root_id = EntityGetParent(parent)
local root_x, root_y, protatio, scale_x = EntityGetTransform(root_id)
local vel_x, vel_y = PhysicsGetComponentVelocity( root_id, EntityGetFirstComponentIncludingDisabled( root_id, "PhysicsBodyComponent"))

local vel_x = vel_x * 3
local vel_y = vel_y * 3

local vel_mag = math.sqrt((vel_x)^2 + (vel_y)^2)
local futurex, futurey = ex+(vel_x*60), ey+(vel_y*60)

local lengthcomp = EntityGetFirstComponent(entity, "VariableStorageComponent", "length")
local target = {GetValueNumber("targ_x", ex), GetValueNumber("targ_y", ey)}
local length = ComponentGetValue2(lengthcomp, "value_int")
local time = GetValueNumber("time", 0)

---------------------------------------------------------------------------------
--Inverse Kinematics
---------------------------------------------------------------------------------
--organize the segments and joints into neat little tables:

for i, child in ipairs(EntityGetAllChildren(entity)) do
    if EntityGetName(child) == "IK" then
        arm_id = child
		arm_x, arm_y = EntityGetTransform(child)
    elseif EntityGetName(child) == "knee" then
        hand_id = child
		hand_x, hand_y, hand_rot = EntityGetTransform(child)
    elseif EntityGetName(child) == "IK_background" then
        arm_bg_id = child
		arm_bg_x, arm_bg_y = EntityGetTransform(child)
    elseif EntityGetName(child) == "knee_background" then
        hand_bg_id = child
		hand_bg_x, hand_bg_y, hand_bg_rot = EntityGetTransform(child)
		
		-- local number = GetValueNumber("number", 0)

		-- if number ~= 6969 then
			-- SetValueNumber("number", number + 1)
		-- end

		-- if number >=60 and not number == 6969 then
			-- SetValueNumber("number", 6969)
			-- local sprite_comp = EntityGetFirstComponentIncludingDisabled( hand_id, "SpriteComponent")
			-- ComponentSetValue2( sprite_comp, "alpha", 1 )
		-- end
    end
end

---------------------------------------------------------------------------------
--AI:
---------------------------------------------------------------------------------
local total_leglength = 40
local length_tolerance = 40 --how far to let the end of the leg get before re-positioning it

--what the leg needs to point at:
local tx = GetValueNumber("tx", target[1])
local ty = GetValueNumber("ty", target[2])

local stepping = GetValueBool("stepping", false)

local found, nx, ny, apd = GetSurfaceNormal(hand_x, hand_y, 10, 5)
local in_terrain = RaytraceSurfaces(hand_x, hand_y, hand_x, hand_y)

if found == true or in_terrain == true then

	if stepping == false then
		SetValueBool("stepping", true)
	end
	
else

	if stepping == true then
		SetValueBool("stepping", false)
	end
	
end


--mix target position between ex, ey and tx, ty, based on time (picks up feet slightly when stepping):
local mult = smoothness_factor * (1+(2/3))
local inv = mult-time
local mix_x = (ex/mult)*time + (tx/mult)*inv
local mix_y = (ey/mult)*time + (ty/mult)*inv

-- if distance_to_target > 16 then

	-- local found, nx, ny, apd = GetSurfaceNormal(target[1], target[2], 3, 3)
	-- if found then
	  
	  -- nx = nx * (math.abs(apd) + 2)
	  -- ny = ny * (math.abs(apd) + 2)
	-- else
	  -- nx = 0
	  -- ny = 0
	-- end

    target[1] = approx_rolling_average(target[1], mix_x, smoothness_factor)
    target[2] = approx_rolling_average(target[2], mix_y, smoothness_factor)
-- end

local angle = get_direction(ex, ey, hand_x, hand_y)
local distance = get_distance(ex, ey, hand_x, hand_y)

--if the leg stretches too far, find a new target:
if distance > 40 then 
	surfaces_found = false
    --try to find nearby surfaces up to 3 times before resorting to stepping on air:
    for i=1,3 do
        potential_x, potential_y = (vel_x*60)+math.random(-total_leglength, total_leglength), (vel_y*60)+math.random(-total_leglength, total_leglength)
        mag = math.sqrt(((parent_x+potential_x)-parent_x)^2 + ((parent_y+potential_y)-parent_y)^2)
        potential_x, potential_y = parent_x + ((potential_x/mag)*total_leglength), parent_y + ((potential_y/mag)*total_leglength)
        did_hit, hit_x, hit_y = RaytracePlatforms(parent_x, parent_y, potential_x, potential_y)
        if did_hit then
            tx = hit_x
            ty = hit_y
			surfaces_found = true
            break
        end
    end
    --step on air:
    if not surfaces_found then
		whatever, nx2, ny2, apd2 = GetSurfaceNormal(ex, ey, 40, 10)
		did_hit2, hit_x2, hit_y2 = RaytracePlatforms(ex, ey, ex, ey + 30)
		
		if whatever then
		
			nearest_ground_x = ex + nx2 * apd2
			nearest_ground_y = ey + ny2 * apd2
			
			did_hit3, hit_x3, hit_y3 = RaytracePlatforms(ex, ey, nearest_ground_x, nearest_ground_y)
			if did_hit3 then
				tx = hit_x3
				ty = hit_y3
			else
				tx = nearest_ground_x
				ty = nearest_ground_y
			end
			
		elseif did_hit2 then
		
			tx = hit_x2
			ty = hit_y2
		
		else
		
			tx = ex
			ty = ey + 30
			
		end

    end
    time = mult
	
end

local sprite_width = 40
local max_distance = 40
local scale = math.max(0, distance) / sprite_width
scale = math.min(scale, sprite_width / max_distance)

EntitySetTransform(arm_id, ex, ey, angle - math.rad(90), scale_x, scale)
EntitySetTransform(hand_id, target[1], target[2], angle - math.rad(90), scale_x)


local limbs_attached = 0

--Toggling physics depending on how many surfaces are being grabbed

-- local did_hit, hit_x, hit_y = RaytraceSurfaces(hand_x, hand_y, hand_x, hand_y + 35)
if GetValueBool("stepping", false) == true then
	-- local surfaces_check_value = ComponentGetValue2(var_comp, "value_bool")
	-- if GetValueBool("stepping", true) then
		
		limbs_attached = limbs_attached + 1
		
		local sprite_comp = EntityGetFirstComponentIncludingDisabled(hand_id, "SpriteComponent")
		if ( ComponentGetValue2(sprite_comp, "rect_animation") ~= "close" ) and ( ComponentGetValue2(sprite_comp, "rect_animation") ~= "spawn" ) then
			ComponentSetValue2(sprite_comp, "rect_animation", "close")
			ComponentSetValue2(sprite_comp, "next_rect_animation", "close")
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki/limb_attach", tx, ty)
		end
	-- end
elseif GetValueBool("stepping", false) == false then
	-- local surfaces_check_value = ComponentGetValue2(var_comp, "value_bool")
	-- if GetValueBool("stepping", false)  then
		
		local sprite_comp = EntityGetFirstComponentIncludingDisabled(hand_id, "SpriteComponent")
		if ( ComponentGetValue2(sprite_comp, "rect_animation") ~= "open" ) and ( ComponentGetValue2(sprite_comp, "rect_animation") ~= "spawn" ) then
			ComponentSetValue2(sprite_comp, "rect_animation", "open")
			ComponentSetValue2(sprite_comp, "next_rect_animation", "open")
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki_tiny/limb_attach", tx, ty)
		end

	-- end
end

--apply changes:
SetValueNumber("tx", tx)
SetValueNumber("ty", ty)
SetValueNumber("targ_x", target[1])
SetValueNumber("targ_y", target[2])
SetValueNumber("time", math.max(0,time-1))

--Control background limb
local ikcomp = EntityGetFirstComponentIncludingDisabled(entity, "IKLimbComponent")

local limb_x, limb_y = ComponentGetValue2( ikcomp, "end_position" )

local limb_x_avg = approx_rolling_average(hand_bg_x, limb_x, smoothness_factor)
local limb_y_avg = approx_rolling_average(hand_bg_y, limb_y, smoothness_factor)
local angle2 = get_direction(ex, ey, limb_x_avg, limb_y_avg)
local distance2 = get_distance(ex, ey, hand_bg_x, hand_bg_y)

local sprite_width2 = 40
local max_distance2 = 40
local scale2 = distance2 / sprite_width2
scale2 = scale2, sprite_width2 / distance2

EntitySetTransform(arm_bg_id, ex, ey, angle2 - math.rad(90), scale_x * -1, scale2)
EntitySetTransform(hand_bg_id, limb_x_avg, limb_y_avg, angle2 - math.rad(90), scale_x * -1)

local stepping_bg = GetValueBool("stepping_bg", false)

local found2 = GetSurfaceNormal(hand_bg_x, hand_bg_y, 10, 5)
local in_terrain2 = RaytraceSurfaces(hand_bg_x, hand_bg_y, hand_bg_x, hand_bg_y)

if found2 == true or in_terrain2 == true then

	if stepping_bg == false then
		SetValueBool("stepping_bg", true)
	end
	
else

	if stepping_bg == true then
		SetValueBool("stepping_bg", false)
	end
	
end

if GetValueBool("stepping_bg", false) == true then

	limbs_attached = limbs_attached + 1
		
	local sprite_comp2 = EntityGetFirstComponentIncludingDisabled(hand_bg_id, "SpriteComponent")
	if ( ComponentGetValue2(sprite_comp2, "rect_animation") ~= "close" ) and ( ComponentGetValue2(sprite_comp2, "rect_animation") ~= "spawn" ) then
		ComponentSetValue2(sprite_comp2, "rect_animation", "close")
		ComponentSetValue2(sprite_comp2, "next_rect_animation", "close")
		GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki/limb_attach", limb_x_avg, limb_y_avg)
	end
		
elseif GetValueBool("stepping_bg", false) == false then
		
	local sprite_comp2 = EntityGetFirstComponentIncludingDisabled(hand_bg_id, "SpriteComponent")
	
	if ( ComponentGetValue2(sprite_comp2, "rect_animation") ~= "open" ) and ( ComponentGetValue2(sprite_comp2, "rect_animation") ~= "spawn" ) then
		ComponentSetValue2(sprite_comp2, "rect_animation", "open")
		ComponentSetValue2(sprite_comp2, "next_rect_animation", "open")
		GamePlaySound("data/audio/Desktop/animals.bank", "animals/lukki_tiny/limb_attach", limb_x_avg, limb_y_avg)
	end
		
end

local var_comp = EntityGetFirstComponentIncludingDisabled(entity, "VariableStorageComponent")

if limbs_attached > 0 then
	ComponentSetValue2(var_comp, "value_bool", true)
else
	ComponentSetValue2(var_comp, "value_bool", false)
end