dofile_once("data/scripts/lib/utilities.lua")

function collision_trigger(colliding_entity)

	function get_distance( x1, y1, x2, y2 )
		local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
		return result
	end

	function constrain(entity, x2, y2, max_dist)
		local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
		local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
		local x1, y1 = EntityGetTransform(entity)
		if (not x1) or (not x2) then return nil end
			
		local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
		local target_x = x2 + (((x1-x2)/dist)*max_dist)
		local target_y = y2 + (((y1-y2)/dist)*max_dist)
		local target_vel_x = target_x - x1
		local target_vel_y = target_y - y1

		if dist > max_dist then
			if proj_vel_x then
				ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*100), proj_vel_y+(target_vel_y*100))
				EntitySetTransform(entity, target_x, target_y)
			end
		end
	end

	local entity_id = GetUpdatedEntityID()
	local x, y, rot = EntityGetTransform(entity_id)

	local maximum = 200

	local state = GetValueNumber("state", 1)
	local timeout = GetValueNumber("timeout", 0)

	local sprite_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	local crawler_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CrawlerAnimalComponent")
	local velocity_component = EntityGetFirstComponentIncludingDisabled(entity_id, "VelocityComponent")

	-- if ComponentGetValue2(crawler_component, "mFrameNextGiveUp") < 999999999 then
		-- local dir_bool = ComponentGetValue2(crawler_component, "mDir")
		-- ComponentSetValue2(crawler_component, "mDir", not dir_bool)
	-- end
	-- ComponentSetValue2(crawler_component, "mFrameNextGiveUp", 999999999)

	--State1 = crawling
	--State2 = falling

	if state == 1 then
		if ComponentGetValue2(sprite_component, "rect_animation") ~= "attack" then
			ComponentSetValue2(sprite_component, "rect_animation", "moving")
			ComponentSetValue2(sprite_component, "next_rect_animation", "moving")
		end
		if ( crawler_component ~= nil ) and ( crawler_component > 0 ) then
			local is_falling = ComponentGetValue2(crawler_component, "mPrevFalling")
			
			local did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x, y - 5)
			if (is_falling == true ) and did_hit then
				-- EntitySetComponentsWithTagEnabled(entity_id, "pitcheck_b", true)
				-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
				SetValueNumber("state", 2)
				
				SetValueNumber("origin_x", x)
				SetValueNumber("origin_y", y)
				
				local slime_id = EntityLoad("data/entities/animals/larva_slime.xml", x, y)
				local var_comp1 = get_variable_storage_component( slime_id, "origin_x" )
				local var_comp2 = get_variable_storage_component( slime_id, "origin_y" )
				ComponentSetValue2(var_comp1, "value_float", x)
				ComponentSetValue2(var_comp2, "value_float", y)
				EntityAddChild(entity_id, slime_id)
				ComponentRemoveTag(sprite_component, "character" )
				ComponentSetValue2(sprite_component, "rect_animation", "tether")
				ComponentSetValue2(sprite_component, "next_rect_animation", "tether")
			end
		end
		
	elseif state == 2 then
		ComponentSetValue2(sprite_component, "rect_animation", "tether")
		ComponentSetValue2(sprite_component, "next_rect_animation", "tether")
		if timeout < maximum then
			SetValueNumber("timeout", timeout + 1)
		end
		
		if timeout >= maximum then
			ComponentSetValue2(crawler_component, "mIsInitialized", false)
			local did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x, y - 20)

			if did_hit then
				EntitySetTransform(entity_id, hit_x, hit_y + 5)
				EntityAddComponent(entity_id, "MoveToSurfaceOnCreateComponent", {
					offset_from_surface="1"
				})
			end
			ComponentAddTag( sprite_component, "character" )
			SetValueNumber("state", 1)
			for _, child in ipairs(EntityGetAllChildren(entity_id) or {} ) do
				if EntityGetName(child) == "slime" then
					EntityKill(child)
				end
			end
			
			ComponentSetValueVector2(velocity_component, "mVelocity", 0, 0)
			ComponentSetValue2(crawler_component, "mVelocityY", 0)
			SetValueNumber("timeout", 0)
			-- SetValueNumber("terrain_found", 0)
			-- EntitySetComponentsWithTagEnabled(entity_id, "pitcheck_b", false)
			-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
		end
		
		if timeout > (maximum/6) then
			local did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x, y + 1)

			if did_hit then
				EntitySetTransform(entity_id, hit_x, hit_y)
				
				ComponentSetValueVector2(velocity_component, "mVelocity", 0, 0)
				ComponentSetValue2(crawler_component, "mVelocityY", 0)
				SetValueNumber("state", 1)
				ComponentAddTag( sprite_component, "character" )
				for _, child in ipairs(EntityGetAllChildren(entity_id) or {} ) do
					if EntityGetName(child) == "slime" then
						EntityKill(child)
					end
				end
				SetValueNumber("timeout", 0)
					-- SetValueNumber("terrain_found", 0)
				-- EntitySetComponentsWithTagEnabled(entity_id, "pitcheck_b", false)
			end
				-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
			
		end
		local origin_x = GetValueNumber("origin_x", x)
		local origin_y = GetValueNumber("origin_y", y)
		if timeout < (maximum / 2) then
			constrain(entity_id, origin_x, origin_y, timeout)
		elseif timeout < maximum then
			constrain(entity_id, origin_x, origin_y, maximum - timeout)
		end
	end
	
	local cx, cy = EntityGetTransform(colliding_entity)
	local crawler_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CrawlerAnimalComponent")
	local sprite_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	local dist = get_distance(x, y, cx, cy)
	if dist < 7 then
		if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 30 == 0 then
			ComponentSetValue2(crawler_component, "mFrameNextDamage", 0)
		else
			ComponentSetValue2(crawler_component, "mFrameNextDamage", 9999999)
		end
	else
		ComponentSetValue2(crawler_component, "mFrameNextDamage", 9999999)
	end
end