dofile_once("data/scripts/lib/utilities.lua")

function damage_received( dmg, msg, source, is_fatal)
	local entity_id2 = GetUpdatedEntityID()
	if ( source ~= nil ) and ( source ~= NULL_ENTITY ) and ( source ~= entity_id2 ) then
		SetValueBool("override", true)
		SetValueNumber("override_target", source)
	end
end

function distance(x1,y1,x2,y2)
    return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
end

local entity_id = GetUpdatedEntityID()
local x, y, rot = EntityGetTransform(entity_id)

local state = GetValueNumber("state", 0)

--Check Nearest Enemy

function RepelEntities(entity1_id, entity2_id, repulsion_force)
    -- Get positions of entities
    local x1, y1 = EntityGetTransform(entity1_id)
    local x2, y2 = EntityGetTransform(entity2_id)

    -- Calculate direction from entity1 to entity2
    local dx = x2 - x1
    local dy = y2 - y1
    local distance = math.sqrt(dx^2 + dy^2)

    -- Normalize direction vector
    if distance ~= 0 then
        dx = dx / distance
        dy = dy / distance
    end

    -- Apply repulsion force to both entities
    local vel_x1, vel_y1 = GameGetVelocityCompVelocity( entity1_id )
    local vel_x2, vel_y2 = GameGetVelocityCompVelocity( entity2_id )

    vel_x1 = vel_x1 - dx * repulsion_force
    vel_y1 = vel_y1 - dy * repulsion_force

    vel_x2 = vel_x2 + dx * repulsion_force
    vel_y2 = vel_y2 + dy * repulsion_force

    -- Set new velocities
	edit_component( entity1_id, "VelocityComponent", function(comp,vars)
		local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
		ComponentSetValueVector2( comp, "mVelocity", vel_x + vel_x1, vel_y + vel_y1)
	end)
	
	edit_component( entity2_id, "VelocityComponent", function(comp,vars)
		local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
		ComponentSetValueVector2( comp, "mVelocity", vel_x + vel_x2, vel_y + vel_y2)
	end)
end

local override = GetValueBool("override", false)
local override_target = GetValueNumber("override_target", 0)
local tick3 = GetValueInteger("tick3", 0)

if override == true then
	SetValueInteger("tick3", tick3 + 1)
end

if tick3 > 600 then
	SetValueBool("override", false)
	SetValueInteger("tick3", 0)
	SetValueNumber("override_target", 0)
end

function GetSurfaceNormal(pos_x, pos_y, ray_length, ray_count)
	local total_normal_x = 0
	local total_normal_y = 0
	local total_distance = 0
	local found_normal = false

	for i = 1, ray_count do
		local angle = (i - 1) * (2 * math.pi / ray_count)
		local ray_x = pos_x + ray_length * math.cos(angle)
		local ray_y = pos_y + ray_length * math.sin(angle)

		local did_hit, hit_x, hit_y = RaytracePlatforms(pos_x, pos_y, ray_x, ray_y)

		if did_hit then
			found_normal = true
			local normal_x, normal_y = hit_x - pos_x, hit_y - pos_y
			total_normal_x = total_normal_x + normal_x
			total_normal_y = total_normal_y + normal_y
			total_distance = total_distance + math.sqrt(normal_x^2 + normal_y^2)
		end
	end

	if found_normal then
		local average_normal_x = total_normal_x / ray_count
		local average_normal_y = total_normal_y / ray_count
		local average_distance = total_distance / ray_count
		return true, average_normal_x, average_normal_y, average_distance
	else
		return false, 0, 0, 0
	end
end

function avoid_walls(entity)
	local ex, ey = EntityGetTransform(entity)
	local found, nx, ny, apd = GetSurfaceNormal(ex, ey, 5, 10)
	if found then
	  nx = nx * (math.abs(apd) + 2)
	  ny = ny * (math.abs(apd) + 2)
	else
	  nx = 0
	  ny = 0
	end
	EntitySetTransform(entity, ex - nx, ey - ny)
end

-- Control the global direction
for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do
    if EntityGetName(child) == "controls" then
	
		cx, cy = EntityGetTransform(child)
		
		-- GameCreateSpriteForXFrames( "data/debug/sui_close_button.png", cx, cy, true, 0, 0, 1, true )

		local enemy = 0
		local enemy_x = 0
		local enemy_y = 0
		
		local radius = 250
		if state == 0 then
			radius = 150
		end
		
		if override == false then
			local mortals = EntityGetInRadiusWithTag(x, y, radius, "mortal")
			local players = EntityGetInRadiusWithTag(x, y, radius, "player_unit")
			if #players > 0 and (EntityGetHerdRelation(entity_id, players[1]) < 40) then
				mortals = players
			end
			for _,id in pairs(mortals) do
				-- is target a valid enemy
				if entity_id ~= id and child ~= id and (EntityGetHerdRelation(entity_id, id) < 40) and (EntityGetComponent(id, "GenomeDataComponent") ~= nil)	then
					local ex, ey = EntityGetFirstHitboxCenter( id )
					enemy = id
					enemy_x = ex
					enemy_y = ey
				end
			end
		else
			if override_target > 0 then
				local tx, ty = EntityGetTransform(override_target)
				if tx ~= 0 then
					enemy = override_target
					enemy_x = tx
					enemy_y = ty
				end
			else
				SetValueBool("override", false)
			end
		end
			
		if state == 0 then
			
			if enemy > 0 then
				-- local did_hit, hit_x, hit_y = RaytraceSurfaces(cx, cy - 5 , enemy_x, enemy_y)
				-- if not did_hit then
					EntitySetComponentsWithTagEnabled(child, "enabled_in_world", false)
					SetValueNumber("state", 1)
				-- end
			end
			
		elseif state == 1 then
		
			avoid_walls(child)
			
			if enemy > 0 then
				local animal_ai_component = EntityGetFirstComponentIncludingDisabled(child, "AnimalAIComponent")
				if ( animal_ai_component ~= nil ) and ( animal_ai_component > 0 ) then
					ComponentSetValue2( animal_ai_component, "mGreatestPrey", enemy )
					ComponentSetValue2( animal_ai_component, "mHasFoundPrey", true )
					ComponentSetValueVector2( animal_ai_component, "mHomePosition", enemy_x, enemy_y )
				end
			end
			local dist = distance(x,y,cx,cy)
			
			local tick2 = GetValueInteger("tick2", 0)
			SetValueInteger("tick2", tick2 + 1)
			
			if ( tick2 > 1200 ) or ( dist > 200 ) then
				SetValueNumber("state", 2)
				SetValueInteger("tick2", 0)
			end
			
		elseif state == 2 then
		
			avoid_walls(child)
			
			local animal_ai_component = EntityGetFirstComponentIncludingDisabled(child, "AnimalAIComponent")
			if ( animal_ai_component ~= nil ) and ( animal_ai_component > 0 ) then
				ComponentSetValue2( animal_ai_component, "mGreatestPrey", entity_id )
				ComponentSetValue2( animal_ai_component, "mHasFoundPrey", true )
				ComponentSetValueVector2( animal_ai_component, "mHomePosition", x, y )
			end
			local tick = GetValueInteger("tick", 0)
			SetValueInteger("tick", tick + 1)
			
			local dist = distance(x,y,cx,cy)
			
			if ( tick > 1200 ) or ( dist < 30 ) then
				SetValueNumber("state", 0)
				SetValueInteger("tick", 0)
				EntitySetComponentsWithTagEnabled(child, "enabled_in_world", true)
			end
		end
    end
end

for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do

    if EntityGetName(child) == "triangle" then
		local is_avoiding_walls = false
		local cx2, cy2 = EntityGetTransform(child)
		
		local dist = distance(x,y,cx2,cy2)
		
		--Avoid walls
		local vel_x_new = 0
		local vel_y_new = 0
		local raycast_distance = 10
		local strength = 0.3

		local raycast_dirs = { {1,0}, {0,-1}, {-1,0}, {0,1} }

		for i=1,4 do
			local rdirs = raycast_dirs[i]
			
			local rdir_x = rdirs[1]
			local rdir_y = rdirs[2]
			
			local raycast_success,r_x,r_y = RaytraceSurfaces( cx2, cy2, cx2 + rdir_x * raycast_distance, cy2 + rdir_y * raycast_distance )
			
			if raycast_success then
				--print( tostring(i) .. ": " .. tostring(( math.abs( r_x - pos_x ) )) .. ", " .. tostring(( math.abs( r_y - pos_y ) )) )
				is_avoiding_walls = true
				vel_x_new = vel_x_new - ( raycast_distance ^ 2 - ( r_x - cx2 ) ^ 2 ) * rdir_x * strength
				vel_y_new = vel_y_new - ( raycast_distance ^ 2 - ( r_y - cy2 ) ^ 2 ) * rdir_y * strength
			end
		end
		
		local dist2 = distance(cx2,cy2,cx,cy)
		
		-- Calculate the dampening factor based on the distance
		local max_dampening_distance = 30 -- Adjust as needed
		local dampening_factor = 0
		if dist2 < max_dampening_distance then
			dampening_factor = (max_dampening_distance - dist2) / max_dampening_distance
		end

		-- Apply the dampening factor to the velocities

		edit_component( child, "VelocityComponent", function(comp,vars)
			local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity")
			local velocity_magnitude = math.sqrt(vel_x^2 + vel_y^2)
			
			-- Normalize the velocity vector (calculate unit vector)
			local normalized_vel_x = vel_x / velocity_magnitude
			local normalized_vel_y = vel_y / velocity_magnitude
			
			local desired_magnitude = 30 -- Change this to your desired magnitude
			
			local extra_vel_x = 0
			local extra_vel_y = 0
			if is_avoiding_walls == false then
				extra_vel_x = normalized_vel_x * desired_magnitude
				extra_vel_y = normalized_vel_y * desired_magnitude
			end
			
			vel_x = extra_vel_x + ( vel_x * (1 - dampening_factor) )
			vel_y = extra_vel_y + ( vel_y * (1 - dampening_factor) )
			
			vel_x = vel_x + vel_x_new
			vel_y = vel_y + vel_y_new
			
			
			
			ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
			ComponentSetValue2( comp, "air_friction", 1 - dampening_factor)
		end)
		
		local entities = EntityGetInRadiusWithTag( cx2, cy2, 10, "vine_projectile" )

		if ( #entities > 1 ) then
			RepelEntities(child, entities[2], 10)
		end
		
		if dist > 300 then
			EntityKill(child)
		end
    end
end