dofile_once( "data/scripts/lib/utilities.lua" )

--Setting The mirror shader to follow the entity
local entity_id = GetUpdatedEntityID()

local x, y = EntityGetTransform( entity_id )

local radius = 0.04

local variable_storage = EntityGetComponent(entity_id, "VariableStorageComponent")

if variable_storage then
	for k, v in pairs(variable_storage)do
		local name = ComponentGetValue2(v, "name")

		if(name == "shader_id")then
			local id = ComponentGetValue2(v, "value_string")   
			GameSetPostFxParameter("ballpos_" .. id, x, y, radius, 1.0)
			break
		end
	end
end


--Repelling all projectiles

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local projectiles = EntityGetWithTag( "projectile" )
local direction_random = math.rad( Random( -30, 30 ) )
if ( #projectiles > 0 ) and Random( 1, 4 ) == 1 then
	for i,projectile_id in ipairs(projectiles) do	
			
		local px, py = EntityGetTransform( projectile_id )
		
		local distance = math.abs( x - px ) + math.abs( y - py )
		local distance_full = 15
		
		if ( distance < distance_full * 1.25 ) then
			distance = get_distance( px, py, x, y )
			direction = get_direction( px, py, x, y )
	
			if ( distance < distance_full ) then
			
				local is_converted = EntityHasTag(projectile_id, "projectile_converted")
				if not is_converted then
					EntityAddTag(projectile_id, "projectile_converted")
					GameEntityPlaySound( projectile_id, "suck_projectile" )
					EntityLoadToEntity("data/entities/animals/boss_centipede/boss_centipede_shield_trail_effect.xml", projectile_id)
				end
				
				local velocitycomponents = EntityGetComponent( projectile_id, "VelocityComponent" )
				
				local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.01)
				local gravity_coeff = 10000000
				
				local herd_id = get_herd_id(entity_id)
				local projectile_comp = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
				component_write( projectile_comp,
				{
					mWhoShot = entity_id,
					mShooterHerdId = herd_id,
					friendly_fire = false,
					collide_with_shooter_frames = -1,
				} )
				
				if ( velocitycomponents ~= nil ) then
					edit_component( projectile_id, "VelocityComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
						
						local offset_x = math.cos( direction + direction_random ) * ( gravity_coeff * gravity_percent )
						local offset_y = 0 - math.sin( direction + direction_random ) * ( gravity_coeff * gravity_percent )

						vel_x = vel_x + offset_x
						vel_y = vel_y + offset_y

						ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
					end)
				end
			end
		end
	end
end

--movement
local player_distance = 1000

local player = get_players()[1]

if(player ~= nil)then
	px, py = EntityGetTransform(player)
	local hit_dir_x = px - x
	local hit_dir_y = py - y
	player_distance = math.sqrt((hit_dir_x*hit_dir_x) + (hit_dir_y*hit_dir_y))
end

local players = EntityGetWithTag("player_unit")

if ( BiomeMapGetName() ~= "$biome_holymountain" and #players > 0 and player_distance < 100)then

	local range = 1000
	local target, targetX, targetY
	local min_dist = 1000
	for _,id in pairs(EntityGetInRadiusWithTag(x, y, range, "player_unit")) do
		-- is target a valid enemy
		if entity_id ~= id  and (EntityGetHerdRelation(entity_id, id) < 40) and EntityGetParent(id) ~= entity_id and (EntityGetComponent(id, "GenomeDataComponent") ~= nil)	then
			local target_x, target_y = EntityGetFirstHitboxCenter( id )
			local dist = get_distance(x, y, target_x, target_y)
			if dist < min_dist then
				min_dist = dist
				target = id
				targetX = target_x
				targetY = target_y
			end
		end
	end
	
	target_id = target

	local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y , targetX, targetY)

	if ( target_id ~= 0) and ( target_id ~= nil) and not did_hit then
		tx, ty = EntityGetTransform( target_id )
		angle = 0 - math.atan2( ty - y, tx - x )
		
		local length = Random( 600, 800 )

		vel_x = math.cos( angle ) * length
		vel_y = 0- math.sin( angle ) * length
		
		local velocity_component = EntityGetComponent(entity_id, "VelocityComponent")
		if(velocity_component ~= nil)then
			local velocity_x, velocity_y = ComponentGetValue2( velocity_component, "mVelocity")

			if(velocity_x ~= nil)then
				PhysicsApplyForce(entity_id, -velocity_x * 0.051, -velocity_y * 0.051)
			end
		end

		PhysicsApplyForce(entity_id, vel_x * 0.051, vel_y * 0.051)
		
	end
	
end