dofile_once( "data/scripts/lib/utilities.lua" )

--Setting The mirror shader to follow the entity
local entity_id = GetUpdatedEntityID()

local x, y = EntityGetTransform( entity_id )

local radius = 0.02

local variable_storage = EntityGetComponent(entity_id, "VariableStorageComponent")

if variable_storage then
	for k, v in pairs(variable_storage)do
		local name = ComponentGetValue2(v, "name")

		if(name == "shader_id")then
			local id = ComponentGetValue2(v, "value_string")   
			GameSetPostFxParameter("ballpos_" .. id, x, y, radius, 1.0)
			break
		end
	end
end

--Area Checker

local parent_id = EntityGetParent( entity_id )

if parent_id > 0 then

	local parent_x, parent_y = EntityGetTransform(parent_id)

	--Area Checker:

	local x, y = EntityGetTransform(entity_id)
	local physics_body_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PhysicsBodyComponent")

	function get_distance( x1, y1, x2, y2 )
		local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
		return result
	end

	function get_direction( x1, y1, x2, y2 )
		return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	end

	local distance = get_distance(x, y, parent_x, parent_y)
	local max_distance = 27
	if distance > max_distance then     
	  local direction = get_direction(parent_x, parent_y, x, y)
	  x = parent_x + math.cos(direction) * max_distance
	  y = parent_y + math.sin(direction) * max_distance
	  -- EntitySetComponentIsEnabled( entity_id, physics_body_component, false )
	end

	EntitySetTransform(entity_id, x, y)
	-- EntitySetComponentIsEnabled( entity_id, physics_body_component, true )
	
end

--Repelling all projectiles

SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local root_id = EntityGetRootEntity(entity_id)

local projectiles = EntityGetWithTag( "projectile" )
local direction_random = math.rad( Random( -30, 30 ) )
if ( #projectiles > 0 ) and Random( 1, 4 ) == 1 then
	for i,projectile_id in ipairs(projectiles) do	
	
		local px, py = EntityGetTransform( projectile_id )
		
		local distance = math.abs( x - px ) + math.abs( y - py )
		local distance_full = 11
		
		if ( distance < distance_full * 1.25 ) then
			distance = get_distance( px, py, x, y )
			direction = get_direction( px, py, x, y )
	
			if ( distance < distance_full ) then
			
				local is_converted = EntityHasTag(projectile_id, "projectile_converted")
				if not is_converted then
					EntityAddTag(projectile_id, "projectile_converted")
					GameEntityPlaySound( projectile_id, "suck_projectile" )
					EntityLoadToEntity("data/entities/animals/boss_centipede/boss_centipede_shield_trail_effect.xml", projectile_id)
				end

				local velocitycomponents = EntityGetComponent( projectile_id, "VelocityComponent" )
				
				local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.01)
				local gravity_coeff = 10000000
				
				local herd_id = get_herd_id(root_id)
				local projectile_comp = EntityGetFirstComponentIncludingDisabled( projectile_id, "ProjectileComponent" )
				component_write( projectile_comp,
				{
					mWhoShot = root_id,
					mShooterHerdId = herd_id,
					friendly_fire = false,
					collide_with_shooter_frames = -1,
				} )
				
				if ( velocitycomponents ~= nil ) then
					edit_component( projectile_id, "VelocityComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
						
						local offset_x = math.cos( direction + direction_random ) * ( gravity_coeff * gravity_percent )
						local offset_y = 0 - math.sin( direction + direction_random ) * ( gravity_coeff * gravity_percent )

						vel_x = vel_x + offset_x
						vel_y = vel_y + offset_y

						ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
					end)
				end
			end
		end
	end
end