dofile_once("data/scripts/lib/utilities.lua")

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function get_direction( x1, y1, x2, y2 )
	return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

local entity_id = GetUpdatedEntityID()
local x, y, rot = EntityGetTransform(entity_id)

--check for enemies
local range = 300
local projectile_velocity = 40
local scatter = 0.2

local enemy, enemy_x, enemy_y
-- locate nearest enemy
for _,id in pairs(EntityGetInRadiusWithTag(x, y, range, "mortal")) do
	-- is target a valid enemy
	if EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(entity_id, id) < 40 then
		local x, y = EntityGetFirstHitboxCenter( id )
		enemy = id
		enemy_x = x
		enemy_y = y
	end
end

local swing_vel = 60

local direction = GetValueNumber("direction", 1)

local timeout = GetValueNumber("attack_timeout", 40)

if timeout <= 20 then
	SetValueNumber("attack_timeout", timeout + 1)
end

local attack_phase = GetValueNumber("attack_phase", 1)

local weak_phase_timeout = GetValueNumber("weak_phase_timeout", 500)

if weak_phase_timeout <= 450 then
	SetValueNumber("weak_phase_timeout", weak_phase_timeout + 1)
end

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 200 == 0 then
	SetValueNumber("direction", direction * -1 )
	
	function split_string(inputstr, sep)
	  sep = sep or "%s"
	  local t= {}
	  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		table.insert(t, str)
	  end
	  return t
	end

	function get_material_damage(entity_id)
	  local damage_model_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
	  if damage_model_component then
		local materials_that_damage = ComponentGetValue2(damage_model_component, "materials_that_damage") --"water,acid,blood,whatever"
		local materials_how_much_damage = ComponentGetValue2(damage_model_component, "materials_how_much_damage")--"1,-1,0.03,0.2"
		materials_that_damage = split_string(materials_that_damage, ",")
		materials_how_much_damage = split_string(materials_how_much_damage, ",")
		local material_damages = {}
		for i, v in ipairs(materials_that_damage) do
		  material_damages[v] = tonumber(materials_how_much_damage[i])
		end
		return material_damages
	  end
	end

	local entities = EntityGetInRadiusWithTag(x, y, 9999999, "mortal")

	for i, entity in ipairs(entities) do

		local material_damages = get_material_damage(entity)

		if material_damages and material_damages.blood_cold_vapour and material_damages.blood_cold_vapour > 0 then

			local entity_damage_model_component = EntityGetFirstComponent(entity, "DamageModelComponent")
			
			if entity_damage_model_component then
			
				EntitySetDamageFromMaterial( entity, "pollen_gas", material_damages.blood_cold_vapour )
				-- EntitySetDamageFromMaterial( entity, "liquid_nitrogen_gas", material_damages.blood_cold_vapour )
				
			end
			
		end

	end
	
end

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 700 == 0 then
	GamePlaySound("data/audio/Desktop/animals.bank", "animals/boss_centipede/open_mouth", x, y)
	SetValueNumber("weak_phase_timeout", 1)
end

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 500 == 0 then
	if ( enemy ~= nil ) and ( enemy > 0 ) then
	
		local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, enemy_x, enemy_y)
		
		if not did_hit then
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/fungus/death_buildup", x, y)
			SetValueNumber("attack_timeout", 0)
		end
	end
end

if timeout == 20 then

	if attack_phase == 1 then
	
		-- if ( enemy ~= nil ) and ( enemy > 0 ) then
		
			-- local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, enemy_x, enemy_y)
			
			-- if not did_hit then
			
				SetRandomSeed( x * y, entity_id )

				local how_many = 16
				local angle_inc = ( 2 * 3.14159 ) / how_many
				local theta = 0

				for i=1,how_many do
					local length = Random( 25, 200 )
					
					local vel_x = math.cos( theta ) * length
					local vel_y = math.sin( theta ) * length
					theta = theta + angle_inc

					shoot_projectile( entity_id, "data/entities/projectiles/seed_shot.xml", x + vel_x * 0.05, y + vel_x * 0.05, vel_x, vel_y )
				end

				EntityLoad( "data/entities/particles/poof_white.xml", x, y )
			-- end
		-- end
		
		SetValueNumber("attack_phase", 2)
		
	elseif attack_phase == 2 then
	
		SetRandomSeed( x * y, entity_id )
		
		-- if ( enemy ~= nil ) and ( enemy > 0 ) then
		
			-- local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, enemy_x, enemy_y)
			
			-- if not did_hit then

				EntityLoad( "data/entities/particles/poof_white.xml", x, y )
				EntityLoad( "data/entities/particles/main_pollen_gas.xml", x, y )
				
			-- end
			
		-- end
		
		SetValueNumber("attack_phase", 3)
		
	elseif attack_phase == 3 then
	
		-- if ( enemy ~= nil ) and ( enemy > 0 ) then
		
			-- local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, enemy_x, enemy_y)
			
			-- if not did_hit then
		
				-- direction
				local vel_x, vel_y = vec_sub(enemy_x, enemy_y, x, y)
				vel_x,vel_y = vec_normalize(vel_x, vel_y)

				-- scatter
				SetRandomSeed(x + GameGetFrameNum(), y)
				vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

				-- apply velocity & shoot
				vel_x,vel_y = vec_mult(vel_x,vel_y, projectile_velocity)
				shoot_projectile( entity_id, "data/entities/projectiles/plant_boss_egg.xml", x, y, vel_x, vel_y)
				
				SetValueNumber("attack_phase", 1)
			-- end
			
		-- end
		
	end
	
end

--Constrain body to a ring around the entity
function constrain(entity, x2, y2, max_dist)
	local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity1, "PhysicsBody2Component") or EntityGetFirstComponent(entity1, "SimplePhysicsComponent")
	local x1, y1 = EntityGetTransform(entity)
	if (not x1) or (not x2) then return nil end
		
	local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
	local target_x = x2 + (((x1-x2)/dist)*max_dist)
	local target_y = y2 + (((y1-y2)/dist)*max_dist)
	local target_vel_x = target_x - x1
	local target_vel_y = target_y - y1

	if dist > max_dist then
		if physcomp or not proj_vel_x then
			PhysicsApplyForce(entity, target_vel_x*100, target_vel_y*100)
		end
	end
end

local var_comp = get_variable_storage_component( entity_id, "initialized" )
local var_comp2 = get_variable_storage_component( entity_id, "origin_x" )
local var_comp3 = get_variable_storage_component( entity_id, "origin_y" )

if ComponentGetValue2(var_comp, "value_bool") == false then
	ComponentSetValue2(var_comp, "value_bool", true)
	ComponentSetValue2(var_comp2, "value_float", x)
	ComponentSetValue2(var_comp3, "value_float", y)
end

local origin_x = ComponentGetValue2(var_comp2, "value_float")
local origin_y = ComponentGetValue2(var_comp3, "value_float")

-- PhysicsApplyTorque(entity_id, -rot * 100)
PhysicsApplyForce(entity_id, swing_vel * direction, 0)

EntitySetTransform(entity_id, x, y, rot + ( x - origin_x ) * 0.005 )

local prev_child = 0

local ivalue = 0

local total_segments = 8

--Getting all my children
for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do
	if EntityGetName(child) == "trunk" then
		ivalue = ivalue + 1
		-- GamePrint(tostring(child) .. " " .. tostring(prev_child))
		if prev_child == 0 then
			EntitySetTransform(child, origin_x, origin_y)
			constrain(entity_id, origin_x, origin_y, 66)
		else
			-- local px, py = EntityGetTransform(prev_child)
			-- constrain(child, px, py, 11)
			
			-- local cx, cy = EntityGetTransform(child)
			-- constrain(prev_child, cx, cy, 11)
			-- local cx, cy = EntityGetTransform(child)
			-- constrain(prev_child, cx, cy, 11)
			local fraction = (ivalue) / (total_segments)
			local target_x = origin_x + (fraction * (x - origin_x))
			local target_y = origin_y + (fraction * (y - origin_y))
			EntitySetTransform(child, target_x, target_y)
		end

		prev_child = child
	end
	
	
	if EntityGetName(child) == "eye_holder" then
		local hx, hy, hrot = EntityGetTransform(child)
		EntitySetTransform(child, hx, hy, hrot + 0.012)
		
		if weak_phase_timeout < 50 then
			offset_x = weak_phase_timeout
		elseif weak_phase_timeout < 400 then
			offset_x = 50
		elseif weak_phase_timeout < 450 then
			offset_x = 450 - weak_phase_timeout
		elseif weak_phase_timeout == 450 then
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/boss_centipede/close_mouth", x, y)
		end
		for i, child2 in ipairs(EntityGetAllChildren(child) or {}) do
			if EntityGetName(child2) == "eye" then
				cx, cy, crot = EntityGetTransform(child2)
				EntitySetTransform(child2, cx, cy, 0)
				if weak_phase_timeout < 450 then

					local itcomp = EntityGetFirstComponent(child2, "InheritTransformComponent")
					local tx, ty, scale_x, scale_y, rot = ComponentGetValue2(itcomp, "Transform")
					ComponentSetValue2(itcomp, "Transform", offset_x, 0, scale_x, scale_y, rot)
				end
				if ( weak_phase_timeout < 400 ) and ( weak_phase_timeout > 50 ) then
					if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 40 == 0 then
						if ( enemy ~= nil ) and ( enemy > 0 ) then
							local projectile_velocity = 80
							local did_hit, hit_x, hit_y = RaytraceSurfaces(cx, cy, enemy_x, enemy_y)
							
							if not did_hit then
								-- direction
								local vel_x, vel_y = vec_sub(enemy_x, enemy_y, cx, cy)
								vel_x,vel_y = vec_normalize(vel_x, vel_y)

								-- scatter
								SetRandomSeed(cx + GameGetFrameNum(), cy)
								vel_x,vel_y = vec_rotate(vel_x,vel_y, rand(-scatter, scatter))

								-- apply velocity & shoot
								vel_x,vel_y = vec_mult(vel_x,vel_y, projectile_velocity)
								shoot_projectile( entity_id, "data/entities/projectiles/orb_plant_boss.xml", cx, cy, vel_x, vel_y)
							end
						end
					end
				end
				
				if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 5 == 0 then
					local comp = EntityGetFirstComponent( child2, "DamageModelComponent" )
					if ( comp ~= nil ) then
						local is_burning = ComponentGetValue2( comp, "mIsOnFire" )
						if is_burning == true then
							EntityInflictDamage( child2, 1, "DAMAGE_FIRE", "", "BLOOD_EXPLOSION", 0, 0, entity_id, 0, 0 )
						end
					end
				end
			end
			
			if EntityGetName(child2) == "neck" then
				--Control Neck limb
				local nx, ny, nrot = EntityGetTransform(child2)
				local angle = get_direction(nx, ny, cx, cy)
				local distance = get_distance(nx, ny, cx, cy)

				local sprite_width = 1
				local scale = math.max(0, distance) / sprite_width

				EntitySetTransform(child2, nx, ny, angle, scale, 1)
			end
		end
	end
	
	local game_effect_component = EntityGetFirstComponentIncludingDisabled(child, "GameEffectComponent")
	
	if game_effect_component and EntityGetName(child) ~= "important" then
		EntityKill(child)
	end
end