dofile_once("data/scripts/lib/utilities.lua")

local range = 200

local entity_id = GetUpdatedEntityID()
local pos_x, pos_y, rot, scale_x = EntityGetTransform( entity_id )

local controls_component = EntityGetFirstComponentIncludingDisabled(entity_id, "ControlsComponent")

local entities = EntityGetInRadiusWithTag(pos_x, pos_y, 9999999, "mortal")

--Go Other direction if touching wall
local did_hit_left = RaytracePlatforms(pos_x, pos_y, pos_x - 4, pos_y - 4)
local did_hit_right = RaytracePlatforms(pos_x, pos_y, pos_x + 4, pos_y - 4)

if did_hit_left and not did_hit_right then
	ComponentSetValue2( controls_component, "mButtonDownLeft", false )
	ComponentSetValue2( controls_component, "mButtonDownRight", true )
elseif did_hit_right and not did_hit_left then
	ComponentSetValue2( controls_component, "mButtonDownLeft", true )
	ComponentSetValue2( controls_component, "mButtonDownRight", false )
end

--Turn All surrounding Ragdolls to entities so I can Kick Them!
for i, entity in ipairs(entities) do
	local AnimalAIComponent = EntityGetFirstComponent(entity, "AnimalAIComponent")
	local PhysicsAIComponent = EntityGetFirstComponent(entity, "PhysicsAIComponent")
	
	if AnimalAIComponent or PhysicsAIComponent then
		
		local lua_comps = EntityGetComponent(entity, "LuaComponent") or {}
		
		local found = false
		
		for i, comp in ipairs(lua_comps) do
			if ComponentGetValue2(comp, "script_damage_received") == "data/scripts/misc/effect_ragdoll_entity.lua" then
				found = true
				break
			end		
		end
		
		if not found then

			EntityAddComponent(entity, "LuaComponent", {
				script_damage_received="data/scripts/misc/effect_ragdoll_entity.lua"
			})
				
		end
		
	end

end

-- Induce Kicking when close to physics bodies or enemies
local entities = EntityGetInRadius(pos_x, pos_y, 40)
for i, entity in ipairs(entities) do

	local physics_body_component = EntityGetFirstComponent(entity, "PhysicsBodyComponent")
	local physics_body2_component = EntityGetFirstComponent(entity, "PhysicsBody2Component")
	local physics_image_shape_component = EntityGetFirstComponent(entity, "PhysicsImageShapeComponent")
	
	if ( physics_body_component or physics_body2_component or physics_image_shape_component or EntityHasTag( entity, "enemy" ) or EntityHasTag( entity, "player_unit" ) ) and EntityGetRootEntity( entity ) ~= entity_id and entity ~= entity_id and not EntityHasTag( entity, "projectile" ) then
		
		-- GamePrint("If is true")
		-- GamePrint(EntityGetName( entity))
		
		local item_pickupper_component = EntityGetFirstComponent(entity, "ItemPickUpperComponent")
		
		if EntityHasTag( entity, "player_unit" ) and ComponentGetValue2( item_pickupper_component, "is_immune_to_kicks") ~= true then
		
			ComponentSetValue2(item_pickupper_component, "is_immune_to_kicks", true)
			
			EntityAddComponent(entity, "LuaComponent", {
				script_source_file="data/scripts/animals/player_kick_immunity_checker.lua",
				execute_every_n_frame=20,
				execute_on_added=0,
			}) 
			
		end
		
		-- local entity_x, entity_y = EntityGetTransform( entity )
		-- local dist = get_distance(pos_x, pos_y, entity_x, entity_y)
		local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
		local animation = ComponentGetValue2( sprite_component, "rect_animation")

		if animation ~= "kick" then
			ComponentSetValue2(controls_component, "mButtonDownKick", true)
			-- ComponentSetValue2(sprite_component, "rect_animation", "kick")
			-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
			ComponentSetValue2(controls_component, "mButtonFrameKick", GameGetFrameNum() + 1)
		else
			ComponentSetValue2(controls_component, "mButtonDownKick", false)
		end
	end
end

-- Give Fall animation when entity is Falling:
local velocity_component = EntityGetFirstComponent(entity_id, "VelocityComponent")
local velocity_x, velocity_y = ComponentGetValueVector2( velocity_component, "mVelocity")

if velocity_y > 0 then
	local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
	local animation = ComponentGetValue2( sprite_component, "rect_animation")
	if animation ~= "jump_fall" then
		local character_data_component = EntityGetFirstComponent(entity_id, "CharacterDataComponent")
		local is_on_ground = ComponentGetValue2( character_data_component, "is_on_ground")
		if not is_on_ground then
			ComponentSetValue2(sprite_component, "rect_animation", "jump_fall")
			local animation2 = ComponentGetValue2( sprite_component, "rect_animation")
		end
	end
end

--Update Hit Box Position
-- for i, child_id in ipairs(EntityGetAllChildren(entity_id) or {}) do
	-- if EntityGetName(child_id) == "Nemesis Child" then
		-- local child_x, child_y = EntityGetTransform( child_id )
		-- EntitySetTransform( entity_id, child_x, child_y )
	-- end
-- end

--Check Nearest Enemy
local enemy, enemy_x, enemy_y
local min_dist = 9999
for _,id in pairs(EntityGetInRadiusWithTag(pos_x, pos_y, range, "mortal")) do
	-- is target a valid enemy
	if entity_id ~= id  and (EntityGetHerdRelation(entity_id, id) < 40) and EntityGetParent(id) ~= entity_id and (EntityGetComponent(id, "GenomeDataComponent") ~= nil)	then
		local x, y = EntityGetFirstHitboxCenter( id )
		local dist = get_distance(pos_x, pos_y, x, y)
		if dist < min_dist then
			min_dist = dist
			enemy = id
			enemy_x = x
			enemy_y = y
		end
	end
end

--Aim Towards Target
if not enemy then return end

local did_hit, hit_x, hit_y = RaytraceSurfaces(pos_x, pos_y, enemy_x, enemy_y)
		
if not did_hit then

	--Induce Averaging of the Aiming position as to induce inaccuracy:
	function approx_rolling_average(avg, target, n)
	  avg = avg - avg / n;
	  avg = avg + target / n;
	  return avg
	end
	
	local smoothness_factor = 10

	local enemy_x_old, enemy_y_old = ComponentGetValueVector2(controls_component, "mMousePosition")
	
	local enemy_x = approx_rolling_average(enemy_x_old, enemy_x, smoothness_factor)
	local enemy_y = approx_rolling_average(enemy_y_old, enemy_y, smoothness_factor)

	ComponentSetValue2( controls_component, "mMousePosition", enemy_x, enemy_y )
	
	local dir_x_old, dir_y_old = ComponentGetValueVector2(controls_component, "mAimingVector")
	
	local dir_x = (enemy_x - pos_x)
	local dir_y = (enemy_y - pos_y)

	local len_sq = dir_x*dir_x + dir_y*dir_y
	
	if len_sq > 0 then
	  local len = math.sqrt(len_sq)
	  dir_x = dir_x / len
	  dir_y = dir_y / len
	  
		local dir_x = approx_rolling_average(dir_x_old, dir_x, smoothness_factor)
	    local dir_y = approx_rolling_average(dir_y_old, dir_y, smoothness_factor)
	  -- dir_x and dir_y is your mAimingVector
	  ComponentSetValue2(controls_component, "mAimingVector", dir_x, dir_y)
	end
	
	--shoot
	local platform_shooter_player_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PlatformShooterPlayerComponent")
	ComponentSetValue2(platform_shooter_player_component, "mForceFireOnNextUpdate", true)
	
	-- ComponentSetValue2(controls_component, "mButtonDownFire", true)
	-- ComponentSetValue2(controls_component, "mButtonFrameFire", GameGetFrameNum() + 1)
	-- ComponentSetValue2(controls_component, "mButtonDownFire2", true)
	-- ComponentSetValue2(controls_component, "mButtonFrameFire2", GameGetFrameNum() + 1)
	-- ComponentSetValue2(controls_component, "mButtonDownLeftClick", true)
	-- ComponentSetValue2(controls_component, "mButtonFrameLeftClick", GameGetFrameNum() + 1)
	
	--Flip Entity to Face Towards Target
	if enemy_x < pos_x and scale_x > 0 then
		EntitySetTransform(entity_id, pos_x, pos_y, rot, -1)
		-- GamePlaySound("data/audio/Desktop/animals.bank", "animals/throw", x, y)
	elseif enemy_x > pos_x and scale_x < 0 then
		EntitySetTransform(entity_id, pos_x, pos_y, rot, 1)
		-- GamePlaySound("data/audio/Desktop/animals.bank", "animals/throw", x, y)
	end
	
	-- local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
	-- local animation = ComponentGetValue2( sprite_component, "rect_animation")
	
	-- if dist < 100 and animation ~= "kick" then
		-- ComponentSetValue2(sprite_component, "rect_animation", "kick")
		-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
	-- end
else
	-- Stop shooting
	local platform_shooter_player_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PlatformShooterPlayerComponent")
	ComponentSetValue2(platform_shooter_player_component, "mForceFireOnNextUpdate", false)
	
	-- ComponentSetValue2(controls_component, "mButtonDownFire", false)
	-- ComponentSetValue2(controls_component, "mButtonDownFire2", false)
	-- ComponentSetValue2(controls_component, "mButtonDownLeftClick", false)
	
end
