dofile_once( "data/scripts/lib/utilities.lua" )
-- dofile( "data/scripts/game_helpers.lua" )

local dash_velocity = 250

local entity_id = GetUpdatedEntityID()
local x, y, rot, scale_x  = EntityGetTransform(entity_id)

local enemy, enemy_x, enemy_y
local min_dist = 9999
for _,id in pairs(EntityGetInRadiusWithTag(x, y, 250, "mortal")) do
	-- is target a valid enemy
	if entity_id ~= id  and (EntityGetHerdRelation(entity_id, id) < 40) and EntityGetParent(id) ~= entity_id and (EntityGetComponent(id, "GenomeDataComponent") ~= nil)	then
		local entity_x, entity_y = EntityGetFirstHitboxCenter( id )
		local dist = get_distance(x, y, entity_x, entity_y)
		if dist < min_dist then
			min_dist = dist
			enemy = id
			enemy_x = entity_x
			enemy_y = entity_y
		end
	end
end

--do nothing if there's no target

if not enemy then return end

--make sure the enemy is seeing the target

local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y - 13 , enemy_x, enemy_y)

if ( not did_hit ) and ( ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 2 == 0 ) then

	--Set velocity direction towards Target 
	local vel_x, vel_y = vec_sub(enemy_x, enemy_y, x, y)
	vel_x,vel_y = vec_normalize(vel_x, vel_y)
	vel_x,vel_y = vec_mult(vel_x,vel_y, dash_velocity)
	
	--Set power of dash depending on distance of target from enemy
	
	local distance = get_distance(x, y, enemy_x, enemy_y)
	local max_power_at_distance = 50
	local scaled_vel_x = math.min(1, distance / max_power_at_distance) * vel_x * 6
	local scaled_vel_y = math.min(1, distance / max_power_at_distance) * vel_y * 6
	
	--getting the direction to go to
	local dir = get_direction(x, y, enemy_x, enemy_y)
	-- local force = 200
	-- local force_x = math.cos(dir) * ( force )
	-- local force_y = math.sin(dir) * ( force )
	
	local cdc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterDataComponent")
	local velocity_x, velocity_y = ComponentGetValue2(cdc_id, "mVelocity")
	
	--Add a dashing trail particle effect
	
	local child_id = EntityCreateNew()
	EntitySetTransform(child_id, x, y)
	EntityAddChild(entity_id, child_id)
	
	local particle_comp = EntityAddComponent2(child_id, "ParticleEmitterComponent", {
		  emitted_material_name="spark_white",
		  x_pos_offset_min=-10,
		  x_pos_offset_max=10,
		  y_pos_offset_min=-10,
		  y_pos_offset_max=10,
		  x_vel_min=-4,
		  x_vel_max=4,
		  y_vel_min=-8,
		  y_vel_max=0,
		  count_min=1,
		  count_max=3,
		  is_trail=true,
		  trail_gap=1,
		  lifetime_min=0.3,
		  lifetime_max=0.3,
		  fade_based_on_lifetime=true,
		  create_real_particles=false,
		  emit_cosmetic_particles=true,
		  collide_with_grid=false,
		  emission_interval_min_frames=0,
		  emission_interval_max_frames=0,
		  airflow_force=0.36,
		  airflow_time=1.20,
		  airflow_scale=0.28,
		  is_emitting=true,
	} )
	ComponentSetValue2(particle_comp, "gravity", 0, 0)
	ComponentSetValue2(particle_comp, "offset", 0, 0)
	
	EntityAddComponent(child_id, "LifetimeComponent", {
		lifetime="30",
	} )

	EntityAddComponent(child_id, "InheritTransformComponent" )
	
	local var_comp = EntityAddComponent(child_id, "VariableStorageComponent", {
		name="direction",
		value_float=dir
	} )
	
	EntityAddComponent(child_id, "LuaComponent", {
		script_source_file="data/scripts/animals/scorpio_move_parent_towards_direction.lua",
		execute_every_n_frame="1"
	} )
	
	-- ComponentSetValueVector2(cdc_id, "mVelocity", velocity_x - force_x, velocity_y + force_y)
	
	GamePlaySound("data/audio/Desktop/animals.bank", "animals/generic/attack_dash", x, y)
	
	local cpc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
	ComponentSetValue2( cpc_id, "pixel_gravity", 0 )
else
	local cpc_id = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
	ComponentSetValue2( cpc_id, "pixel_gravity", 600 )
end
