dofile_once("data/scripts/lib/utilities.lua")

-- Hash function used for Perlin noise
local function hash(n)
  return math.sin(n) * 43758.5453123 % 1
end

-- Linear interpolation
local function lerp(a, b, t)
  return a + t * (b - a)
end

-- Smooth interpolation
local function smoothstep(t)
  return t * t * (3 - 2 * t)
end

-- Perlin noise function
local function perlinNoise(x)
  x = x + GameGetRealWorldTimeSinceStarted()
  local x0 = math.floor(x)
  local x1 = x0 + 1
  local t = x - x0

  local h0 = hash(x0)
  local h1 = hash(x1)

  local smoothT = smoothstep(t)

  -- Interpolate between the hash values
  local interpolated = lerp(h0, h1, smoothT)

  return interpolated
end

-- Function to check if a point is empty
local function isPointEmpty(x, y)
    local did_hit, _, _ = RaytraceSurfaces(x, y, x, y)
    return not did_hit
end

-- Function to find the nearest empty space in a circular pattern
local function findNearestEmptySpace(x, y, radius)
    local step = 0.1 -- Adjust the step size as needed
    local currentAngle = 0

    while currentAngle < 2 * math.pi do
        local offsetX = radius * math.cos(currentAngle)
        local offsetY = radius * math.sin(currentAngle)

        local newX = x + offsetX
        local newY = y + offsetY

        if isPointEmpty(newX, newY) then
            return newX, newY
        end

        currentAngle = currentAngle + step
    end

    -- If no empty space is found, return the original position
    return x, y
end

-- Function to calculate the angle between two points
local function calculateAngle(x1, y1, x2, y2)
    return math.atan2(y2 - y1, x2 - x1)
end

local entity_id = GetUpdatedEntityID()
local x, y, rot, sx = EntityGetTransform(entity_id)
local rx, ry, rangle, rsx = EntityGetTransform(EntityGetRootEntity(entity_id))
local vx, vy = GameGetVelocityCompVelocity(EntityGetRootEntity(entity_id))

local angle = GetValueNumber("angle", rot)

SetRandomSeed(x + entity_id + GameGetFrameNum(), y)

local momentum = 0.9
local range = 10

local swing = 0
local storage = get_variable_storage_component(entity_id, "swing")

local max_angle_1 = math.pi / 4 -- First maximum angle
local max_angle_2 = math.pi / 2 -- Second maximum angle

-- Generate a random value between the two maximum angles
local random_angle = math.random() * (max_angle_2 - max_angle_1) + max_angle_1

-- Calculate the swing based on the random angle
swing = (perlinNoise(GameGetRealWorldTimeSinceStarted() + (GetUpdatedComponentID() * 0.8 )) - angle) * momentum

-- Apply the swing to the angle, considering the range
angle = math.max(math.min(angle + swing, range), -range)

SetValueNumber("angle",angle)

if rsx < 0 then
    angle = (angle * -1) - math.rad(90)
else
	angle = angle - math.rad(90)
end

local eye_x, eye_y = vec_rotate(19, -5, angle)
eye_x = x + eye_x
eye_y = y + eye_y

local did_hit1, hit_x, hit_y = RaytraceSurfaces(eye_x, eye_y + 10, eye_x, eye_y - 10)
local did_hit2, hit_x2, hit_y2 = RaytraceSurfaces(eye_x + 10, eye_y, eye_x - 10, eye_y )
local did_hit3, hit_x3, hit_y3 = RaytraceSurfaces(eye_x + 10, eye_y + 10, eye_x - 10, eye_y - 10 )
local did_hit4, hit_x4, hit_y4 = RaytraceSurfaces(eye_x - 10, eye_y + 10, eye_x + 10, eye_y - 10 )

-- Instead of using RaytraceSurfaces, find the nearest empty space
if did_hit1 or didhit2 or didhit3 or didhit4 then
	local free_x, free_y = findNearestEmptySpace(x, y, 19)
	angle = calculateAngle(x, y, free_x, free_y) - (angle * 0.5)
	-- Update the entity's angle
	if rsx < 0 then
		angle = (math.pi) -angle
	end
end


EntitySetTransform(entity_id, x, y, angle)


-- Save the swing value to the variable storage component
if storage ~= nil then
    ComponentSetValue(storage, "value_int", swing)
end