dofile_once("data/scripts/lib/utilities.lua")

function approx_rolling_average(avg, new_sample, n)
  avg = avg - avg / n;
  avg = avg + new_sample / n;
  return avg
end

function move_angle_towards(current_angle, target_angle, speed)
  -- Make angle go from 0 to 2pi
  if target_angle < 0 then
	target_angle = target_angle + math.pi * 2
  end
  if current_angle < 0 then
	current_angle = current_angle + math.pi * 2
  end

  local dist_up = math.abs(target_angle - current_angle)
  local dist_down = math.abs((current_angle + math.pi * 2) - target_angle)
  if current_angle > target_angle then
	dist_up = math.abs((target_angle + math.pi * 2) - current_angle)
	dist_down = math.abs(current_angle - target_angle)
  end
  
  local dir
  if math.abs(dist_up) < math.abs(dist_down) then
	dir = 1
  else
	dir = -1
  end
  local dist_smallest = dist_up
  if dist_down < dist_up then
	dist_smallest = dist_down
  end

  local new_angle = current_angle + speed * dir
  if dir > 0 and current_angle < target_angle and new_angle > target_angle then
	new_angle = target_angle
  elseif dir < 0 and current_angle > target_angle and new_angle < target_angle  then
	new_angle = target_angle
  end
  new_angle = new_angle % (math.pi * 2)

  return new_angle
end

function get_direction( x1, y1, x2, y2 )
	local result = math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	return result
end

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

local waking_range = 60
local wandering_range = 200
local wandering_from_home_range = 300

local entity_id = GetUpdatedEntityID()
local x, y, rot = EntityGetTransform(entity_id)

local state = GetValueNumber("state", 1)
local timeout = GetValueNumber("timeout", 0)

local sprite_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")

--State1 = sleeping
--State2 = moving towards player
--State4 = moving to home

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") == 1 then
	SetValueNumber("origin_x", x)
	SetValueNumber("origin_y", y)
end

if state == 1 then

	local target = EntityGetClosestWithTag(x, y, "player_unit")
	if (target ~= nil) and (target > 0) then
		if EntityGetFirstComponentIncludingDisabled(target, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(entity_id, target) < 40 then
			local tx, ty = EntityGetTransform(target)
			local dist = get_distance(tx, ty, x, y)

			if dist < waking_range then
				
				ComponentSetValue2(sprite_component, "rect_animation", "waking")
				ComponentSetValue2(sprite_component, "next_rect_animation", "move")
				
				SetValueNumber("state", 2)
				SetValueNumber("timeout", 10)
				
				ComponentSetValue2(GetUpdatedComponentID(), "execute_every_n_frame", 1)

			end
		end
	end
elseif state == 2 then

	if timeout > 0 then
		SetValueNumber("timeout", timeout - 1)
	end
	
	if timeout <= 0 then
		EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", true)
		EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
		ComponentSetValue2(sprite_component, "rect_animation", "move")
		ComponentSetValue2(sprite_component, "next_rect_animation", "move")
		
		if EntityHasTag( entity_id, "homing_target") == false then	
			EntityAddTag( entity_id, "homing_target" )
		end
	end
	
	local target = EntityGetClosestWithTag(x, y, "player_unit")
	if (target ~= nil) and (target > 0) then
		local tx, ty = EntityGetTransform(target)
		local dist = get_distance(tx, ty, x, y)
		local origin_x = GetValueNumber("origin_x", x)
		local origin_y = GetValueNumber("origin_y", y)
		local dist2 = get_distance(x, y, origin_x, origin_y)
		
		if dist2 > wandering_from_home_range or
		
		( ( BiomeMapGetName( x, y ) ~= "Crypt" ) and
		( BiomeMapGetName( x, y ) ~= "Holy Temple" ) and
		( BiomeMapGetName( x, y ) ~= "$biome_tower" ) and 
		( BiomeMapGetName( x, y ) ~= "$biome_wandcave" ) and 
		( BiomeMapGetName( x, y ) ~= "$biome_pyramid" ) and 
		( BiomeMapGetName( x, y ) ~= "$biome_crypt" ))  then
		
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", false)
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
			ComponentSetValue2(sprite_component, "rect_animation", "sleeping")
			ComponentSetValue2(sprite_component, "next_rect_animation", "sleep")
			EntityRemoveTag( entity_id, "homing_target" )
			SetValueNumber("state", 3)
			ComponentSetValue2(GetUpdatedComponentID(), "execute_every_n_frame", 80)
			
		elseif dist < wandering_range then

			-- local old_target_x = GetValueNumber("old_targ_x", target_x)
			-- local old_target_y = GetValueNumber("old_targ_y", target_y)

			-- local new_average_target_x = approx_rolling_average(old_target_x, target_x, 5)
			-- local new_average_target_y = approx_rolling_average(old_target_y, target_y, 5)

			-- local new_angle = get_direction( x, y, new_average_target_x, new_average_target_y )

			-- local new_average_target_player_x = approx_rolling_average(new_average_target_x, tx, 5)
			-- local new_average_target_player_y = approx_rolling_average(new_average_target_y, ty, 5)
			
			-- local new_angle2 = get_direction( x, y, new_average_target_player_x, new_average_target_player_y )
			
			local old_average = GetValueNumber("average_rotation", 0)

			local new_average = move_angle_towards(old_average, rot, 1/60) -- if the number is higher than 5 the turning speed would be smoother and slower, if less it would be more instantaneous
			
			local mandible_x, mandible_y = vec_rotate(0, 1, new_average + math.rad(-90))
			local target_x = x + mandible_x
			local target_y = y + mandible_y
			
			if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 4 == 0 then
				x = target_x
				y = target_y
			end

			SetValueNumber("average_rotation", new_average)

			EntitySetTransform(entity_id, x, y, new_average)
			
			-- SetValueNumber("old_targ_x", target_x)
			-- SetValueNumber("old_targ_y", target_y)
			
			-- GameCreateSpriteForXFrames( "mods/pin_corpse/dot1.png", new_average_target_player_x, new_average_target_player_y, true, 0, 0, 1, true )
			-- GameCreateSpriteForXFrames( "mods/pin_corpse/dot2.png", new_average_target_x, new_average_target_y, true, 0, 0, 1, true )

			-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
			
		else
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
			EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", false)
			ComponentSetValue2(sprite_component, "rect_animation", "sleeping")
			ComponentSetValue2(sprite_component, "next_rect_animation", "sleep")
			SetValueNumber("state", 1)
			EntityRemoveTag( entity_id, "homing_target" )
			ComponentSetValue2(GetUpdatedComponentID(), "execute_every_n_frame", 80)
		end
	else
		EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
		EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_hand", false)
		ComponentSetValue2(sprite_component, "rect_animation", "sleeping")
		ComponentSetValue2(sprite_component, "next_rect_animation", "sleep")
		SetValueNumber("state", 1)
		EntityRemoveTag( entity_id, "homing_target" )
		ComponentSetValue2(GetUpdatedComponentID(), "execute_every_n_frame", 80)
	end
elseif state == 3 then
	ComponentSetValue2(GetUpdatedComponentID(), "execute_every_n_frame", -1)
end