dofile_once("data/scripts/lib/utilities.lua")

--rotating sprite based on velocity
function approx_rolling_average(avg, new_sample, n)
  avg = avg - avg / n;
  avg = avg + new_sample / n;
  return avg
end

function get_direction( x1, y1, x2, y2 )
  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

local entity_id = GetUpdatedEntityID()
local x, y, rot, scale_x, scale_y = EntityGetTransform(entity_id)

SetRandomSeed(GameGetFrameNum(), x + y + entity_id)

--Adding detection for the surfaces
surface_found = false
surface_found_quantity = 0

for _, child in ipairs(EntityGetAllChildren(entity_id) or {} ) do
	if EntityGetName( child ) == "neck" then
		head_id = child
		hx, hy = EntityGetTransform(head_id)
	end
	
	if EntityGetName( child ) == "head" then
		head_id2 = child
		hx2, hy2, hrot, hsx = EntityGetTransform(head_id2)
	end
	
	local surface_found_var_comp = EntityGetFirstComponentIncludingDisabled(child, "VariableStorageComponent")
	if ( surface_found_var_comp ~= nil ) and ( surface_found_var_comp > 0 ) then
		local surface_check_value = ComponentGetValue2(surface_found_var_comp, "value_bool")
		if surface_check_value == true then
		  surface_found_quantity = surface_found_quantity + 1
		end
	end
end

--turning animation
local last_scale_x = GetValueNumber("last_xscale", hsx)
local sprite_comp = EntityGetFirstComponentIncludingDisabled(head_id2, "SpriteComponent")

local is_turning = GetValueBool("turning", false)

if ( last_scale_x ~= hsx ) and ( is_turning == false ) then

	if ( ComponentGetValue2( sprite_comp, "rect_animation" ) ~= "attack" ) then
		ComponentRemoveTag( sprite_comp, "character" )
		ComponentSetValue2( sprite_comp, "rect_animation", "turn" )
		ComponentSetValue2( sprite_comp, "next_rect_animation", "stand" )
		SetValueBool("turning", true)
		SetValueNumber("timeout_frames", 0)
	end
	
end

if is_turning == true then
	SetValueNumber("timeout_frames", GetValueNumber("timeout_frames", 20) + 1)
	if GetValueNumber("timeout_frames", 20) > 20 then
		ComponentAddTag( sprite_comp, "character" )
		SetValueBool("turning", false)
	end
end

local is_turning = GetValueBool("turning", false)

SetValueNumber("last_xscale", hsx)

if surface_found_quantity > 0 then
	surface_found = true
end

--Set Origin Point
local var_comp = EntityGetFirstComponent(entity_id, "VariableStorageComponent", "initialised")
local var_comp2 = EntityGetFirstComponentIncludingDisabled(entity_id, "VariableStorageComponent")

if ComponentGetValue2(var_comp, "value_bool") == false then
	ComponentSetValue2(var_comp, "value_bool", true)

	local origin_x = ComponentGetValue2(var_comp2, "value_int")
	if origin_x == false or origin_x == 0 then
		ComponentSetValue2(var_comp2, "value_int", x)
	end
	
	local origin_y = tonumber(ComponentGetValue2(var_comp2, "value_string"))
	if origin_y == false or origin_y == 0 then
		ComponentSetValue2(var_comp2, "value_string", y)
	end
	
end

--If origin y is too high, just reset it
local origin_y = tonumber(ComponentGetValue2(var_comp2, "value_string"))

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 60 == 0 then

	if y < origin_y - 150 then

		ComponentSetValue2(var_comp2, "value_string", y)
	elseif y > origin_y + 150 then

		ComponentSetValue2(var_comp2, "value_string", y)

	end
	
end

--Move right to left
local move_right = GetValueBool("move_right", true)

if x > ( ComponentGetValue2(var_comp2, "value_int") + 200 ) then
	SetValueBool("move_right", false)
	SetValueNumber("target_y", Random(-80,80))
elseif x < ( ComponentGetValue2(var_comp2, "value_int") - 200 ) then
	SetValueBool("move_right", true)
	SetValueNumber("target_y", Random(-80,80))
end

local limb_boss_component = EntityGetFirstComponentIncludingDisabled(entity_id, "LimbBossComponent")

local old_average_x = GetValueNumber("average_x", x)
local old_average_y = GetValueNumber("average_y", y)

if limb_boss_component ~= nil and limb_boss_component > 0 then
	local root_id = EntityGetRootEntity( entity_id )

	-- locate nearest enemy
	local enemy, enemy_x, enemy_y
	local min_dist = 9999
	for _,id in pairs(EntityGetInRadiusWithTag(x, y, 300, "mortal")) do
		-- is target a valid enemy
		if EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(root_id, id) < 40 then
			local ex, ey = EntityGetFirstHitboxCenter( id )
			local dist = get_distance(hx, hy, ex, ey)
			if dist < min_dist then
				min_dist = dist
				enemy = id
				enemy_x = ex
				enemy_y = ey
			end
		end
	end
	
	if enemy and ( enemy_x < ( ComponentGetValue2(var_comp2, "value_int") + 200 ) ) and ( enemy_x > ( ComponentGetValue2(var_comp2, "value_int") - 200 ) ) and ( enemy_y > origin_y - 150 ) and ( enemy_y < origin_y + 150 ) then
		
		ComponentSetValue2(limb_boss_component, "state", 4)
		ComponentSetValue2(limb_boss_component, "mMoveToPositionX", enemy_x)
		ComponentSetValue2(limb_boss_component, "mMoveToPositionY", enemy_y)
		
		if enemy_x > x then
			SetValueBool("move_right", true)
			EntitySetTransform(entity_id, x, y, rot, 1)
		elseif enemy_x < x then
			SetValueBool("move_right", false)
			EntitySetTransform(entity_id, x, y, rot, -1)
		end
		-- edit_component( head_id, "IKLimbComponent", function(comp,vars)
			-- new_average_x, new_average_y = ComponentGetValueVector2( comp, "end_position")
		-- end)
		
		--Control head and neck
		
		local dist_head = get_distance(hx, hy, enemy_x, enemy_y)
		
		if dist_head < 120 then
			local new_average_x = approx_rolling_average(old_average_x, enemy_x, 100)
			local new_average_y = approx_rolling_average(old_average_y, enemy_y, 100)
			
			SetValueNumber("average_x", new_average_x)
			SetValueNumber("average_y", new_average_y)
			
			local angle2 = get_direction(hx, hy, hx2, hy2)
			local distance2 = get_distance(hx, hy, hx2, hy2)

			local sprite_width2 = 50
			local max_distance2 = 75
			local scale2 = distance2 / sprite_width2
			scale2 = scale2, sprite_width2 / distance2

			EntitySetTransform(head_id, hx, hy, angle2 - math.rad(90), 1, scale2)

			-- if is_turning == true then
				EntitySetTransform(head_id2, new_average_x, new_average_y)
			-- elseif enemy_x > hx2 then
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, 1)
			-- elseif enemy_x < hx2 then
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, -1)
			-- end
			
		elseif enemy_x > hx then
		
			local new_average_x = approx_rolling_average(old_average_x, hx + 30, 50)
			local new_average_y = approx_rolling_average(old_average_y, hy - 38, 50)
			
			SetValueNumber("average_x", new_average_x)
			SetValueNumber("average_y", new_average_y)
			
			local angle2 = get_direction(hx, hy, hx2, hy2)
			local distance2 = get_distance(hx, hy, hx2, hy2)

			local sprite_width2 = 50
			local max_distance2 = 75
			local scale2 = distance2 / sprite_width2
			scale2 = scale2, sprite_width2 / distance2

			EntitySetTransform(head_id, hx, hy, angle2 - math.rad(90), 1, scale2)
			-- if is_turning == true then
				EntitySetTransform(head_id2, new_average_x, new_average_y)
			-- else
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, 1)
			-- end
			
		elseif enemy_x < hx then
		
			local new_average_x = approx_rolling_average(old_average_x, hx - 30, 50)
			local new_average_y = approx_rolling_average(old_average_y, hy - 38, 50)
			
			SetValueNumber("average_x", new_average_x)
			SetValueNumber("average_y", new_average_y)
			
			local angle2 = get_direction(hx, hy, hx2, hy2)
			local distance2 = get_distance(hx, hy, hx2, hy2)

			local sprite_width2 = 50
			local max_distance2 = 75
			local scale2 = distance2 / sprite_width2
			scale2 = scale2, sprite_width2 / distance2

			EntitySetTransform(head_id, hx, hy, angle2 - math.rad(90), -1, scale2)
			
			-- if is_turning == true then
				EntitySetTransform(head_id2, new_average_x, new_average_y)
			-- else
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, -1)
			-- end
			
		end
	
	else
		if move_right == true then

			ComponentSetValue2(limb_boss_component, "state", 4)
			ComponentSetValue2(limb_boss_component, "mMoveToPositionX", ComponentGetValue2(var_comp2, "value_int") + 250)
			ComponentSetValue2(limb_boss_component, "mMoveToPositionY", tonumber(ComponentGetValue2(var_comp2, "value_string")) + GetValueNumber("target_y", 0))
			EntitySetTransform(entity_id, x, y, rot, 1)
			-- edit_component( head_id, "IKLimbComponent", function(comp,vars)
				-- new_average_x, new_average_y = ComponentGetValueVector2( comp, "end_position")
			-- end)
			
			--Control head and neck
			
			local new_average_x = approx_rolling_average(old_average_x, hx + 30, 50)
			local new_average_y = approx_rolling_average(old_average_y, hy - 38, 50)
			
			SetValueNumber("average_x", new_average_x)
			SetValueNumber("average_y", new_average_y)
			
			local angle2 = get_direction(hx, hy, hx2, hy2)
			local distance2 = get_distance(hx, hy, hx2, hy2)

			local sprite_width2 = 50
			local max_distance2 = 75
			local scale2 = distance2 / sprite_width2
			scale2 = scale2, sprite_width2 / distance2

			EntitySetTransform(head_id, hx, hy, angle2 - math.rad(90), 1, scale2)
			
			-- if is_turning == true then
				EntitySetTransform(head_id2, new_average_x, new_average_y)
			-- else
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, 1)
			-- end
			
		elseif move_right == false then
		
			local new_average_x = approx_rolling_average(old_average_x, hx - 30, 50)
			local new_average_y = approx_rolling_average(old_average_y, hy - 38, 50)
			
			SetValueNumber("average_x", new_average_x)
			SetValueNumber("average_y", new_average_y)

			ComponentSetValue2(limb_boss_component, "state", 4)
			ComponentSetValue2(limb_boss_component, "mMoveToPositionX", ComponentGetValue2(var_comp2, "value_int") - 250)
			ComponentSetValue2(limb_boss_component, "mMoveToPositionY", tonumber(ComponentGetValue2(var_comp2, "value_string")) + GetValueNumber("target_y", 0))
			EntitySetTransform(entity_id, x, y, rot, -1)
			-- edit_component( head_id, "IKLimbComponent", function(comp,vars)
				-- new_average_x, new_average_y = ComponentGetValueVector2( comp, "end_position")
			-- end)
			
			--Control head and neck
			
			local angle2 = get_direction(hx, hy, hx2, hy2)
			local distance2 = get_distance(hx, hy, hx2, hy2)

			local sprite_width2 = 50
			local max_distance2 = 75
			local scale2 = distance2 / sprite_width2
			scale2 = scale2, sprite_width2 / distance2

			EntitySetTransform(head_id, hx, hy, angle2 - math.rad(90), -1, scale2)
			
			-- if is_turning == true then
				EntitySetTransform(head_id2, new_average_x, new_average_y)
			-- else
				-- EntitySetTransform(head_id2, new_average_x, new_average_y, 0, -1)
			-- end
		end
	end
end

--expand the lava converter 
local components = EntityGetComponent(entity_id, "MagicConvertMaterialComponent")
if components then
	for k, component in pairs(components)do
		local old_radius = ComponentGetValue2(component, "radius")
		if(old_radius < 45)then
			ComponentSetValue2(component, "radius", old_radius + 1)
			ComponentSetValue2(component, "min_radius", old_radius)
		else
			ComponentSetValue2(component, "radius", 1)
			ComponentSetValue2(component, "min_radius", 1)
		end
	end
end

local components = EntityGetComponent(head_id2, "MagicConvertMaterialComponent")
if components then
	for k, component in pairs(components) do
		local old_radius = ComponentGetValue2(component, "radius")
		if(old_radius < 25)then
			ComponentSetValue2(component, "radius", old_radius + 1)
			ComponentSetValue2(component, "min_radius", old_radius)
		else
			ComponentSetValue2(component, "radius", 1)
			ComponentSetValue2(component, "min_radius", 1)
		end
	end
end


--Toggle movement based on surfaces

if surface_found == true then

	local limb_boss_component = EntityGetFirstComponentIncludingDisabled(entity_id, "LimbBossComponent")
	
	local targetX = ComponentGetValue2(limb_boss_component, "mMoveToPositionX")
	local targetY = ComponentGetValue2(limb_boss_component, "mMoveToPositionY")
	
	local dx = targetX - x
	local dy = targetY - y
	local angle = math.atan2(dy, dx)
	
	local force = 10
	
	local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
	local animation = ComponentGetValue2( sprite_component, "rect_animation")
	if animation ~= "stand" then
		ComponentSetValue2(sprite_component, "rect_animation", "stand")
		ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
	end
	
	PhysicsApplyForce(entity_id, force * math.cos(angle), force * math.sin(angle) )
	
	--Keep the creature elevated. cancel gravity because limbs are grabbing land
	PhysicsApplyForce(entity_id, 0, -19.6 )
	
	-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)

-- else

	-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
	
else

	local sprite_component = EntityGetFirstComponent(entity_id, "SpriteComponent")
	local animation = ComponentGetValue2( sprite_component, "rect_animation")
	if animation ~= "fly" then
		ComponentSetValue2(sprite_component, "rect_animation", "fly")
		ComponentSetValue2(sprite_component, "next_rect_animation", "fly")
	end
	--Make the creature fall, But don't let it fall too quickly, so we just reduce the upward force a little
	PhysicsApplyForce(entity_id, 0, -8.5 )
	
end

--Constrain the head to the body!
function constrain(entity, x2, y2, max_dist)
	local chardatacomp = EntityGetFirstComponent(entity, "CharacterDataComponent")
	local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
	local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity1, "PhysicsBody2Component") or EntityGetFirstComponent(entity1, "SimplePhysicsComponent")
	local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
	local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
	local x1, y1 = EntityGetTransform(entity)
	if (not x1) or (not x2) then return nil end
		
	local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
	local target_x = x2 + (((x1-x2)/dist)*max_dist)
	local target_y = y2 + (((y1-y2)/dist)*max_dist)
	local target_vel_x = target_x - x1
	local target_vel_y = target_y - y1

	if dist > max_dist then
		if physcomp or not proj_vel_x then
			PhysicsApplyForce(entity, target_vel_x*100, target_vel_y*100)
		end
		if vel_x then
			vel_y = vel_y - 60
			local target_mag = math.sqrt((target_vel_x^2) + (target_vel_y^2))
			local mag = math.sqrt((vel_x^2) + (vel_y^2)) / 40
			local force_x = ((target_vel_x/target_mag)*mag)
			local force_y = ((target_vel_y/target_mag)*mag)

			vel_x = ((vel_x + force_x)*1.025) + (target_vel_x*dist*0.8)
			vel_y = ((vel_y + force_y)*1.025) + (target_vel_y*dist*0.8)
			EntitySetTransform(entity, target_x, target_y)
			if ComponentGetValue2(chardatacomp, "is_on_ground") == false then vel_x = vel_x*1.16 end
			ComponentSetValue2(chardatacomp, "mVelocity", vel_x, vel_y+60)
		elseif proj_vel_x then
			ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*100), proj_vel_y+(target_vel_y*100))
			EntitySetTransform(entity, target_x, target_y)
		end
	end
end

constrain(head_id2, hx, hy, 57)