dofile_once( "data/scripts/lib/utilities.lua" )

function collision_trigger(colliding_entity)

	function GetSurfaceNormal(pos_x, pos_y, ray_length, ray_count)
		local total_normal_x = 0
		local total_normal_y = 0
		local total_distance = 0
		local found_normal = false

		for i = 1, ray_count do
			local angle = (i - 1) * (2 * math.pi / ray_count)
			local ray_x = pos_x + ray_length * math.cos(angle)
			local ray_y = pos_y + ray_length * math.sin(angle)

			local did_hit, hit_x, hit_y = RaytracePlatforms(pos_x, pos_y, ray_x, ray_y)

			if did_hit then
				found_normal = true
				local normal_x, normal_y = hit_x - pos_x, hit_y - pos_y
				total_normal_x = total_normal_x + normal_x
				total_normal_y = total_normal_y + normal_y
				total_distance = total_distance + math.sqrt(normal_x^2 + normal_y^2)
			end
		end

		if found_normal then
			local average_normal_x = total_normal_x / ray_count
			local average_normal_y = total_normal_y / ray_count
			local average_distance = total_distance / ray_count
			return true, average_normal_x, average_normal_y, average_distance
		else
			return false, 0, 0, 0
		end
	end

	function approx_rolling_average(avg, new_sample, n)
	  avg = avg - avg / n;
	  avg = avg + new_sample / n;
	  return avg
	end

	local old_average = GetValueNumber("average_rotation", 0)

	function get_direction( x1, y1, x2, y2 )
	  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	end

	local entity_id = GetUpdatedEntityID()
	local x, y, rot = EntityGetTransform(entity_id)

	--Moving our normal detectors based on entity rotation
	local right_corner = { x1 = -8, y1 = 0 }
	local right_x, right_y = vec_rotate(right_corner.x1, right_corner.y1, rot)
	right_x = x + right_x
	right_y = y + right_y

	local left_corner = { x2 = 8, y2 = 0 }
	local left_x, left_y = vec_rotate(left_corner.x2, left_corner.y2, rot)
	left_x = x + left_x
	left_y = y + left_y

	local found_normal, normal_x, normal_y, approximate_distance_from_surface = GetSurfaceNormal(right_x, right_y, 10, 16)
	local found_normal2, normal_x2, normal_y2, approximate_distance_from_surface = GetSurfaceNormal(left_x, left_y, 10, 16)
	local found_normal3, normal_x3, normal_y3, approximate_distance_from_surface = GetSurfaceNormal(x, y, 10, 16)

	local normal_in_solid, hit_x, hit_y = RaytracePlatforms( right_x, right_y, right_x, right_y )
	local normal_in_solid2, hit_x2, hit_y2 = RaytracePlatforms( left_x, left_y, left_x, left_y )
	local normal_in_solid3, hit_x3, hit_y3 = RaytracePlatforms( x, y, x, y )

	normal_count = 3

	if normal_in_solid or not found_normal then
		normal_x = 0
		normal_y = 0
		normal_count = normal_count - 1
	end

	if normal_in_solid2 or not found_normal2 then
		normal_x2 = 0
		normal_y2 = 0
		normal_count = normal_count - 1
	end

	if normal_in_solid3 or not found_normal3 then
		normal_x3 = 0
		normal_y3 = 0
		normal_count = normal_count - 1
	end


	if normal_count > 0 then

		local normal_x_average = (normal_x + normal_x2 + normal_x3) / normal_count
		local normal_y_average = (normal_y + normal_y2 + normal_y3) / normal_count

		local dir = get_direction(0, 0, normal_x_average, normal_y_average)
		local degree_shift = math.rad(-90) + dir -- 90 degree rotation to properly orient the entity
		local new_average = approx_rolling_average(old_average, degree_shift, 5) -- if the number is higher than 5 the turning speed would be smoother and slower, if less it would be more instantaneous
		SetValueNumber("average_rotation", new_average)

		local max_angle = math.rad(45)
		local new_average_capped_angle = math.min(max_angle, math.max(new_average, -max_angle))
		EntitySetTransform(entity_id, x, y, new_average_capped_angle)
	end
	
end