function collision_trigger(colliding_entity)

	function GetSurfaceNormal(pos_x, pos_y, ray_length, ray_count)
		local total_normal_x = 0
		local total_normal_y = 0
		local total_distance = 0
		local found_normal = false

		for i = 1, ray_count do
			local angle = (i - 1) * (2 * math.pi / ray_count)
			local ray_x = pos_x + ray_length * math.cos(angle)
			local ray_y = pos_y + ray_length * math.sin(angle)

			local did_hit, hit_x, hit_y = RaytracePlatforms(pos_x, pos_y, ray_x, ray_y)

			if did_hit then
				found_normal = true
				local normal_x, normal_y = hit_x - pos_x, hit_y - pos_y
				total_normal_x = total_normal_x + normal_x
				total_normal_y = total_normal_y + normal_y
				total_distance = total_distance + math.sqrt(normal_x^2 + normal_y^2)
			end
		end

		if found_normal then
			local average_normal_x = total_normal_x / ray_count
			local average_normal_y = total_normal_y / ray_count
			local average_distance = total_distance / ray_count
			return true, average_normal_x, average_normal_y, average_distance
		else
			return false, 0, 0, 0
		end
	end
	
	function approx_rolling_average(avg, new_sample, n)
	  avg = avg - avg / n;
	  avg = avg + new_sample / n;
	  return avg
	end

	local old_average = GetValueNumber("average_rotation", 0)

	function get_direction( x1, y1, x2, y2 )
	  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	end

	local entity_id = GetUpdatedEntityID()
	local x, y = EntityGetTransform(entity_id)
	local found_normal, normal_x, normal_y, approximate_distance_from_surface = GetSurfaceNormal(x, y, 60, 16)
	local dir = get_direction(0, 0, normal_x, normal_y)
	local degree_shift = math.rad(-90) + dir
	local new_average = approx_rolling_average(old_average, degree_shift, 20)
	SetValueNumber("average_rotation", new_average)


	if found_normal then
	 EntitySetTransform(entity_id, x, y, new_average)
	end

	dofile_once( "data/scripts/game_helpers.lua" )

	local damage_model_component = EntityGetFirstComponent(entity_id, "DamageModelComponent")
	if damage_model_component ~= nil then
		local hp = ComponentGetValue(damage_model_component, "hp")
		local max_hp = ComponentGetValue(damage_model_component, "max_hp")
		local percentage = hp / max_hp * 100

		if percentage < 20 then
			play_animation( entity_id, "hut5")
		elseif percentage < 40 then
			play_animation( entity_id, "hut4")
		elseif percentage < 60 then
			play_animation( entity_id, "hut3")
		elseif percentage < 80 then
			play_animation( entity_id, "hut2")
		end
	end
end