dofile_once( "data/scripts/lib/utilities.lua" )

function collision_trigger(colliding_entity)

	function GetSurfaceNormal(pos_x, pos_y, ray_length, ray_count)
		local total_normal_x = 0
		local total_normal_y = 0
		local total_distance = 0
		local found_normal = false

		for i = 1, ray_count do
			local angle = (i - 1) * (2 * math.pi / ray_count)
			local ray_x = pos_x + ray_length * math.cos(angle)
			local ray_y = pos_y + ray_length * math.sin(angle)

			local did_hit, hit_x, hit_y = RaytracePlatforms(pos_x, pos_y, ray_x, ray_y)

			if did_hit then
				found_normal = true
				local normal_x, normal_y = hit_x - pos_x, hit_y - pos_y
				total_normal_x = total_normal_x + normal_x
				total_normal_y = total_normal_y + normal_y
				total_distance = total_distance + math.sqrt(normal_x^2 + normal_y^2)
			end
		end

		if found_normal then
			local average_normal_x = total_normal_x / ray_count
			local average_normal_y = total_normal_y / ray_count
			local average_distance = total_distance / ray_count
			return true, average_normal_x, average_normal_y, average_distance
		else
			return false, 0, 0, 0
		end
	end

	function approx_rolling_average(avg, new_sample, n)
	  avg = avg - avg / n;
	  avg = avg + new_sample / n;
	  return avg
	end

	local old_average = GetValueNumber("average_rotation", 0)

	function get_direction( x1, y1, x2, y2 )
	  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	end

	local entity_id = GetUpdatedEntityID()
	local x, y, rot, scale_x = EntityGetTransform(entity_id)
	
	SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

	--Moving our normal detectors based on entity rotation
	local right_corner = { x1 = -30, y1 = 0 }
	local right_x, right_y = vec_rotate(right_corner.x1, right_corner.y1, rot)
	right_x = x + right_x
	right_y = y + right_y

	local left_corner = { x2 = 30, y2 = 0 }
	local left_x, left_y = vec_rotate(left_corner.x2, left_corner.y2, rot)
	left_x = x + left_x
	left_y = y + left_y
	
	local right_corner2 = { x1 = -15, y1 = 0 }
	local right_x2, right_y2 = vec_rotate(right_corner2.x1, right_corner2.y1, rot)
	right_x2 = x + right_x2
	right_y2 = y + right_y2

	local left_corner2 = { x2 = 15, y2 = 0 }
	local left_x2, left_y2 = vec_rotate(left_corner2.x2, left_corner2.y2, rot)
	left_x2 = x + left_x2
	left_y2 = y + left_y2
	
	function get_normals(radius)

		local found_normal, normal_x, normal_y, approximate_distance_from_surface = GetSurfaceNormal(right_x, right_y, radius, 16)
		local found_normal2, normal_x2, normal_y2, approximate_distance_from_surface = GetSurfaceNormal(left_x, left_y, radius, 16)
		local found_normal3, normal_x3, normal_y3, approximate_distance_from_surface3 = GetSurfaceNormal(x, y, radius, 16)
		local found_normal4, normal_x4, normal_y4, approximate_distance_from_surface = GetSurfaceNormal(right_x2, right_y2, radius, 16)
		local found_normal5, normal_x5, normal_y5, approximate_distance_from_surface = GetSurfaceNormal(left_x2, left_y2, radius, 16)

		local normal_in_solid, hit_x, hit_y = RaytracePlatforms( right_x, right_y, right_x, right_y )
		local normal_in_solid2, hit_x2, hit_y2 = RaytracePlatforms( left_x, left_y, left_x, left_y )
		local normal_in_solid3, hit_x3, hit_y3 = RaytracePlatforms( x, y, x, y )
		local normal_in_solid4, hit_x4, hit_y4 = RaytracePlatforms( right_x2, right_y2, right_x2, right_y2 )
		local normal_in_solid5, hit_x5, hit_y5 = RaytracePlatforms( left_x2, left_y2, left_x2, left_y2 )

		local normal_count = 5

		if normal_in_solid or not found_normal then
			normal_x = 0
			normal_y = 0
			normal_count = normal_count - 1
		end

		if normal_in_solid2 or not found_normal2 then
			normal_x2 = 0
			normal_y2 = 0
			normal_count = normal_count - 1
		end

		if normal_in_solid3 or not found_normal3 then
			normal_x3 = 0
			normal_y3 = 0
			normal_count = normal_count - 1
		end
		
		if normal_in_solid4 or not found_normal4 then
			normal_x4 = 0
			normal_y4 = 0
			normal_count = normal_count - 1
		end
		
		if normal_in_solid5 or not found_normal5 then
			normal_x5 = 0
			normal_y5 = 0
			normal_count = normal_count - 1
		end
		
		local normal_x_average = (normal_x + normal_x2 + normal_x3 + normal_x4 + normal_x5) / normal_count
		local normal_y_average = (normal_y + normal_y2 + normal_y3 + normal_y4 + normal_y5) / normal_count
		
		return normal_count, normal_x_average, normal_y_average
	end

	local path_finding_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PathFindingComponent")
	local animal_ai_component = EntityGetFirstComponentIncludingDisabled(entity_id, "AnimalAIComponent")
	local character_platforming_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")

	local normal_count, normal_x_average, normal_y_average = get_normals(35)
	local normal_count2, normal_x_average, normal_y_average = get_normals(20)
	
	if normal_count2 > 0 then
	
		local dir = get_direction(0, 0, normal_x_average, normal_y_average)
		local degree_shift = math.rad(-90) + dir -- 90 degree rotation to properly orient the entity
		local new_average = approx_rolling_average(old_average, degree_shift, 40) -- if the number is higher than 5 the turning speed would be smoother and slower, if less it would be more instantaneous
		SetValueNumber("average_rotation", new_average)
		
		EntitySetTransform(entity_id, x, y, new_average)
		
	else
	
		local degree_shift = 0
		local new_average = approx_rolling_average(old_average, degree_shift, 40) -- if the number is higher than 5 the turning speed would be smoother and slower, if less it would be more instantaneous
		SetValueNumber("average_rotation", new_average)

		EntitySetTransform(entity_id, x, y, new_average)
		
	end
	
	-- SetValueNumber("timeout_frames", GetValueNumber("timeout_frames", 60) + 1)
	
	local x, y, rot2 = EntityGetTransform(entity_id)
	
	rotation = -90
	local ray_length = 25
	local end_x = x + math.cos(rot2 - math.rad(rotation)) * ray_length
	local end_y = y + math.sin(rot2 - math.rad(rotation)) * ray_length
	
	local ground_below, ghx, ghy = RaytracePlatforms( x, y, end_x, end_y )
	
	if normal_count > 0 then
	
		-- ComponentSetValue2(path_finding_component, "can_fly", true)
		-- ComponentSetValue2(animal_ai_component, "can_fly", true)

		-- ComponentSetValue2(character_platforming_component, "pixel_gravity", 600)
		-- local character_data_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterDataComponent")
		-- local vel_x, vel_y = ComponentGetValueVector2( character_data_component, "mVelocity" )
		-- -- ComponentSetValue2(character_data_component, "is_on_ground", true)
		-- ComponentSetValueVector2(character_data_component, "mVelocity", vel_x, 0)
	else
		local character_data_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterDataComponent")
		local vel_x, vel_y = ComponentGetValueVector2( character_data_component, "mVelocity" )
		-- -- ComponentSetValue2(character_data_component, "is_on_ground", true)
		if vel_y < 0 then
			vel_y = 0
		end
		ComponentSetValueVector2(character_data_component, "mVelocity", vel_x, 0)
		-- ComponentSetValue2(path_finding_component, "can_fly", false)
		-- ComponentSetValue2(animal_ai_component, "can_fly", false)
		-- ComponentSetValue2(character_platforming_component, "pixel_gravity", 600)
		local controls_component = EntityGetFirstComponentIncludingDisabled(entity_id, "ControlsComponent")
		ComponentSetValue2(controls_component, "mButtonDownFly", false)
	end
	
	
	
	-- if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") % 200 == 0 then
		-- if Random(1,3) == 1 then
			-- EntitySetTransform(entity_id, x, y, rot2 + math.rad(-180))
			-- SetValueNumber("average_rotation", rot2 + math.rad(-180))
			-- SetValueNumber("timeout_frames", 0)

			-- local end_x2 = x + math.cos(rot2 + math.rad(-180) - math.rad(rotation)) * ray_length
			-- local end_y2 = y + math.sin(rot2 + math.rad(-180) - math.rad(rotation)) * ray_length
			
			-- local ground_below2, ghx2, ghy2 = RaytracePlatforms( x, y, end_x2, end_y2 )
			
			-- if ghy2 < y then
				-- SetValueNumber("crawl_on_celing", 1)
			-- else
				-- SetValueNumber("crawl_on_celing", 0)
			-- end
			
			-- local ent = EntityLoad("data/entities/verlet_chains/skullspider/spider_web.xml", x, y)
			-- local joint1 = EntityAddComponent2(ent, "VerletWorldJointComponent", {
				-- verlet_point_index = 1,
			-- } )
			-- ComponentSetValue2(joint1, "world_position", x, y)
			
			-- local joint2 = EntityAddComponent2(ent, "VerletWorldJointComponent", {
				-- verlet_point_index = 16
			-- } )
			-- ComponentSetValue2(joint2, "world_position", ghx2, ghy2)
		-- end
	-- end
		
		
		
		
		
		
		
		
		
	-- local sprite_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
	
	-- local parent_id = EntityGetParent( entity_id )
	-- local crawler_component = EntityGetFirstComponentIncludingDisabled(parent_id, "CrawlerAnimalComponent")
	
	-- if ComponentGetValue2(sprite_component, "rect_animation") == "stand" or ComponentGetValue2(sprite_component, "rect_animation") == "attack" or ComponentGetValue2(sprite_component, "rect_animation") == "attack_ranged" then

		-- ComponentSetValue2(crawler_component, "speed", 0)
		
	-- else
		-- ComponentSetValue2(crawler_component, "speed", 2)
	-- end
	
	
	
	--Turning Animation:
	--This is purely turning animation code
	local last_scale_x = GetValueNumber("last_xscale", scale_x)

	local var_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "VariableStorageComponent")

	if ComponentGetValue2(var_comp, "value_bool") == false then

		if last_scale_x ~= scale_x then
		
			ComponentSetValue2(var_comp, "value_bool", true)
			
			local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
			
			for i, comp in ipairs(sprite_comps) do
			
				ComponentRemoveTag( comp, "character" )
				
				-- if ( ComponentGetValue2(comp, "rect_animation") == "stand" ) then
					ComponentSetValue2( comp, "rect_animation", "turn_right" )
				-- end
				
			end
			
			GamePlaySound("data/audio/Desktop/animals.bank", "animals/giant/attack_shoot", x, y)
			
			SetValueNumber("timeout_frames", 0)
			
		end
		
	end
	
	if ComponentGetValue2(var_comp, "value_bool") == true then
		SetValueNumber("timeout_frames", GetValueNumber("timeout_frames", 20) + 1)
		if ( GetValueNumber("timeout_frames", 20) > 20 ) then
			ComponentSetValue2(var_comp, "value_bool", false)
			
			local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
			for i, comp in ipairs(sprite_comps) do
				ComponentAddTag( comp, "character" )
			end
		elseif ( GetValueNumber("timeout_frames", 20) > 1 ) and ( GetValueNumber("timeout_frames", 20) < 20 ) then
			local x, y, rotation, scale_x = EntityGetTransform(entity_id)
			if last_scale_x ~= scale_x then
				EntitySetTransform(entity_id, x, y, rotation, last_scale_x)
			end
		end
	end

	SetValueNumber("last_xscale", scale_x)
	
end