dofile_once("data/scripts/lib/utilities.lua")

local SPEEDUP_FRAMES = 1800 			--How many frames the speedup will last
local ATTACK_SPEED_DIVISOR = 2.0 		--Attack recharge time of sped up creatures is divided by this
local GHOST_MAX_SPEED = 70 				--Maximum speed a ghost can attain with the speedup

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local enemies = EntityGetInRadius(x, y, 250)

for i, enemy in ipairs(enemies) do
	--Ghosts and enemies are allowed to get the speed buff, and first we check whether we have a ghost
	local genome = EntityGetFirstComponent(enemy, "GenomeDataComponent")
	local is_ghost = false
	if ( genome ~= nil ) and ( genome > 0 ) and ( ComponentGetValue2(genome, "herd_id") == StringToHerdId("ghost") or ComponentGetValue2(genome, "herd_id") == StringToHerdId("slimes") ) then
		is_ghost = true
	end
	if enemy ~= entity_id and (is_ghost == true or EntityHasTag(enemy, "enemy")) and not EntityHasTag(enemy, "player_unit") then --We don't want to speed up the wraith itself
		--Check if speedup is in effect already
		local speedup_storage_component = nil
		local var_store_comps = EntityGetComponent(enemy, "VariableStorageComponent") or {}
		for i, comp in ipairs(var_store_comps) do
			if ComponentGetValue2(comp, "name") == "speedup_storage_component" then	
				speedup_storage_component = comp
				break	
			end
		end
		if speedup_storage_component == nil then				
			--Add a storage component to signify the movement speedup
			EntityAddComponent(enemy, "VariableStorageComponent", {
				name = "speedup_storage_component",
				value_int = SPEEDUP_FRAMES, --Remaining speedup time
			})
			var_store_comps = EntityGetComponent(enemy, "VariableStorageComponent") or {}
			for i, comp in ipairs(var_store_comps) do
				if ComponentGetValue2(comp, "name") == "speedup_storage_component" then	
					speedup_storage_component = comp
					break	
				end
			end
			
			--Make the enemy's attacks recharge faster
			local ai = EntityGetFirstComponent(enemy, "AnimalAIComponent")
			if ( ai ~= nil ) and ( ai > 0 ) then
				local current_attack_melee_frames_between = ComponentGetValue2(ai, "attack_melee_frames_between")
				local current_attack_dash_frames_between = ComponentGetValue2(ai, "attack_dash_frames_between")
				local current_attack_ranged_frames_between = ComponentGetValue2(ai, "attack_ranged_frames_between")
				if current_attack_melee_frames_between ~= nil then
					ComponentSetValue2(ai, "attack_melee_frames_between", math.ceil(current_attack_melee_frames_between / ATTACK_SPEED_DIVISOR))
				end
				if current_attack_dash_frames_between ~= nil then
					ComponentSetValue2(ai, "attack_dash_frames_between", math.ceil(current_attack_dash_frames_between / ATTACK_SPEED_DIVISOR))
				end
				if current_attack_ranged_frames_between ~= nil then
					ComponentSetValue2(ai, "attack_ranged_frames_between", math.ceil(current_attack_ranged_frames_between / ATTACK_SPEED_DIVISOR))
				end
			end
			
			--Speed up fliers in y-direction
			local pathfinding = EntityGetFirstComponent(enemy, "PathFindingComponent")
			if ( pathfinding ~= nil ) and ( pathfinding > 0 ) then
				if ComponentGetValue2(pathfinding, "can_fly") == true then
					local current_distance_to_reach_node_x = ComponentGetValue2(pathfinding, "distance_to_reach_node_x")
					local current_distance_to_reach_node_y = ComponentGetValue2(pathfinding, "distance_to_reach_node_y")
					if current_distance_to_reach_node_x ~= nil then
						ComponentSetValue2(pathfinding, "distance_to_reach_node_x", math.ceil(current_distance_to_reach_node_x * 2))
					end
					if current_distance_to_reach_node_y ~= nil then
						ComponentSetValue2(pathfinding, "distance_to_reach_node_y", math.ceil(current_distance_to_reach_node_y * 2))
					end
				end
			end
			
			--Ghost-specific movement speedup
			-- local ghost_component = EntityGetFirstComponent(enemy, "GhostComponent")
			-- if is_ghost == true and ghost_component ~= 0 then
				-- local current_speed = ComponentGetValue2(ghost_component, "speed")
				-- if current_speed ~= nil then
					-- ComponentSetValue2(ghost_component, "speed", math.min(math.ceil(current_speed * 2), GHOST_MAX_SPEED)) --The doubled speed may not go above GHOST_MAX_SPEED
					-- --Note the original speed of the ghost so it can be returned when the speedup effect is done
					-- EntityAddComponent(enemy, "VariableStorageComponent", {
						-- name = "ghost_original_speed",
						-- value_int = current_speed,
					-- })
				-- end
			-- end

			--Worm-specific movement speedup (nevermind, this was too brutal)
			--[[ 
			local worm_ai = EntityGetFirstComponent(enemy, "WormAIComponent")
			if worm_ai ~= 0 then
				local current_speed = ComponentGetValue2(worm_ai, "speed")
				local current_speed_hunt = ComponentGetValue2(worm_ai, "speed_hunt")
				if current_speed ~= nil then
					ComponentSetValue2(worm_ai, "speed", math.ceil(current_speed * 2))
				end
				if current_speed_hunt ~= nil then
					ComponentSetValue2(worm_ai, "speed_hunt", math.ceil(current_speed_hunt * 2))
				end
				local worm_component = EntityGetFirstComponent(enemy, "WormComponent")
				if worm_component ~= 0 then
					local current_acceleration = ComponentGetValue2(worm_component, "acceleration")
					if current_acceleration ~= nil then
						ComponentSetValue2(worm_component, "acceleration", math.ceil(current_acceleration * 2))
					end
				end
			end ]]--
		
			--Add a script that applies a speedup game effect every 60 frames and removes the speed buff related things when the time is done
			EntityAddComponent(enemy, "LuaComponent", {
				script_source_file="data/scripts/animals/wraith_speed_checker.lua",
				execute_every_n_frame=60,
				execute_on_added=0,
			})
			
			--attack_speed_divisor is transmitted to wraith_speed_checker.lua via a VariableStorageComponent
			EntityAddComponent(enemy, "VariableStorageComponent", {
				name = "attack_speed_divisor",
				value_int = ATTACK_SPEED_DIVISOR,
			})
		end
		--Always set the remaining speedup time to maximum when inside the Speed Wraith aura
		ComponentSetValue2(speedup_storage_component, "value_int", SPEEDUP_FRAMES)
	end
end