dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local speedup_storage_component = nil
local speedup_frames_left = 0
local var_store_comps = EntityGetComponent(entity_id, "VariableStorageComponent") or {}
for i, comp in ipairs(var_store_comps) do
	if ComponentGetValue2(comp, "name") == "speedup_storage_component" then	
		speedup_storage_component = comp
		speedup_frames_left = ComponentGetValue2(comp, "value_int")
		break	
	end		
end

if speedup_frames_left > 0 then
	--Tick down the speedup frame count equal to the interval that this script is loaded, i.e. a second
	ComponentSetValue2(speedup_storage_component, "value_int", speedup_frames_left-60)	
	--Load the speedup game effect which lasts for 60 frames
	LoadGameEffectEntityTo(entity_id, "data/entities/misc/effect_movement_faster_2x_wraith.xml")
else
	local divisor_storage_component = nil
	local attack_speed_divisor = 0
	for i, comp in ipairs(var_store_comps) do
		if ComponentGetValue2(comp, "name") == "attack_speed_divisor" then	
			divisor_storage_component = comp
			attack_speed_divisor = ComponentGetValue2(comp, "value_int")
			break	
		end		
	end
	
	--Undo the attack recharge buffs
	local ai = EntityGetFirstComponent(entity_id, "AnimalAIComponent")
	if ( ai ~= nil ) and ( ai > 0 ) then
		local current_attack_melee_frames_between = ComponentGetValue2(ai, "attack_melee_frames_between")
		local current_attack_dash_frames_between = ComponentGetValue2(ai, "attack_dash_frames_between")
		local current_attack_ranged_frames_between = ComponentGetValue2(ai, "attack_ranged_frames_between")
		if current_attack_melee_frames_between ~= nil then
			ComponentSetValue2(ai, "attack_melee_frames_between", math.ceil(current_attack_melee_frames_between * attack_speed_divisor))
		end
		if current_attack_dash_frames_between ~= nil then
			ComponentSetValue2(ai, "attack_dash_frames_between", math.ceil(current_attack_dash_frames_between * attack_speed_divisor))
		end
		if current_attack_ranged_frames_between ~= nil then
			ComponentSetValue2(ai, "attack_ranged_frames_between", math.ceil(current_attack_ranged_frames_between * attack_speed_divisor))
		end
	end
	
	--Undo flier speedup in y-direction
	local pathfinding = EntityGetFirstComponent(entity_id, "PathFindingComponent")
	if ( pathfinding ~= nil ) and ( pathfinding > 0 ) then
		if ComponentGetValue2(pathfinding, "can_fly") == true then
			local current_distance_to_reach_node_x = ComponentGetValue2(pathfinding, "distance_to_reach_node_x")
			local current_distance_to_reach_node_y = ComponentGetValue2(pathfinding, "distance_to_reach_node_y")
			if current_distance_to_reach_node_x ~= nil then
				ComponentSetValue2(pathfinding, "distance_to_reach_node_x", math.ceil(current_distance_to_reach_node_x / 2))
			end
			if current_distance_to_reach_node_y ~= nil then
				ComponentSetValue2(pathfinding, "distance_to_reach_node_y", math.ceil(current_distance_to_reach_node_y / 2))
			end
		end
	end	
	
	--Undo the ghost-specific movement speedup
	for i, comp in ipairs(var_store_comps) do
		if ComponentGetValue2(comp, "name") == "ghost_original_speed" then
			local ghost_component = EntityGetFirstComponent(entity_id, "GhostComponent")
			if ghost_component ~= 0 then
				ComponentSetValue2(ghost_component, "speed", ComponentGetValue2(comp, "value_int")) --We set the ghost's speed to what it was before the speedup
			end
			EntityRemoveComponent(entity_id, comp)
			break	
		end
	end
	
	--Undo the worm-specific movement speedups (nevermind, this was too brutal)
	--[[
	local worm_ai = EntityGetFirstComponent(entity_id, "WormAIComponent")
	if worm_ai ~= 0 then
		local current_speed = ComponentGetValue2(worm_ai, "speed")
		local current_speed_hunt = ComponentGetValue2(worm_ai, "speed_hunt")
		if current_speed ~= nil then
			ComponentSetValue2(worm_ai, "speed", math.ceil(current_speed / 2))
		end
		if current_speed_hunt ~= nil then
			ComponentSetValue2(worm_ai, "speed_hunt", math.ceil(current_speed_hunt / 2))
		end
		local worm_component = EntityGetFirstComponent(entity_id, "WormComponent")
		if worm_component ~= 0 then
			local current_acceleration = ComponentGetValue2(worm_component, "acceleration")
			if current_acceleration ~= nil then
				ComponentSetValue2(worm_component, "acceleration", math.ceil(current_acceleration / 2))
			end
		end
	end ]]--
	
	--Cleanup
	EntityRemoveComponent(entity_id, speedup_storage_component)
	EntityRemoveComponent(entity_id, divisor_storage_component)
	local this_lua_component = GetUpdatedComponentID()
	EntityRemoveComponent(entity_id, this_lua_component)
	
end