function stringsplit (inputstr, sep)
  sep = sep or "%s"
  local t = {}
  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
    table.insert(t, str)
  end
  return t
end

function give_material_immunity(entity_id, materials)
  if type(materials) == "string" then
    materials = { materials }
  end
  local damage_model_component = EntityGetFirstComponent(entity_id, "DamageModelComponent")
  if damage_model_component ~= nil then
    -- Store all old values
    local old_values = {}
    local old_damage_multipliers = {}
    for k,v in pairs(ComponentGetMembers(damage_model_component)) do
      if k == "ragdoll_fx_forced" then
        v = ComponentGetValue2(damage_model_component, k)
      end
      old_values[k] = v
    end
    for k,_ in pairs(ComponentObjectGetMembers(damage_model_component, "damage_multipliers")) do
      old_damage_multipliers[k] = ComponentObjectGetValue(damage_model_component, "damage_multipliers", k)
    end
    local materials_that_damage = stringsplit(old_values.materials_that_damage, ",")
    local materials_how_much_damage = stringsplit(old_values.materials_how_much_damage, ",")

    local found = false
    for _, material in ipairs(materials) do
      for i, material_that_damages in ipairs(materials_that_damage) do
        if material_that_damages == material then
          table.remove(materials_that_damage, i)
          table.remove(materials_how_much_damage, i)
          found = true
          break
        end
      end
    end
    -- Only remove and rebuild the DamageModelComponent if the material is actually present
    if found then
      -- Rebuild the comma separated string
      old_values.materials_that_damage = ""
      old_values.materials_how_much_damage = ""

      for i, v in ipairs(materials_that_damage) do
        old_values.materials_that_damage = old_values.materials_that_damage .. v
        if i < #materials_that_damage then
          old_values.materials_that_damage = old_values.materials_that_damage .. ","
        end
      end

      for i, v in ipairs(materials_how_much_damage) do
        old_values.materials_how_much_damage = old_values.materials_how_much_damage .. v
        if i < #materials_how_much_damage then
          old_values.materials_how_much_damage = old_values.materials_how_much_damage .. ","
        end
      end

      EntityRemoveComponent(entity_id, damage_model_component)
      damage_model_component = EntityAddComponent(entity_id, "DamageModelComponent", old_values)
      ComponentSetValue2(damage_model_component, "ragdoll_fx_forced", old_values.ragdoll_fx_forced)
      for k, v in pairs(old_damage_multipliers) do
        ComponentObjectSetValue(damage_model_component, "damage_multipliers", k, v)
      end
    end
  end
end
