dofile_once("data/scripts/lib/utilities.lua")
local categories = dofile_once("data/scripts/enemies_gui/enemies.lua")

local function deepcopy(orig)
  local orig_type = type(orig)
  local copy
  if orig_type == 'table' then
      copy = {}
      for orig_key, orig_value in next, orig, nil do
          copy[deepcopy(orig_key)] = deepcopy(orig_value)
      end
      setmetatable(copy, deepcopy(getmetatable(orig)))
  else -- number, string, boolean, etc
      copy = orig
  end
  return copy
end

local sorting_functions = {
  {
    name = "Alphabet ascending",
    func = function(t)
      table.sort(t, function(a, b)
        return a.name < b.name
      end)
      return t
    end,
  },
  {
    name = "Alphabet descending",
    func = function(t)
      table.sort(t, function(a, b)
        return a.name > b.name
      end)
      return t
    end,
  },
  {
    name = "Difficulty easy to hard",
    func = function(t)
      return t
    end,
  },
  {
    name = "Difficulty hard to easy",
    func = function(t)
      local temp = {}
      for i=#t, 1,-1 do
        -- print(i)
        table.insert(temp, t[i])
      end
      return temp
    end,
  },
}

gui = gui or GuiCreate()
open = open or false
page = page or 1
current_id = 1
selected_category_index = selected_category_index or 1
selected_page_number = selected_page_number or 1
sorting_function_index = sorting_function_index or 1
enemies = sorting_functions[sorting_function_index].func(deepcopy(categories[selected_category_index].enemies))
enemies_per_page = 40
enemies_per_row = 8

function new_id()
  current_id = current_id + 1
  return current_id
end

function CustomImageButton(gui, offset_x, offset_y, text, image)
  GuiImage(gui, new_id(), -1, 0, "mods/new_enemies/files/gui/placeholder.png", 1, 1, 1, 0)
  local ret_val = GuiButton(gui, new_id(), offset_x, offset_y, " " .. text .. " ")
  local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.Layout_NoLayouting)
  GuiZSetForNextWidget(gui, 2)
  GuiImageNinePiece(gui, new_id(), x + 2, y, draw_width - 5, draw_height, 1, image)
  return ret_val
end

GuiStartFrame(gui)
GuiOptionsAdd(gui, GUI_OPTION.NoPositionTween)
-- Menu toggle button
GuiLayoutBeginVertical(gui, 60, 0)
if GuiButton(gui, 20, 0, "[NE]", new_id()) then
  open = not open
end
GuiLayoutEnd(gui)

GuiLayoutBeginVertical(gui, 3, 13)

if open then
  local cx, cy = GameGetCameraPos()
  GameCreateSpriteForXFrames("mods/new_enemies/files/gui/background.png", cx, cy, true, 0, 0, 2)
  GuiLayoutBeginHorizontal(gui, 0, 0)
  GuiText(gui, 3, 0, "Toggle Enemies On and Off. You need to restart the game to see effects")
  for i,category in ipairs(categories) do
    if (i-1) % 7 == 0 then
      -- End the current column and start a new one
      GuiLayoutEnd(gui)
      GuiLayoutBeginHorizontal(gui, 0, 1)
    end
    if CustomImageButton(gui, 0, 0, category.category, "mods/new_enemies/files/gui/button_9piece"..((selected_category_index == i) and "_active" or "")..".png") then
      selected_category_index = i
      selected_page_number = 1
      categories = dofile("data/scripts/enemies_gui/enemies.lua")
      enemies = sorting_functions[sorting_function_index].func(deepcopy(categories[selected_category_index].enemies))
    end
  end
  GuiLayoutEnd(gui)

  local horizontal_gap = 0
  local offset = -horizontal_gap
  local page_count = math.ceil(#enemies / enemies_per_page)

  GuiLayoutBeginHorizontal(gui, 0, 1.5)
  if page_count > 1 then
    -- Show page numbers
    for i=1,page_count do
      if CustomImageButton(gui, 0, 0, " " .. i .. " ", "mods/new_enemies/files/gui/button_9piece"..((selected_page_number == i) and "_active" or "")..".png") then
        selected_page_number = i
      end
    end
  end
  GuiLayoutEnd(gui)

  local function get_flag(setting)
    return (setting.default == "disabled" and "nee_" or "ned_") .. setting.key
  end

  local function is_active(setting)
    local active
    local flag = get_flag(setting)
    if setting.default == "disabled" then
      active = HasFlagPersistent(flag)
    else
      active = not HasFlagPersistent(flag)
    end
    return active
  end

  local function toggle_setting(setting, new_state)
    local active = not new_state
    local flag = get_flag(setting)
    if active then
      if setting.default == "disabled" then
        RemoveFlagPersistent(flag)
      else
        AddFlagPersistent(flag)
      end
    else
      if setting.default == "disabled" then
        AddFlagPersistent(flag)
      else
        RemoveFlagPersistent(flag)
      end
    end
  end

  GuiLayoutBeginHorizontal(gui, 0, 2)
  if CustomImageButton(gui, 0, 0, "Enable all", "mods/new_enemies/files/gui/button_9piece_green.png") then
    for i, enemy in ipairs(enemies) do
      toggle_setting(enemy, true)
    end
  end

  if CustomImageButton(gui, 0, 0, "Disable all", "mods/new_enemies/files/gui/button_9piece_red.png") then
    for i, enemy in ipairs(enemies) do
      toggle_setting(enemy, false)
    end
  end

  if CustomImageButton(gui, 0, 0, "Sort by: " .. sorting_functions[sorting_function_index].name, "mods/new_enemies/files/gui/button_9piece_blue.png") then
    sorting_function_index = (sorting_function_index % #sorting_functions) + 1
    enemies = sorting_functions[sorting_function_index].func(deepcopy(categories[selected_category_index].enemies))
  end
  GuiLayoutEnd(gui)

  GuiLayoutBeginVertical(gui, 0, 2)

  local page_start = 1 + (selected_page_number-1) * enemies_per_page
  local page_end = math.min(selected_page_number * enemies_per_page, #enemies)
  local widest_word = 0
  for i=page_start, page_end do
    if i ~= page_start and (((i-1) % enemies_per_page) % enemies_per_row == 0) then
      -- End the current column and start a new one
      GuiLayoutEnd(gui)
      offset = offset + horizontal_gap + widest_word + 40
      widest_word = 0
      GuiLayoutBeginVertical(gui, 0, 2)
    end
    local enemy = enemies[i]
    local checked = is_active(enemy)
    -- Entry begin
    GuiLayoutBeginHorizontal(gui, 0, 0)
    if GuiImageButton(gui, new_id(), offset, 0, "", "mods/new_enemies/files/gui/" .. (checked and "on" or "off") .. ".png") then
      toggle_setting(enemy, not is_active(enemy))
    end
    if enemy.image then
      GuiImage(gui, new_id(), 0, 0, enemy.image, 1, 1, 1, 0)
      GuiZSetForNextWidget(gui, 2)
      GuiImage(gui, new_id(), -18, 0, "data/ui_gfx/animal_icons/extras/back.png", 1, 1, 1, 0)
    end
    if GuiButton(gui, new_id(), 0, 3, " " .. enemy.name) then
      toggle_setting(enemy, not is_active(enemy))
    end
    GuiLayoutEnd(gui)
    -- Entry end
    widest_word = math.max(widest_word, GuiGetTextDimensions(gui, enemy.name))
  end
  GuiLayoutEnd(gui)
end

GuiLayoutEnd(gui)