dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local root_id = EntityGetRootEntity( entity_id )
local x, y, rotation = EntityGetTransform(entity_id)
local pos_x, pos_y = EntityGetTransform(entity_id)

SetRandomSeed(x + GameGetFrameNum(), y)

--half width and height
local w = 30
local h = 15

-- Define the coordinates of the four corners of the rectangle
local right_x, right_y = vec_rotate(120, -h, rotation)
right_x = x + right_x
right_y = y + right_y

local left_x, left_y = vec_rotate(0, -h, rotation)
left_x = x + left_x
left_y = y + left_y

local right_x2, right_y2 = vec_rotate(120, h, rotation)
right_x2 = x + right_x2
right_y2 = y + right_y2

local left_x2, left_y2 = vec_rotate(0, h, rotation)
left_x2 = x + left_x2
left_y2 = y + left_y2

-- GameCreateSpriteForXFrames( "data/debug/circle_16.png", right_x, right_y)
-- GameCreateSpriteForXFrames( "data/debug/circle_16.png", right_x2, right_y2)
-- GameCreateSpriteForXFrames( "data/debug/circle_16.png", left_x, left_y)
-- GameCreateSpriteForXFrames( "data/debug/circle_16.png", left_x2, left_y2)

--Checking if coordinate is inside the rotating rectangle
local function pointInsideRect(x2, y2)
  local nx, ny = vec_rotate(x2 - x, y2 - y, -rotation)
  return (nx + x) >= x and (ny + y) >= y - h and (nx + x) <= x + 120 and (ny + y) <= y + h
end

function rotate_degrees(x, y, degrees)
	return rotate_radians(x, y, math.rad(degrees))
end

function rotate_radians(x, y, radians)
	local ca = math.cos(radians)
	local sa = math.sin(radians)
	local out_x = ca * x - sa * y
	local out_y = sa * x + ca * y
	return out_x, out_y
end

function shoot_at_entity(self_entity, target_entity, general_offset, x_offset, y_offset, speed, projectile, maxdistance, max_player_distance, rotation_offset)
	local rad_offset = 0
	if(rotation_offset ~= nil)then
		rad_offset = rotation_offset
	end
	local x, y = EntityGetTransform(self_entity)
	
	if(target_entity ~= nil)then
		local entityX, entityY = EntityGetTransform( target_entity )

		local headingX = entityX - (pos_x + x_offset)
		local headingY = (entityY-5) - (pos_y + y_offset)
		
		local len = math.sqrt((headingX*headingX) + (headingY*headingY))

		local directionX = headingX / len
		local directionY = headingY / len

		local didHit, hitX, hitY = RaytraceSurfaces( pos_x, pos_y, entityX, entityY )

		local headingHitX = entityX - hitX
		local headingHitY = entityY - hitY

		local hitDistance = math.sqrt((headingHitX*headingHitX) + (headingHitY*headingHitY))

		local pos_x, pos_y, currentRotation = EntityGetTransform( self_entity )	

		if(directionX < 0)then
			EntitySetTransform(self_entity,pos_x,pos_y,currentRotation,-1,1)
		else
			EntitySetTransform(self_entity,pos_x,pos_y,currentRotation,1,1)
		end
	
		if(len < max_player_distance)then
			if(hitDistance < maxdistance)then
			
				directionX, directionY = rotate_degrees(directionX, directionY, rad_offset)
				
				
				local ourProjectile = shoot_projectile( self_entity, projectile, pos_x + (directionX * general_offset) + x_offset, pos_y + (directionY * general_offset) + y_offset, directionX * speed, directionY * speed, false )
				edit_component( ourProjectile, "ProjectileComponent", function(comp,vars)
					vars.mWhoShot       = self_entity
					vars.mShooterHerdId = "player"
				end)
			end
		end
	end
end

for _,id in pairs(EntityGetInRadiusWithTag(x, y, 300, "mortal")) do
	-- is target a valid enemy
	if EntityGetComponent(id, "GenomeDataComponent") ~= nil and EntityGetHerdRelation(root_id, id) < 40 and ( id ~= entity_id ) and ( id ~= root_id ) then
		local ex, ey = EntityGetFirstHitboxCenter( id )
		if pointInsideRect(ex, ey) then
			-- shoot_projectile( entity_id, "data/entities/projectiles/selective_hole_particle.xml", pos_x, pos_y, 0, 0 )
			shoot_at_entity(id, root_id, 1, Random(-8,8), Random(-8,8), 50, "mods/new_enemies/files/selective_hole/selective_hole_particle.xml", 900, 99999999)
				
			-- EntityIngestMaterial( parent_id, CellFactory_GetType( "blood" ), 1 )
			EntityInflictDamage( root_id, -0.0005, "DAMAGE_MATERIAL", "Blood Absorption", "NONE", 0, 0, root_id, ex, ey, 0)
			EntityInflictDamage( id, 0.03, "DAMAGE_BITE", "Blood Absorption", "NONE", 0, 0, root_id, ex, ey, 0)
		end
	end
end