dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local x, y, rot = EntityGetTransform( entity_id )

local projectiles = EntityGetWithTag( "mortal" )

local distance_full = 60

if ( #projectiles > 0 ) then
	for i,projectile_id in ipairs(projectiles) do	
		local px, py = EntityGetTransform( projectile_id )
		
		local distance = math.abs( x - px ) + math.abs( y - py )
		local distance_full = 50
		
		if ( distance < distance_full * 1.25 )  and not EntityHasTag(projectile_id,"monk_hand") then
			distance = math.sqrt( ( x - px ) ^ 2 + ( y - py ) ^ 2 )
			direction = 0 - math.atan2( ( y - py ), ( x - px ) )
	
			if ( distance < distance_full ) then
				local velocitycomponents = EntityGetComponent( projectile_id, "CharacterDataComponent" )
				
				local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.2)
				local gravity_coeff = 40
				
				if ( velocitycomponents ~= nil ) then
					edit_component( projectile_id, "CharacterDataComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
						
						local offset_x = math.cos( direction ) * ( gravity_coeff * gravity_percent )
						local offset_y = 0 - math.sin( direction ) * ( gravity_coeff * gravity_percent )

						vel_x = vel_x + offset_x
						vel_y = vel_y + offset_y

						ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
					end)
				end
			end
		end
	end
end

local projectiles = EntityGetWithTag( "projectile_player" )

if ( #projectiles > 0 ) then
	for i,projectile_id in ipairs(projectiles) do	
		local px, py = EntityGetTransform( projectile_id )
		
		local distance = math.abs( x - px ) + math.abs( y - py )
		local distance_full = 60
		
		if ( distance < distance_full * 1.25 ) then
			distance = math.sqrt( ( x - px ) ^ 2 + ( y - py ) ^ 2 )
			direction = 0 - math.atan2( ( y - py ), ( x - px ) )
	
			if ( distance < distance_full ) then
				local velocitycomponents = EntityGetComponent( projectile_id, "VelocityComponent" )
				
				local gravity_percent = math.max(( distance_full - distance ) / distance_full, 0.2)
				local gravity_coeff = 450
				
				if ( velocitycomponents ~= nil ) then
					edit_component( projectile_id, "VelocityComponent", function(comp,vars)
						local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
						
						local offset_x = math.cos( direction ) * ( gravity_coeff * gravity_percent )
						local offset_y = 0 - math.sin( direction ) * ( gravity_coeff * gravity_percent )

						vel_x = vel_x + offset_x
						vel_y = vel_y + offset_y

						ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
					end)
				end
			end
		end
	end
end


function calculate_force_at(body_x, body_y)
	local distance = math.sqrt( ( x - body_x ) ^ 2 + ( y - body_y ) ^ 2 )
	local direction = 0 - math.atan2( ( y - body_y ), ( x - body_x ) )

	local gravity_percent = ( distance_full - distance ) / distance_full
	local gravity_coeff = 196
	
	local fx = math.cos( direction ) * ( gravity_coeff * gravity_percent )
	local fy = -math.sin( direction ) * ( gravity_coeff * gravity_percent )

    return fx,fy
end


-- force field for physics bodies
function calculate_force_for_body( entity, body_mass, body_x, body_y, body_vel_x, body_vel_y, body_vel_angular )
	local fx, fy = calculate_force_at(body_x, body_y)

	fx = fx * 0.2 * body_mass
	fy = fy * 0.2 * body_mass

    return body_x,body_y,fx,fy,0 -- forcePosX,forcePosY,forceX,forceY,forceAngular
end
local size = distance_full * 0.5
PhysicsApplyForceOnArea( calculate_force_for_body, entity_id, x-size, y-size, x+size, y+size )

