dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local pos_x, pos_y = EntityGetTransform( entity_id )

-- sense to 2 directions and steer towards the one with most space
local sensor_angle = math.pi * 0.4
local sensor_distance = 100
local steer_strength = 0.2

edit_component2( entity_id, "VelocityComponent", function(comp,vars)
	local vel_x,vel_y = ComponentGetValueVector2( comp, "mVelocity")
	
	if get_magnitude(vel_x, vel_y) < 0.01 then
		-- init to random dir if needed
		vel_y = -1
		vel_x, vel_y = vec_rotate(vel_x , vel_y , 0)
	end
	
	-- add sine wobble
	local t = GameGetFrameNum()
	t = t + entity_id * 0.01
	local wobble = math.sin(t * 0.2) * 0.3 -- hi freq
	wobble = wobble + math.sin(t * 0.0394) * 0.1 -- low freq
	vel_x, vel_y = vec_rotate(vel_x, vel_y, wobble)

	local original_vel_x = vel_x
	local original_vel_y = vel_y
	local sensor_x = vel_x
	local sensor_y = vel_y
	local max_distance = 0

	-- sense left and right
	sensor_x, sensor_y = vec_rotate(sensor_x, sensor_y, -sensor_angle)
	for i=0, 2 do
		-- offset ray start positions so it won't collide with root itself
		local did_hit,hit_x,hit_y = RaytraceSurfaces(pos_x + sensor_x * 16, pos_y + sensor_y * 16, pos_x + sensor_x * sensor_distance, pos_y + sensor_y * sensor_distance)
		if did_hit then
			local hit_distance = get_distance2(pos_x, pos_y, hit_x, hit_y)
			if hit_distance >= max_distance then
				-- result found
				max_distance = hit_distance
				vel_x = sensor_x
				vel_y = sensor_y
			end
		else
			-- ray hit nothing making it the longest
			max_distance = sensor_distance
			vel_x = sensor_x
			vel_y = sensor_y
		end

		-- rotate sensor from left to right
		if i==0 then
			sensor_x, sensor_y = vec_rotate(sensor_x, sensor_y, sensor_angle * 2)
		end
	end
	
	function get_direction( x1, y1, x2, y2 )
	  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
	end
	
	local direction = get_direction(0, 0, vel_x, vel_y)
	
	-- blend result with old heading for smooth turns
	-- this also helps hide the fact that we have very limited sensors
	vel_x = lerp(vel_x, original_vel_x, steer_strength)
	vel_y = lerp(vel_y, original_vel_y, steer_strength)

	vel_x,vel_y = vec_normalize(vel_x,vel_y) -- these SHOULD remain normalized but let's make sure we don't mess it up in subtle way

	-- store movement heading and move
	ComponentSetValueVector2( comp, "mVelocity", vel_x, vel_y)
	
	--rotate entity towards direction
	
	EntitySetTransform(entity_id, pos_x + vel_x * 2, pos_y + vel_y * 2, direction) -- coarser movement (multiplier of 2) works well enough
	
	local did_hit,hit_x,hit_y = RaytraceSurfaces(pos_x, pos_y, pos_x, pos_y)
	if did_hit or BiomeMapGetName( pos_x, pos_y ) == "$biome_holymountain"	then
	else
		local entities = EntityGetInRadiusWithTag( pos_x, pos_y, 5, "vine_projectile" )

		if ( #entities > 1 ) then
		else
			local pos_x, pos_y, rot = EntityGetTransform( entity_id )
			local vine_projectile = EntityLoad( "data/entities/projectiles/mother_nature/vine_projectile.xml", pos_x, pos_y )
			EntitySetTransform(vine_projectile, pos_x, pos_y, rot)
			EntityAddChild(entity_id, vine_projectile)
		end
	end
end)



-- spice it up with some extra particles
