dofile_once( "data/scripts/lib/utilities.lua" )

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

function ring_explosion(center_x, center_y, min_radius, num_points, speed, projectile)
    local angle_step = 2 * math.pi / num_points
    
    for i = 1, num_points do
        local angle = i * angle_step
        local x_offset = math.cos(angle) * min_radius
        local y_offset = math.sin(angle) * min_radius
        
        local projectile_x = center_x + x_offset
        local projectile_y = center_y + y_offset
        
        -- Calculate the velocity components based on the angle and speed
        local velocity_x = math.cos(angle) * speed
        local velocity_y = math.sin(angle) * speed
        
        shoot_projectile(nil, projectile, projectile_x, projectile_y, velocity_x, velocity_y, false)
    end
end

local times_executed = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")
local old_range = GetValueNumber("old_range", 0)
local new_range = old_range + 20

if ( times_executed % 2 == 1 ) then

	local child_id = EntityCreateNew()
	EntitySetTransform(child_id, x, y)
	EntityAddChild(entity_id, child_id)
	
	if old_range > 0 then
		local particle_comp = EntityAddComponent2(child_id, "ParticleEmitterComponent", {
			emitted_material_name="fire",
			lifetime_min=0.5,
			lifetime_max=1.5,
			count_min=40,
			count_max=40,
			render_on_grid=true,
			fade_based_on_lifetime=true,
			cosmetic_force_create=false,
			airflow_force=0.3,
			airflow_time=0.01,
			airflow_scale=0.05,
			emission_interval_min_frames=1,
			emission_interval_max_frames=1,
			emit_cosmetic_particles=true,
			is_emitting=true
		} )
		ComponentSetValue2(particle_comp, "gravity", 0, 0)
		ComponentSetValue2(particle_comp, "area_circle_radius", old_range, old_range)
	end
	
	local particle_comp = EntityAddComponent2(child_id, "ParticleEmitterComponent", {
		emitted_material_name="fire",
		lifetime_min=0.5,
		lifetime_max=1.5,
		count_min=10,
		count_max=10,
		render_on_grid=true,
		fade_based_on_lifetime=true,
		cosmetic_force_create=false,
		airflow_force=0.3,
		airflow_time=0.01,
		airflow_scale=0.05,
		emission_interval_min_frames=1,
		emission_interval_max_frames=1,
		emit_cosmetic_particles=true,
		is_emitting=true
	} )
	ComponentSetValue2(particle_comp, "gravity", 0, 0)
	ComponentSetValue2(particle_comp, "area_circle_radius", new_range, old_range)
	
	local particle_comp = EntityAddComponent2(child_id, "ParticleEmitterComponent", {
		emitted_material_name="fire",
		lifetime_min=0.5,
		lifetime_max=1.5,
		count_min=40,
		count_max=40,
		render_on_grid=true,
		fade_based_on_lifetime=true,
		cosmetic_force_create=false,
		airflow_force=0.3,
		airflow_time=0.01,
		airflow_scale=0.05,
		emission_interval_min_frames=1,
		emission_interval_max_frames=1,
		emit_cosmetic_particles=true,
		is_emitting=true
	} )
	ComponentSetValue2(particle_comp, "gravity", 0, 0)
	ComponentSetValue2(particle_comp, "area_circle_radius", new_range, new_range)
	
	EntityAddComponent(child_id, "LifetimeComponent", {
		lifetime="40",
	} )
else
	local entities = EntityGetInRadiusWithTag( x, y, 300, "player_unit" )

	if ( #entities > 0 ) then
		ring_explosion(x, y, old_range - 18, 10 * old_range, 1400, "data/entities/projectiles/ring_explosion_part.xml")
		GamePlaySound("data/audio/Desktop/explosion.bank", "explosions/magic_rocket_big", x, y)
	end
	
	SetValueNumber("old_range", new_range)
	
end

if times_executed == 10 then
	EntityKill(entity_id)
end

ComponentSetValue(GetUpdatedComponentID(), "execute_every_n_frame", 20)