function toggle_entity_visuals(entity, enable_visuals)
	
	local requires_restoration = false
	
	local verlet_component = EntityGetFirstComponentIncludingDisabled(entity, "VerletPhysicsComponent")
	if ( verlet_component ~= nil ) and ( verlet_component > 0 ) then
		if enable_visuals == true then
			
			local itc = EntityGetFirstComponentIncludingDisabled( entity, "InheritTransformComponent" )
			if ( itc ~= nil ) and ( itc > 0 ) then
				EntitySetComponentIsEnabled( entity, itc, true )
			end

			local lua_components = EntityGetComponent(entity, "LuaComponent", "disabled_visual") or {}
			for _, comp in ipairs(lua_components) do
				EntityRemoveComponent( entity, comp )
			end
		else
			
			local itc = EntityGetFirstComponentIncludingDisabled( entity, "InheritTransformComponent" )
			if ( itc ~= nil ) and ( itc > 0 ) then
				EntitySetComponentIsEnabled( entity, itc, false )
			end
			
			local lua_components = EntityGetComponent(entity, "LuaComponent", "disabled_visual") or {}
			
			if #lua_components > 0 then
			else
				local lua_comp = EntityAddComponent2(entity, "LuaComponent", {
					script_source_file="data/scripts/misc/unseen_delay_script.lua",
					execute_every_n_frame=1,
					execute_on_added=false,
					remove_after_executed=false
				})
				ComponentAddTag(lua_comp, "disabled_visual")
			end
		end
	end
	
	local sprite_components = EntityGetComponent(entity, "SpriteComponent") or {}
	
    for _, comp in ipairs(sprite_components) do
        if enable_visuals == true then
            -- If visuals are being enabled, check if the component has the unique tag
            if ComponentHasTag(comp, "disabled_visual") == true then
                ComponentRemoveTag(comp, "disabled_visual")
				ComponentSetValue2(comp, "visible", enable_visuals)
                EntityRefreshSprite(entity, comp)
            end
        else
            -- If visuals are being disabled, disable the component and add the unique tag
			local is_visible = ComponentGetValue2(comp, "visible")
            if ( ComponentHasTag(comp, "disabled_visual") == false ) and ( ComponentHasTag(comp, "aiming_reticle") == false ) and ( is_visible == true ) then
                ComponentAddTag(comp, "disabled_visual")
                ComponentSetValue2(comp, "visible", enable_visuals)
                EntityRefreshSprite(entity, comp)
				requires_restoration = true
            end
        end
    end
	
	local particle_components = EntityGetComponentIncludingDisabled(entity, "ParticleEmitterComponent") or {}
	
    for _, comp in ipairs(particle_components) do
        if enable_visuals == true then
            -- If visuals are being enabled, check if the component has the unique tag
			if ComponentHasTag(comp, "disabled_visual") == true then 
				if ( ComponentHasTag(comp, "jetpack") == true ) or ( ComponentHasTag(comp, "fast_movement_particles") == true ) then
					ComponentRemoveTag(comp, "disabled_visual")
					-- local interval_min = ComponentGetValue2(comp, "emission_interval_min_frames")
					ComponentSetValue2(comp, "delay_frames", 0)
					-- local interval_max = ComponentGetValue2(comp, "emission_interval_max_frames")
					-- ComponentSetValue2(comp, "emission_interval_max_frames", 1)
					if ComponentHasTag(comp, "fast_movement_particles") == true then
						ComponentSetValue2(comp, "is_trail", true)
						-- local interval_min = ComponentGetValue2(comp, "emission_interval_min_frames")
						-- ComponentSetValue2(comp, "emission_interval_min_frames", 1)
						-- local interval_max = ComponentGetValue2(comp, "emission_interval_max_frames")
						-- ComponentSetValue2(comp, "emission_interval_max_frames", 1)
					end
					
					-- GamePrint(tostring(interval_min) .. " out " .. tostring(interval_max))
				elseif ( ComponentGetIsEnabled( comp ) == true ) then
					ComponentRemoveTag(comp, "disabled_visual")
					ComponentSetValue2(comp, "is_emitting", enable_visuals)
				end
			end
			-- local lua_components = EntityGetComponent(entity, "LuaComponent", "disabled_visual") or {}
			-- for _, comp in ipairs(lua_components) do
				-- EntityRemoveComponent( entity, comp )
			-- end
        else
            -- If visuals are being disabled, disable the component and add the unique tag
			local is_emitting = ComponentGetValue2(comp, "is_emitting")
			if not ComponentHasTag(comp, "disabled_visual") then
				if ( ComponentHasTag(comp, "jetpack") == true ) or ( ComponentHasTag(comp, "fast_movement_particles") == true ) then
					ComponentAddTag(comp, "disabled_visual")
					-- local interval_min = ComponentGetValue2(comp, "emission_interval_min_frames")
					ComponentSetValue2(comp, "delay_frames", 9999)
					-- local interval_max = ComponentGetValue2(comp, "emission_interval_max_frames")
					-- ComponentSetValue2(comp, "emission_interval_max_frames", 99999)
					
					-- GamePrint(tostring(interval_min) .. " in " .. tostring(interval_max))
					if ComponentHasTag(comp, "fast_movement_particles") == false then
						ComponentSetValue2(comp, "is_trail", true)
					end
					
					-- local lua_components = EntityGetComponent(entity, "LuaComponent", "disabled_visual") or {}
					
					-- if #lua_components > 0 then
					-- else
						-- local lua_comp = EntityAddComponent2(entity, "LuaComponent", {
							-- script_source_file="data/scripts/misc/unseen_jetpack_script.lua",
							-- execute_every_n_frame=1,
							-- execute_times=300,
							-- execute_on_added=false,
							-- remove_after_executed=false
						-- })
					-- end
				elseif ( is_emitting == true ) and ( ComponentGetIsEnabled( comp ) == true ) then
					ComponentAddTag(comp, "disabled_visual")
					ComponentSetValue2(comp, "is_emitting", enable_visuals)
					requires_restoration = true
				end
			end
        end
    end
	
	local sprite_particle_components = EntityGetComponent(entity, "SpriteParticleEmitterComponent") or {}
	
    for _, comp in ipairs(sprite_particle_components) do
        if enable_visuals == true then
            -- If visuals are being enabled, check if the component has the unique tag
            if ComponentHasTag(comp, "disabled_visual") then
                ComponentRemoveTag(comp, "disabled_visual")
				ComponentSetValue2(comp, "is_emitting", enable_visuals)
            end
        else
			local is_emitting = ComponentGetValue2(comp, "is_emitting")
            -- If visuals are being disabled, disable the component and add the unique tag
            if ( not ComponentHasTag(comp, "disabled_visual") ) and ( is_emitting == true ) then
                ComponentAddTag(comp, "disabled_visual")
                ComponentSetValue2(comp, "is_emitting", enable_visuals)
            end
        end
    end

	if requires_restoration == true then
		if EntityGetName(entity) ~= "DEBUG_NAME:player" and EntityGetName(entity) ~= "arm_r" then
		
			local lua_components = EntityGetComponent(entity, "LuaComponent", "disabled_visual") or {}
			
			if #lua_components > 0 then
			else
				local lua_comp = EntityAddComponent2(entity, "LuaComponent", {
					script_source_file="data/scripts/misc/unseen_restore_script.lua",
					execute_every_n_frame=300,
					execute_on_added=false
				})
				
				ComponentAddTag(lua_comp, "disabled_visual")
				ComponentAddTag(lua_comp, "enabled_in_world")
				ComponentAddTag(lua_comp, "enabled_in_hand")
				ComponentAddTag(lua_comp, "enabled_in_inventory")
			end
		end
	end
end

function toggle_entity_and_children_visuals(entity, enable_visuals)
    -- Toggle visuals for the current entity
    toggle_entity_visuals(entity, enable_visuals)

    -- Get all children of the current entity
    local children = EntityGetAllChildren(entity)
    if children ~= nil then
        -- Recursively toggle visuals for each child
        for _, child_id in ipairs(children) do
            toggle_entity_and_children_visuals(child_id, enable_visuals)
        end
    end
end

function remove_sprite_from_explosion(entity_value, component_name)
	local val_comps = EntityGetComponent(entity_value, component_name) or {}
	
	for i, comp in ipairs(val_comps) do
			
		if ComponentObjectGetValue2( comp, "config_explosion", "explosion_sprite" ) ~= "" then
				
			ComponentObjectSetValue2( comp, "config_explosion", "explosion_sprite", "" )
		end
	end
end