dofile_once("data/scripts/gun/procedural/wands.lua")
dofile_once("data/scripts/lib/utilities.lua")
dofile_once("mods/new_enemies/files/resize_entity_lib/globals.lua")
dofile_once("mods/new_enemies/files/resize_entity_lib/utility.lua")

function revert_scalable_player(player)
    local player_sprites = EntityGetComponentIncludingDisabled(player, "SpriteComponent") or {}
    local player_luas = EntityGetComponentIncludingDisabled(player, "LuaComponent") or {}
    local player_audio = EntityGetComponentIncludingDisabled(player, "AudioComponent") or {}

    --remove the dummy_body sprite
    for _, sprite in ipairs(player_sprites) do
        if ComponentHasTag(sprite, "dummy_body") then
            EntityRemoveComponent(player, sprite)
        end
    end

    --make the regular body sprite visible
    local body_sprite_component = EntityGetFirstComponentIncludingDisabled(player, "SpriteComponent", "lukki_disable")
    ComponentSetValue2(body_sprite_component, "visible", true)

    for _, lua in ipairs(player_luas) do
        --remove the enemy scaling lua script
        if ComponentHasTag(lua, "enemy_scale_setup") then
            EntityRemoveComponent(player, lua)
        end
    end

    --remove grow audio components
    for _, audio in ipairs(player_audio) do
        if ComponentGetValue2(audio, "event_root") == "beeg_smol" then
            EntityRemoveComponent(player, audio)
        end
    end

    --re-create the hand hotspot
    EntityAddComponent2(player, "HotspotComponent", {
        _tags = "hand",
        sprite_hotspot_name="hand"
    })

    --add a new sprite stains comp
    EntityAddComponent2(player, "SpriteStainsComponent", {})

    local children = EntityGetAllChildren(player)
    for _, child in ipairs(children) do
        --kill the verlet chain cape
        if EntityHasTag(child, "mock_verlet") == true then
            EntityKill(child)
        end
		
		if EntityGetName(child) == "cape" then
			EntityKill(child)
		end

        --kill the arm hotspot tracker
        if EntityHasTag(child, "arm_hotspot_tracker") then
            EntityKill(child)
        end

        --kill the dummy arm
        if EntityHasTag(child, "dummy_arm") then
            EntityKill(child)
        end

        if EntityGetName(child) == "arm_r" then
            --add the inherit transform back on to player_arm_r
            EntityAddComponent2(child, "InheritTransformComponent", {
                parent_hotspot_tag = "right_arm_root",
                only_position = true
            })

            --set the arm sprite back to original
            local arm_sprite = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
            ComponentSetValue2(arm_sprite, "image_file", "data/enemies_gfx/player_arm.xml")
        end

        --set player component values back to their original defaults and then delete the scalable_player entity
        if EntityGetName(child) == "scalable_player" then
            local original_values_string = get_variable_component(child, 'beeg_smol.original_values', "value_string")
            local original_values = string_to_table(original_values_string)

            local character_data_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterDataComponent")
            local character_platforming_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterPlatformingComponent")
            local platform_shooter_player_component = EntityGetFirstComponentIncludingDisabled(player, "PlatformShooterPlayerComponent")
            local damage_model_component = EntityGetFirstComponentIncludingDisabled(player, "DamageModelComponent")
            local kick_component = EntityGetFirstComponentIncludingDisabled(player, "KickComponent")
            local hitbox_component = EntityGetFirstComponentIncludingDisabled(player, "HitboxComponent")

            ComponentSetValue2(character_data_component, "buoyancy_check_offset_y", tonumber(original_values.buoyancy_check_offset_y))
            ComponentSetValue2(character_data_component, "climb_over_y", tonumber(original_values.climb_over_y))
            ComponentSetValue2(character_data_component, "check_collision_max_size_x", tonumber(original_values.check_collision_max_size_x))
            ComponentSetValue2(character_data_component, "check_collision_max_size_y", tonumber(original_values.check_collision_max_size_y))
            ComponentSetValue2(character_data_component, "collision_aabb_min_x", tonumber(original_values.collision_aabb_min_x))
            ComponentSetValue2(character_data_component, "collision_aabb_max_x", tonumber(original_values.collision_aabb_max_x))
            ComponentSetValue2(character_data_component, "collision_aabb_min_y", tonumber(original_values.collision_aabb_min_y))
            ComponentSetValue2(character_data_component, "collision_aabb_max_y", tonumber(original_values.collision_aabb_max_y))
            ComponentSetValue2(character_data_component, "eff_hg_size_x", tonumber(original_values.eff_hg_size_x))
            ComponentSetValue2(character_data_component, "eff_hg_size_y", tonumber(original_values.eff_hg_size_y))
            ComponentSetValue2(character_data_component, "eff_hg_damage_min", tonumber(original_values.eff_hg_damage_min))
            ComponentSetValue2(character_data_component, "eff_hg_damage_max", tonumber(original_values.eff_hg_damage_max))

            ComponentSetValue2(character_platforming_component, "fly_speed_mult", tonumber(original_values.fly_speed_mult))
            ComponentSetValue2(character_platforming_component, "velocity_min_x", tonumber(original_values.velocity_min_x))
            ComponentSetValue2(character_platforming_component, "velocity_max_x", tonumber(original_values.velocity_max_x))
            
            ComponentSetValue2(platform_shooter_player_component, "eating_area_min", tonumber(original_values.eating_area_min_x), tonumber(original_values.eating_area_min_y))
            ComponentSetValue2(platform_shooter_player_component, "eating_area_max", tonumber(original_values.eating_area_max_x), tonumber(original_values.eating_area_max_y))
            ComponentSetValue2(platform_shooter_player_component, "eating_cells_per_frame", tonumber(original_values.eating_cells_per_frame))


            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "curse", tonumber(original_values.damage_multiplier_curse))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "drill", tonumber(original_values.damage_multiplier_drill))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "electricity", tonumber(original_values.damage_multiplier_electricity))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "explosion", tonumber(original_values.damage_multiplier_explosion))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "fire", tonumber(original_values.damage_multiplier_fire))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "healing", tonumber(original_values.damage_multiplier_healing))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "ice", tonumber(original_values.damage_multiplier_ice))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "melee", tonumber(original_values.damage_multiplier_melee))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "overeating", tonumber(original_values.damage_multiplier_overeating))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "poison", tonumber(original_values.damage_multiplier_poison))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "projectile", tonumber(original_values.damage_multiplier_projectile))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "radioactive", tonumber(original_values.damage_multiplier_radioactive))
            ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "slice", tonumber(original_values.damage_multiplier_slice))

            ComponentSetValue2(kick_component, "kick_radius", tonumber(original_values.kick_radius))

            ComponentSetValue2(hitbox_component, "aabb_min_x", tonumber(original_values.hitbox_aabb_min_x))
            ComponentSetValue2(hitbox_component, "aabb_max_x", tonumber(original_values.hitbox_aabb_max_x))
            ComponentSetValue2(hitbox_component, "aabb_min_y", tonumber(original_values.hitbox_aabb_min_y))
            ComponentSetValue2(hitbox_component, "aabb_max_y", tonumber(original_values.hitbox_aabb_max_y))

            EntityKill(child)
        end
	end
	
    --add the cloth cape back on to the player
    -- local player_x, player_y = EntityGetTransform(player)
    -- local original_cape = EntityLoad("data/entities/verlet_chains/cape/cape.xml", player_x, player_y)
    -- EntityAddChild(player, original_cape)
	
end

function setup_scalable_player(scalable_player)
    local player = EntityGetWithTag("player_unit")[1]

    --move the player over to the left of spawn just a tad
    --so they can approach the new set piece on their own
    local player_x, player_y, player_rotation, player_scale_x, player_scale_y = EntityGetTransform(player)
    -- EntitySetTransform(player, player_x - 75, player_y, player_rotation, player_scale_x, player_scale_y)

    local original_values_key = 'beeg_smol.original_values'
    local original_values_string = get_variable_component(scalable_player, original_values_key, "value_string")
    if original_values_string ~= nil then return end

    local body_sprite_component = EntityGetFirstComponentIncludingDisabled(player, "SpriteComponent", "lukki_disable")
    ComponentSetValue2(body_sprite_component, "visible", false)

    --add dummy body sprite
    EntityAddComponent2(player, "SpriteComponent", {
        _tags= "dummy_body",
        alpha = 1,
        image_file = "data/enemies_gfx/player.xml",
        next_rect_animation = "",
        offset_x = 6,
        offset_y = 14,
        rect_animation = "stand",
        z_index = 0.6
    })

    --put a script on the entity root to snitch about hits taken and damage mdoel health
    EntityAddComponent2(player, "LuaComponent", {
        script_damage_received = "mods/new_enemies/files/resize_entity_lib/report_damage.lua"
    })

    --kill the default cloth cape
    local children = EntityGetAllChildren(player)
    for _, child in ipairs(children) do
		if EntityHasTag(child, "mock_verlet") == true then
			EntityKill(child)
		end
		if EntityHasTag(child, "cape") == true then
			EntityKill(child)
		end
	end

    local verlet_chain_cape = EntityLoad("mods/new_enemies/files/resize_entity_lib/verlet_chain_cape/verlet_chain_cape.xml", player_x, player_y)
    EntityAddChild(player, verlet_chain_cape)

    --delete the default hand hotspot
    local player_hotspots = EntityGetComponent(player, "HotspotComponent")
    for _,comp in ipairs(player_hotspots) do
        local name = ComponentGetValue2(comp, "sprite_hotspot_name")
        if name == "hand" then
            EntityRemoveComponent(player, comp)
        end
    end

    --turn the regular arm into a single-pixel invisible sprite for easy manipulation
    local regular_arm = EntityGetWithTag("player_arm_r")[1]
    local regular_arm_inherit_transform = EntityGetFirstComponentIncludingDisabled(regular_arm, "InheritTransformComponent")
    EntityRemoveComponent(regular_arm, regular_arm_inherit_transform)
    local regular_arm_sprite = EntityGetFirstComponentIncludingDisabled(regular_arm, "SpriteComponent")
    ComponentSetValue2(regular_arm_sprite, "image_file", "mods/new_enemies/files/resize_entity_lib/1_pixel_arm.xml")

    --arm hotspot tracker
    local arm_hotspot_tracker = EntityCreateNew("arm_hotspot_tracker")
    EntityAddComponent2(arm_hotspot_tracker, "InheritTransformComponent", {
        parent_hotspot_tag = "right_arm_root",
        only_position = false
    })
    EntityAddTag(arm_hotspot_tracker, "arm_hotspot_tracker")
    EntityAddChild(player, arm_hotspot_tracker)

    --dummy arm that will anchor to the arm_hotspot_tracker
    --also has hotspot anchor for a hand entity
    local dummy_arm = EntityCreateNew("dummy_arm")
    EntityAddComponent2(dummy_arm, "SpriteComponent", {
        alpha = 1,
        image_file = "data/enemies_gfx/player_arm.xml",
        next_rect_animation = "",
        rect_animation= "default",
        z_index = 0.59
    })
    local hand_hotspot = EntityAddComponent2(dummy_arm, "HotspotComponent", {
        sprite_hotspot_name = "hand",
        transform_with_scale = true
    })
    ComponentAddTag(hand_hotspot, "dummy_hand")
    EntityAddTag(dummy_arm, "dummy_arm")
    EntityAddChild(player, dummy_arm)

    --hand hotspot tracker (on dummy arm)
    local hand_hotspot_tracker = EntityCreateNew("hand_hotspot_tracker")
    EntityAddComponent2(hand_hotspot_tracker, "InheritTransformComponent", {
        parent_hotspot_tag = "dummy_hand",
        only_position = false
    })
    EntityAddTag(hand_hotspot_tracker, "hand_hotspot_tracker")
    EntityAddChild(dummy_arm, hand_hotspot_tracker)

    local sprite_stains_component = EntityGetFirstComponentIncludingDisabled(player, "SpriteStainsComponent")
    EntityRemoveComponent(player, sprite_stains_component)
    EntityAddComponent2(player, "SpriteStainsComponent", {
        sprite_id = 8
    })

    --character data
    local character_data_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterDataComponent")
    local buoyancy_check_offset_y = ComponentGetValue2(character_data_component, "buoyancy_check_offset_y")
    local climb_over_y = ComponentGetValue2(character_data_component, "climb_over_y")
    local check_collision_max_size_x = ComponentGetValue2(character_data_component, "check_collision_max_size_x")
    local check_collision_max_size_y = ComponentGetValue2(character_data_component, "check_collision_max_size_y")
    local collision_aabb_min_x = ComponentGetValue2(character_data_component, "collision_aabb_min_x")
    local collision_aabb_max_x = ComponentGetValue2(character_data_component, "collision_aabb_max_x")
    local collision_aabb_min_y = ComponentGetValue2(character_data_component, "collision_aabb_min_y")
    local collision_aabb_max_y = ComponentGetValue2(character_data_component, "collision_aabb_max_y")
    local eff_hg_size_x = ComponentGetValue2(character_data_component, "eff_hg_size_x")
    local eff_hg_size_y = ComponentGetValue2(character_data_component, "eff_hg_size_y")
    local eff_hg_damage_min = ComponentGetValue2(character_data_component, "eff_hg_damage_min")
    local eff_hg_damage_max = ComponentGetValue2(character_data_component, "eff_hg_damage_max")

    --character platforming
    local character_playforming_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterPlatformingComponent")
    local fly_speed_mult = ComponentGetValue2(character_playforming_component, "fly_speed_mult")
    local velocity_min_x = ComponentGetValue2(character_playforming_component, "velocity_min_x")
    local velocity_max_x = ComponentGetValue2(character_playforming_component, "velocity_max_x")

    --platform shooter player
    local platform_shooter_player_component = EntityGetFirstComponentIncludingDisabled(player, "PlatformShooterPlayerComponent")
    local eating_area_min_x, eating_area_min_y = ComponentGetValue2(platform_shooter_player_component, "eating_area_min")
    local eating_area_max_x, eating_area_max_y = ComponentGetValue2(platform_shooter_player_component, "eating_area_max")
    local eating_cells_per_frame =  ComponentGetValue2(platform_shooter_player_component, "eating_cells_per_frame")

    --damage model
    local damage_model_component = EntityGetFirstComponentIncludingDisabled(player, "DamageModelComponent")
    local current_hp = ComponentGetValue2(damage_model_component, "hp")
    local current_max_hp = ComponentGetValue2(damage_model_component, "max_hp")
    --local damage_multipliers = ComponentObjectGetValue2(damage_model_component, "damage_multipliers")
    local damage_multiplier_curse = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "curse")
    local damage_multiplier_drill = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "drill")
    local damage_multiplier_electricity = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "electricity")
    local damage_multiplier_explosion = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "explosion")
    local damage_multiplier_fire = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "fire")
    local damage_multiplier_healing = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "healing")
    local damage_multiplier_ice = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "ice")
    local damage_multiplier_melee = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "melee")
    local damage_multiplier_overeating = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "overeating")
    --local damage_multiplier_physics_hit = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "physics_hit")
    local damage_multiplier_poison = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "poison")
    local damage_multiplier_projectile = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "projectile")
    local damage_multiplier_radioactive = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "radioactive")
    local damage_multiplier_slice = ComponentObjectGetValue2(damage_model_component, "damage_multipliers", "slice")

    --kick
    local kick_component = EntityGetFirstComponentIncludingDisabled(player, "KickComponent")
    local kick_radius = ComponentGetValue2(kick_component, "kick_radius")

    --hitbox
    local hitbox_component = EntityGetFirstComponentIncludingDisabled(player, "HitboxComponent")
    local hitbox_aabb_min_x = ComponentGetValue2(hitbox_component ,"aabb_min_x")
    local hitbox_aabb_max_x = ComponentGetValue2(hitbox_component ,"aabb_max_x")
    local hitbox_aabb_min_y = ComponentGetValue2(hitbox_component ,"aabb_min_y")
    local hitbox_aabb_max_y = ComponentGetValue2(hitbox_component ,"aabb_max_y")

    --script that regularly checks for nearby enemies that haven't been set up for scaling, and sets them up
    local enemy_scale_setup = EntityGetFirstComponentIncludingDisabled(player, "LuaComponent", "enemy_scale_setup")
    if enemy_scale_setup == nil then
        -- EntityAddComponent2(player, "LuaComponent", {
            -- script_source_file = "mods/new_enemies/files/resize_entity_lib/scan_for_uninitialized.lua",
            -- execute_every_n_frame = 120
        -- })
    end

    local original_values = {
        buoyancy_check_offset_y = buoyancy_check_offset_y,
        climb_over_y = climb_over_y,
        check_collision_max_size_x = check_collision_max_size_x,
        check_collision_max_size_y = check_collision_max_size_y,
        collision_aabb_min_x = collision_aabb_min_x,
        collision_aabb_max_x = collision_aabb_max_x,
        collision_aabb_min_y = collision_aabb_min_y,
        collision_aabb_max_y = collision_aabb_max_y,
        hitbox_aabb_min_x = hitbox_aabb_min_x,
        hitbox_aabb_max_x = hitbox_aabb_max_x,
        hitbox_aabb_min_y = hitbox_aabb_min_y,
        hitbox_aabb_max_y = hitbox_aabb_max_y,
        eff_hg_size_x = eff_hg_size_x,
        eff_hg_size_y = eff_hg_size_y,
        eff_hg_damage_min = eff_hg_damage_min,
        eff_hg_damage_max = eff_hg_damage_max,
        destroy_ground = 1,
        fly_speed_mult = fly_speed_mult,
        velocity_min_x = velocity_min_x,
        velocity_max_x = velocity_max_x,
        eating_area_min_x = eating_area_min_x,
        eating_area_max_x = eating_area_max_x,
        eating_area_min_y = eating_area_min_y,
        eating_area_max_y = eating_area_max_y,
        eating_cells_per_frame = eating_cells_per_frame,
        kick_radius = kick_radius,
        damage_multiplier_curse = damage_multiplier_curse,
        damage_multiplier_drill = damage_multiplier_drill,
        damage_multiplier_electricity = damage_multiplier_electricity,
        damage_multiplier_explosion = damage_multiplier_explosion,
        damage_multiplier_fire = damage_multiplier_fire,
        damage_multiplier_healing = damage_multiplier_healing,
        damage_multiplier_ice = damage_multiplier_ice,
        damage_multiplier_melee = damage_multiplier_melee,
        damage_multiplier_overeating = damage_multiplier_overeating,
        damage_multiplier_physics_hit = damage_multiplier_physics_hit,
        damage_multiplier_poison = damage_multiplier_poison,
        damage_multiplier_projectile = damage_multiplier_projectile,
        damage_multiplier_radioactive = damage_multiplier_radioactive,
        damage_multiplier_slice = damage_multiplier_slice
    }
    original_values_string = table_to_string(original_values)
    set_variable_component(scalable_player, original_values_key, "value_string", original_values_string)

    set_variable_component(scalable_player, "beeg_smol.last_hp", "value_float", current_hp)
    set_variable_component(scalable_player, "beeg_smol.last_max_hp", "value_float", current_max_hp)

    local current_kills = tonumber(StatsGetValue("enemies_killed"))
    set_variable_component(scalable_player, "beeg_smol.last_kills", "value_int", tonumber(current_kills))

    set_variable_component(scalable_player, "beeg_smol.scale", "value_float", initial_scale)
    set_variable_component(scalable_player, "beeg_smol.last_reported_scale", "value_float", initial_scale)
    set_variable_component(scalable_player, "beeg_smol.last_reported_scale_frame", "value_int", 1)
    set_variable_component(scalable_player, "beeg_smol.scale_throttled", "value_float", initial_scale)
    set_variable_component(scalable_player, "beeg_smol.last_sfx_frame", "value_int", -9999)
    set_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string", "neither")
    
    --EntityAddTag(player, "scalable_initialized")
end

function setup_mock_verlet(original_verlet)
    local scalable_verlet_entity = EntityCreateNew("scalable_verlet_" .. tostring(original_verlet))
    EntityAddTag(scalable_verlet_entity, "mock_verlet")
    local original_verlet_component = EntityGetFirstComponentIncludingDisabled(original_verlet, "VerletPhysicsComponent")
    local original_sprite_components = EntityGetComponentIncludingDisabled(original_verlet, "SpriteComponent")

    --put the original verlet's id into a variable component for verification later
    set_variable_component(scalable_verlet_entity, "beeg_smol.original_verlet", "value_int", original_verlet)

    for i = 1, ComponentGetValue2(original_verlet_component, "num_links") do
        local new_scalable_verlet_segment = EntityCreateNew("sprite_segment_" .. tostring(original_sprite_components[i]))
        EntityAddComponent2(new_scalable_verlet_segment, "SpriteComponent", {
            image_file = ComponentGetValue2(original_sprite_components[i], "image_file"),
            offset_x = ComponentGetValue2(original_sprite_components[i], "offset_x"),
            offset_y = ComponentGetValue2(original_sprite_components[i], "offset_y"),
            rect_animation = ComponentGetValue2(original_sprite_components[i], "rect_animation"),
            z_index = ComponentGetValue2(original_sprite_components[i], "z_index"),
            alpha = ComponentGetValue2(original_sprite_components[i], "alpha"),
            visible = true, --ComponentGetValue2(original_sprite_components[i], "visible"),
            update_transform = true,
            update_transform_rotation = true
        })
        --add a stains component cause why the heck not
        EntityAddComponent2(new_scalable_verlet_segment, "SpriteStainsComponent", {})

        EntityAddChild(scalable_verlet_entity, new_scalable_verlet_segment)
        ComponentSetValue2(original_sprite_components[i], "visible", false)
    end
    local original_inherit_transform_component = EntityGetFirstComponentIncludingDisabled(original_verlet, "InheritTransformComponent")
    if original_inherit_transform_component ~= nil then
        local t1, t2, t3, t4, t5 = ComponentGetValue2(original_inherit_transform_component, "Transform")
        local inherit_transform = EntityAddComponent2(scalable_verlet_entity, "InheritTransformComponent", {
            use_root_parent = ComponentGetValue2(original_inherit_transform_component, "use_root_parent"),
            only_position = ComponentGetValue2(original_inherit_transform_component, "only_position"),
            parent_hotspot_tag = ComponentGetValue2(original_inherit_transform_component, "parent_hotspot_tag"),
            parent_sprite_id = ComponentGetValue2(original_inherit_transform_component, "parent_sprite_id"),
            always_use_immediate_parent_rotation = ComponentGetValue2(original_inherit_transform_component, "always_use_immediate_parent_rotation"),
            rotate_based_on_x_scale = ComponentGetValue2(original_inherit_transform_component, "rotate_based_on_x_scale"),
        })
        ComponentSetValue2(inherit_transform, "Transform", t1, t2, t3, t4, t5)
    end

    --local player = EntityGetWithTag("player_unit")[1]
    --EntityAddChild(player, scalable_verlet_entity)
    local parent = EntityGetParent(original_verlet)
    EntityAddChild(parent, scalable_verlet_entity)

    return scalable_verlet_entity
end

function update_mock_verlet(original_verlet, scale, sign_x)
    if original_verlet == 0 or original_verlet == nil then return nil end
    if scale == nil then return nil end
    if sign_x == nil then sign_x = 1 end

    --sorry, can't do cloth type verlets
    local original_verlet_component = EntityGetFirstComponentIncludingDisabled(original_verlet, "VerletPhysicsComponent")
    local type = ComponentGetValue2(original_verlet_component, "type")
    if type == "CLOTH" then return end

    local mock_verlet = EntityGetWithName("scalable_verlet_" .. tostring(original_verlet))
    if mock_verlet == 0 or mock_verlet == nil then
        --before setting up a new mock verlet, kill any mock verlets that don't correspond to an original verlet
        --this happens after a polymorph, for some reason it deletes the base verlet but not the mock verlet
        local player = EntityGetRootEntity(original_verlet)
        local children = EntityGetAllChildren(player)
        for _, child in ipairs(children) do
            if EntityHasTag(child, "mock_verlet") then
                local children2 = EntityGetAllChildren(child)
                if children2 == nil or #children == 0 then
                    EntityKill(child)
                else
                    local original_verlet_id = get_variable_component(child, "beeg_smol.original_verlet", "value_int")
                    local in_use = false
                    for _, child2 in ipairs(children) do
                        if child2 == original_verlet_id then
                            in_use = true
                        end
                    end

                    if in_use == false then
                        EntityKill(child)
                    end
                end
            end
        end

        --set_variable_component(new_scalable_verlet_segment, "beeg_smol.original_verlet", "value_int", original_verlet)
        mock_verlet = setup_mock_verlet(original_verlet)
    end

    local original_verlet_sprites = EntityGetComponentIncludingDisabled(original_verlet, "SpriteComponent")

    local original_verlet_positions = ComponentGetValue2(original_verlet_component, "positions")
    local link_x_index = 1
    --loop through original chain's link positions
    for i = 1, ComponentGetValue2(original_verlet_component, "num_links") do
        --two links to determine a sprite rotation
        local this_orig_x, this_orig_y = original_verlet_positions[link_x_index], original_verlet_positions[link_x_index + 1]
        local next_orig_x, next_orig_y = original_verlet_positions[link_x_index + 2], original_verlet_positions[link_x_index + 3]
        link_x_index = link_x_index + 2

        --there is probably a better way of accomodating noita's flipped y scale
        --than what I'm doing on the next few lines, but I am not a smart man
        local new_segment_rotation = math.atan2(next_orig_x - this_orig_x, -(next_orig_y - this_orig_y)) - (math.pi * 0.5)
        if new_segment_rotation > math.pi then new_segment_rotation = new_segment_rotation - (2 * math.pi) end
        if new_segment_rotation < -math.pi then new_segment_rotation = new_segment_rotation + (2 * math.pi) end

        --scale segment offsets
        local verlet_diff_x, verlet_diff_y = this_orig_x - original_verlet_positions[1], this_orig_y - original_verlet_positions[2]
        local scaled_verlet_diff_x, scaled_verlet_diff_y = verlet_diff_x * scale, verlet_diff_y * scale
        this_orig_x, this_orig_y = this_orig_x + (scaled_verlet_diff_x - verlet_diff_x), this_orig_y + (scaled_verlet_diff_y - verlet_diff_y)

        --get the segment and adjust it
        local scalable_segment_name = "sprite_segment_" .. tostring(original_verlet_sprites[i])
        local scalable_segment = EntityGetWithName(scalable_segment_name)
        local scalable_segment_x, scalable_segment_y, scalable_segment_rotation = EntityGetTransform(scalable_segment)
        --more rotation normalization
        if scalable_segment_rotation > math.pi then scalable_segment_rotation = scalable_segment_rotation - (2 * math.pi) end
        if scalable_segment_rotation < -math.pi then scalable_segment_rotation = scalable_segment_rotation + (2 * math.pi) end
        EntitySetTransform(scalable_segment, this_orig_x, this_orig_y, new_segment_rotation, scale, scale)

        local scalable_sprite = EntityGetFirstComponentIncludingDisabled(scalable_segment, "SpriteComponent")
        ComponentSetValue2(scalable_sprite, "has_special_scale", true)
        ComponentSetValue2(scalable_sprite, "special_scale_x", scale)
        ComponentSetValue2(scalable_sprite, "special_scale_y", scale * sign_x)
        
        --game doesn't remember visibility edit when entities go off screen
        ComponentSetValue2(original_verlet_sprites[i], "visible", false)
    end

    return mock_verlet
end

function scan_for_uninitialized_scalable_enemies(entity_id)
    local x, y = EntityGetTransform(entity_id)
	local enemy = EntityGetClosestWithTag(  x, y, "enemy")
	local enemy_id = enemy
	local has_scalable_tag = EntityHasTag(enemy_id, "scalable_initialized")
	--local is_valid_enemy_for_scaling = is_valid_enemy_for_scaling(enemy_id)
	if has_scalable_tag ~= true then
		local has_prop_tag = EntityHasTag(enemy_id, "prop")
		local has_player_tag = EntityHasTag(enemy_id, "player_unit")
		--local has_boss_centipede_tag = EntityHasTag(enemy_id, "boss_centipede")
		if not has_prop_tag and not has_player_tag --[[and not has_boss_centipede_tag]] then
			local scalable_enemy = EntityLoad("mods/new_enemies/files/resize_entity_lib/scalable_enemy.xml", 0, 0)
			EntityAddChild(enemy_id, scalable_enemy)
		else
			--not what we're looking for but tag it so it doesn't come up again
			EntityAddTag(enemy_id, "scalable_initialized")
		end
	end
end

function is_valid_enemy_for_scaling(enemy_id)
    if 1 == 1 then return true end

    if (enemy_id == nil) and (enemy_id == NULL_ENTITY) then
        return false
    end

    local enemy_filename = EntityGetFilename(enemy_id)
	local animals_path = "data/entities/animals/"
    local boss_path = "/boss_"
    local is_boss = string.find(enemy_filename, boss_path)
	if (string.sub(enemy_filename, 1, #animals_path) ~= animals_path)
    or (is_boss ~= nil) then
		return false
	end

    local worm = EntityGetComponent(target, "WormAIComponent")
    local dragon = EntityGetComponent(target, "BossDragonComponent")
    local ghost = EntityGetComponent(target, "GhostComponent")
    local lukki = EntityGetComponent(target, "LimbBossComponent")
	if (worm ~= nil) or (dragon ~= nil) or (ghost ~= nil) or (lukki ~= nil) then
        return false
    end
	
	return true
end

function setup_scalable_enemy(scalable_enemy)
    local root = EntityGetParent(scalable_enemy)

    --put a script on the entity root to snitch about hits taken and damage model health
    EntityAddComponent2(root, "LuaComponent", {
        script_damage_received = "mods/new_enemies/files/resize_entity_lib/report_damage.lua"
    })

    --let entity transform take over sprite scales by default
    local sprite_components = EntityGetComponentIncludingDisabled(root, "SpriteComponent")
    if sprite_components ~= nil then
        for _, sprite_component in ipairs(sprite_components) do
            ComponentSetValue2(sprite_component, "has_special_scale", false)
        end
    end

    set_variable_component(scalable_enemy, "beeg_smol.scale", "value_float", 1)
    set_variable_component(scalable_enemy, "beeg_smol.scale_throttled", "value_float", 1)

    local initial_values = {
        has_damage_model = false,
        has_character_platforming = false,
        has_character_data = false,
        has_boss_dragon_component = false,
        has_worm_component = false,
        has_worm_ai_component = false,
        has_cell_eater_component = false
    }

    --damage model
    local damage_model_component = EntityGetFirstComponentIncludingDisabled(root, "DamageModelComponent")
    if damage_model_component ~= nil then 
        local last_hp = ComponentGetValue2(damage_model_component, "hp")

        --these go into variable components instead of a string object because they get used constantly
        set_variable_component(scalable_enemy, "beeg_smol.has_damage_model", "value_bool", true)
        set_variable_component(scalable_enemy, "beeg_smol.hits", "value_int", 0)
        set_variable_component(scalable_enemy, "beeg_smol.last_hits", "value_int", 0)
        set_variable_component(scalable_enemy, "beeg_smol.last_hp", "value_float", last_hp)
    end

    --character data
    local buoyancy_check_offset_y = nil
    local climb_over_y = nil
    local check_collision_max_size_x = nil
    local check_collision_max_size_y = nil
    local collision_aabb_min_x = nil
    local collision_aabb_max_x = nil
    local collision_aabb_min_y = nil
    local collision_aabb_max_y = nil
    local character_data_component = EntityGetFirstComponentIncludingDisabled(root, "CharacterDataComponent")
    if character_data_component ~= nil then
        buoyancy_check_offset_y = ComponentGetValue2(character_data_component, "buoyancy_check_offset_y")
        climb_over_y = ComponentGetValue2(character_data_component, "climb_over_y")
        check_collision_max_size_x = ComponentGetValue2(character_data_component, "check_collision_max_size_x")
        check_collision_max_size_y = ComponentGetValue2(character_data_component, "check_collision_max_size_y")
        collision_aabb_min_x = ComponentGetValue2(character_data_component, "collision_aabb_min_x")
        collision_aabb_max_x = ComponentGetValue2(character_data_component, "collision_aabb_max_x")
        collision_aabb_min_y = ComponentGetValue2(character_data_component, "collision_aabb_min_y")
        collision_aabb_max_y = ComponentGetValue2(character_data_component, "collision_aabb_max_y")

        initial_values["has_character_data"] = true
        initial_values["buoyancy_check_offset_y"] = buoyancy_check_offset_y
        initial_values["climb_over_y"] = climb_over_y
        initial_values["check_collision_max_size_x"] = check_collision_max_size_x
        initial_values["check_collision_max_size_y"] = check_collision_max_size_y
        initial_values["collision_aabb_min_x"] = collision_aabb_min_x
        initial_values["collision_aabb_max_x"] = collision_aabb_max_x
        initial_values["collision_aabb_min_y"] = collision_aabb_min_y
        initial_values["collision_aabb_max_y"] = collision_aabb_max_y
    end
    
    --character platforming
    local fly_speed_mult = nil
    local velocity_min_x = nil
    local velocity_max_x = nil
    local character_platforming_component = EntityGetFirstComponentIncludingDisabled(root, "CharacterPlatformingComponent")
    if character_platforming_component ~= nil then
        fly_speed_mult = ComponentGetValue2(character_platforming_component, "fly_speed_mult")
        velocity_min_x = ComponentGetValue2(character_platforming_component, "velocity_min_x")
        velocity_max_x = ComponentGetValue2(character_platforming_component, "velocity_max_x")

        initial_values["has_character_platforming"] = true
        initial_values["fly_speed_mult"] = fly_speed_mult
        initial_values["velocity_min_x"] = velocity_min_x
        initial_values["velocity_max_x"] = velocity_max_x
    end

    --boss dragon
    local boss_dragon_component = EntityGetFirstComponentIncludingDisabled(root, "BossDragonComponent")
    if boss_dragon_component ~= nil then
        initial_values["has_boss_dragon_component"] = true
        initial_values["dragon_part_distance"] = ComponentGetValue2(boss_dragon_component, "part_distance")
        initial_values["dragon_hitbox_radius"] = ComponentGetValue2(boss_dragon_component, "hitbox_radius")
        initial_values["dragon_ground_check_offset"] = ComponentGetValue2(boss_dragon_component, "ground_check_offset")
        initial_values["dragon_speed"] = ComponentGetValue2(boss_dragon_component, "speed")
        initial_values["dragon_speed_hunt"] = ComponentGetValue2(boss_dragon_component, "speed_hunt")
        setup_scalable_dragon_sprites(root)
    end

    --worm
    local worm_hitbox_radius = nil
    local worm_part_distance = nil
    local worm_ground_check_offset = nil
    local worm_target_kill_radius = nil
    local worm_gravity = nil
    local worm_tail_gravity = nil
    local worm_speed = nil
    local worm_component = EntityGetFirstComponentIncludingDisabled(root, "WormComponent")
    if worm_component ~= nil then
        worm_part_distance = ComponentGetValue2(worm_component, "part_distance")
        worm_hitbox_radius = ComponentGetValue2(worm_component, "hitbox_radius")
        worm_ground_check_offset = ComponentGetValue2(worm_component, "ground_check_offset")
        worm_target_kill_radius = ComponentGetValue2(worm_component, "target_kill_radius")
        worm_gravity = ComponentGetValue2(worm_component, "gravity")
        worm_tail_gravity = ComponentGetValue2(worm_component, "tail_gravity")
        worm_speed = ComponentGetValue2(worm_component, "speed")
        setup_scalable_worm_sprites(root)

        initial_values["has_worm_component"] = true
        initial_values["worm_part_distance"] = worm_part_distance
        initial_values["worm_hitbox_radius"] = worm_hitbox_radius
        initial_values["worm_ground_check_offset"] = worm_ground_check_offset
        initial_values["worm_target_kill_radius"] = worm_target_kill_radius
        initial_values["worm_gravity"] = worm_gravity
        initial_values["worm_tail_gravity"] = worm_tail_gravity
        initial_values["worm_speed"] = worm_speed
    end

    --worm_ai
    local worm_ai_give_up_area_radius = nil
    local worm_ai_speed = nil
    local worm_ai_speed_hunt = nil
    local worm_ai_component = EntityGetFirstComponentIncludingDisabled(root, "WormAIComponent")
    if worm_ai_component ~= nil then
        worm_ai_give_up_area_radius = ComponentGetValue2(worm_ai_component, "give_up_area_radius")
        worm_ai_speed = ComponentGetValue2(worm_ai_component, "speed")
        worm_ai_speed_hunt = ComponentGetValue2(worm_ai_component, "speed_hunt")

        initial_values["has_worm_ai_component"] = true
        initial_values["worm_ai_give_up_area_radius"] = worm_ai_give_up_area_radius
        initial_values["worm_ai_speed"] = worm_ai_speed
        initial_values["worm_ai_speed_hunt"] = worm_ai_speed_hunt
    end

    --cell eater
    local cell_eater_radius = nil
    local cell_eater_component = EntityGetFirstComponentIncludingDisabled(root, "CellEaterComponent")
    if cell_eater_component ~= nil then
        cell_eater_radius = ComponentGetValue2(cell_eater_component, "radius")

        initial_values["has_cell_eater_component"] = true
        initial_values["cell_eater_radius"] = cell_eater_radius
    end

    local hitbox_num = 1
    local hitbox_components = EntityGetComponentIncludingDisabled(root, "HitboxComponent")
    if hitbox_components ~= nil then
        for _, hitbox_component in ipairs(hitbox_components) do
            initial_values["hitbox_aabb_min_x_" .. tostring(hitbox_num)] = ComponentGetValue2(hitbox_component ,"aabb_min_x")
            initial_values["hitbox_aabb_max_x_" .. tostring(hitbox_num)] = ComponentGetValue2(hitbox_component ,"aabb_max_x")
            initial_values["hitbox_aabb_min_y_" .. tostring(hitbox_num)] = ComponentGetValue2(hitbox_component ,"aabb_min_y")
            initial_values["hitbox_aabb_max_y_" .. tostring(hitbox_num)] = ComponentGetValue2(hitbox_component ,"aabb_max_y")
            hitbox_num = hitbox_num + 1
        end
    end

    set_variable_component(scalable_enemy, "beeg_smol.enemy_values", "value_string", table_to_string(initial_values))
    EntityAddTag(root, "scalable_initialized")
end

function setup_scalable_dragon_sprites(root)
    --clean up any old scalable sprite entities from before
    --(happens if entity left viscinity of player and it's IDs get updated)
    local children = EntityGetAllChildren(root)
    for _, child in ipairs(children) do
        if string.find(EntityGetName(child), 'scalable_sprites_') ~= nil then
            EntityKill(child)
        end
    end

    local original_sprites = EntityGetComponentIncludingDisabled(root, "SpriteComponent")
    local scalable_sprites_entity = EntityCreateNew("scalable_sprites_" .. tostring(root))
    for _, sprite in ipairs(original_sprites) do
        --skip the health bar ui elements
        if string.find(ComponentGetValue2(sprite, "image_file"), 'enemies_gfx') ~= nil then
            --make original hidden
            ComponentSetValue2(sprite, "visible", false)
            local scalable_sprite_entity = EntityCreateNew("scalable_sprite_" .. tostring(sprite))
            EntityAddComponent2(scalable_sprite_entity, "SpriteComponent", {
                image_file = ComponentGetValue2(sprite, "image_file"),
                alpha = ComponentGetValue2(sprite, "alpha"),
                emissive = ComponentGetValue2(sprite, "emissive"),
                additive = ComponentGetValue2(sprite, "additive"),
                rect_animation = ComponentGetValue2(sprite, "rect_animation"),
                offset_x = ComponentGetValue2(sprite, "offset_x"),
                offset_y = ComponentGetValue2(sprite, "offset_y"),
                z_index = ComponentGetValue2(sprite, "z_index"),
                update_transform = true,
                update_transform_rotation = true
            })
            EntityAddChild(scalable_sprites_entity, scalable_sprite_entity)
        end
    end
    EntityAddChild(root, scalable_sprites_entity)
    return scalable_sprites_entity
end

function setup_scalable_worm_sprites(root)
    --clean up any old scalable sprite entities from before
    --(happens if entity left viscinity of player and it's IDs get updated)
    local children = EntityGetAllChildren(root)
    for _, child in ipairs(children) do
        if string.find(EntityGetName(child), 'scalable_sprites_') ~= nil then
            EntityKill(child)
        end
    end

    local original_sprites = EntityGetComponentIncludingDisabled(root, "SpriteComponent")
    local scalable_sprites_entity = EntityCreateNew("scalable_sprites_" .. tostring(root))
    for _, sprite in ipairs(original_sprites) do
        --skip the health bar ui elements
        if string.find(ComponentGetValue2(sprite, "image_file"), 'enemies_gfx') ~= nil then
            --make original hidden
            ComponentSetValue2(sprite, "visible", false)
            local scalable_sprite_entity = EntityCreateNew("scalable_sprite_" .. tostring(sprite))
            EntityAddComponent2(scalable_sprite_entity, "SpriteComponent", {
                image_file = ComponentGetValue2(sprite, "image_file"),
                alpha = ComponentGetValue2(sprite, "alpha"),
                emissive = ComponentGetValue2(sprite, "emissive"),
                additive = ComponentGetValue2(sprite, "additive"),
                rect_animation = ComponentGetValue2(sprite, "rect_animation"),
                offset_x = ComponentGetValue2(sprite, "offset_x"),
                offset_y = ComponentGetValue2(sprite, "offset_y"),
                z_index = ComponentGetValue2(sprite, "z_index"),
                update_transform = true,
                update_transform_rotation = true
            })
            --add a stains component in case original sprite was meant to be stainable
            EntityAddComponent2(scalable_sprite_entity, "SpriteStainsComponent", {})
            EntityAddChild(scalable_sprites_entity, scalable_sprite_entity)
        end
    end
    EntityAddChild(root, scalable_sprites_entity)
    return scalable_sprites_entity
end

function enemy_increment_hits(enemy_id, increment)
    local children = EntityGetAllChildren(enemy_id)
    if children == nil then return end
    for _, child in ipairs(children) do
        if EntityGetName(child) == "scalable_enemy" then
            local hits = get_variable_component(child, "beeg_smol.hits", "value_int") + increment
            set_variable_component(child, "beeg_smol.hits", "value_int", hits)
        end
    end
end

function get_scalable_entity_current_scale(scalable_entity)
    local scale_throttled = get_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float")
    if scale_throttled ~= nil then
        return scale_throttled
    else
        return 1
    end
end

function update_entity_scale_from_combat(scalable_entity)
    local root = EntityGetParent(scalable_entity)

    --update player scaling values
    if EntityHasTag(root, "player_unit") then
        local scale = get_variable_component(scalable_entity, "beeg_smol.scale", "value_float")
        if scale == nil then return 1 end
        local scale_throttled = get_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float")
        local last_hp = get_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float")
        local last_max_hp = get_variable_component(scalable_entity, "beeg_smol.last_max_hp", "value_float")
        local last_kills = get_variable_component(scalable_entity, "beeg_smol.last_kills", "value_int")

        --get current hp percent and kills
        local damage_model_component = EntityGetFirstComponentIncludingDisabled(root, "DamageModelComponent")
        local current_hp = ComponentGetValue2(damage_model_component, "hp")
        local current_max_hp = ComponentGetValue2(damage_model_component, "max_hp")
        local current_hp_percent = current_hp / current_max_hp
        local last_hp_percent = last_hp / last_max_hp
        local current_kills = tonumber(StatsGetValue("enemies_killed"))
        local scale_down, scale_up = 0, 0

        --record new hp values without scaling on hp increases or any max_hp changes (healing, hearts, heart mage curse, etc)
        -- if math.round(current_max_hp, 5) ~= math.round(last_max_hp, 5) or math.round(current_hp_percent, 5) > math.round(last_hp_percent, 5) then
            -- set_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float", current_hp)
            -- set_variable_component(scalable_entity, "beeg_smol.last_max_hp", "value_float", current_max_hp)
        -- --scale down from damage
        -- elseif math.round(current_hp_percent, 5) < math.round(last_hp_percent, 5) then
            -- local percent_diff = last_hp_percent - current_hp_percent
            -- local magnitude_multiplier = math.max(scale, min_scale) --use current scale as input to adjust scale down magnitude
            -- scale_down = percent_diff * -damage_effect_on_scale * magnitude_multiplier
            -- set_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float", current_hp)
        -- end

        --scale up from kills
        -- if current_kills > last_kills then
            -- local new_kills = current_kills - last_kills
            -- scale_up = new_kills * kills_effect_on_scale
            -- set_variable_component(scalable_entity, "beeg_smol.last_kills", "value_int", current_kills)
        -- end

        --update scale on scale_up or scale_down
        if scale_up ~= 0 or scale_down ~= 0 then
            --clamp new scale to global min and max, save new value
            scale = math.clamp(scale + scale_down + scale_up, min_scale, max_scale)
            local frame_num = GameGetFrameNum()
            local last_reported_scale_frame = get_variable_component(scalable_entity, "beeg_smol.last_reported_scale_frame", "value_int")
            if frame_num - 100 > last_reported_scale_frame then
                -- GamePrint(tostring(math.round(scale, 2)) .. 'x scale')
                set_variable_component(scalable_entity, "beeg_smol.last_reported_scale", "value_float", scale)
                set_variable_component(scalable_entity, "beeg_smol.last_reported_scale_frame", "value_int", frame_num)
            end
            set_variable_component(scalable_entity, "beeg_smol.scale", "value_float", scale)
        end

        --throttle all scale changes to a speed limit
        if scale_throttled ~= scale then
            if math.abs(scale_throttled - scale) < scale_throttle then
                scale_throttled = scale
            else
                scale_throttled = scale_throttled - math.clamp(scale_throttled - scale, -scale_throttle, scale_throttle)
            end
            set_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float", scale_throttled)
            return scale_throttled
        --return nil instead of a number if there has been no throttled scale change
        else
            return nil
        end
    --update enemy scaling values
    else
        if get_variable_component(scalable_entity, "beeg_smol.has_damage_model", "value_bool") ~= true then return end

        local scale = get_variable_component(scalable_entity, "beeg_smol.scale", "value_float")
        local scale_throttled = get_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float")
        local hits = get_variable_component(scalable_entity, "beeg_smol.hits", "value_int")
        local last_hits = get_variable_component(scalable_entity, "beeg_smol.last_hits", "value_int")
        local last_hp = get_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float")

        --external influences can override the entity's scale for an arbitrary number of frames
        --to "beeg" or "smol", using a variable component
        local force_scale_frame = get_variable_component(scalable_entity, "beeg_smol.force_scale", "value_int")
        if force_scale_frame ~= nil then
            if GameGetFrameNum() > force_scale_frame then
                remove_variable_component(scalable_entity, "beeg_smol.force_scale")
            else
                local type = get_variable_component(scalable_entity, "beeg_smol.force_scale", "value_string")
                if type == "beeg" then
                    scale = max_scale
                elseif type == "smol" then
                    scale = min_scale
                end
            end
        end

        local damage_model_component = EntityGetFirstComponentIncludingDisabled(root, "DamageModelComponent")
        if damage_model_component == nil then return end
        local current_hp = ComponentGetValue2(damage_model_component, "hp")
        local max_hp = ComponentGetValue2(damage_model_component, "max_hp")
        local current_hp_percent = current_hp / max_hp
        local last_hp_percent = last_hp / max_hp
        local scale_down, scale_up = 0, 0

        --this first check is purely just a fix for kolmi's activation frame, when his HP adjusts according to orb count
        if last_hp_percent > 1.0 then
            set_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float", current_hp)
        --normal damage check
        elseif math.round(current_hp_percent, 5) < math.round(last_hp_percent, 5) then
            local percent_diff = last_hp_percent - current_hp_percent
            scale_down = percent_diff * -enemy_damage_effect_on_scale
            set_variable_component(scalable_entity, "beeg_smol.last_hp", "value_float", current_hp)
        end

        if hits > last_hits then
            local hit_diff = hits - last_hits
            scale_up = hit_diff * enemy_hits_effect_on_scale
            set_variable_component(scalable_entity, "beeg_smol.last_hits", "value_int", hits)
        end

        if scale_up ~= 0 or scale_down ~= 0 then
            --clamp new scale to global min and max, save new value
            --enemy_values.scale = math.clamp(enemy_values.scale + scale_down + scale_up, min_scale, max_scale)
            scale = math.clamp(scale + scale_down + scale_up, min_scale, max_scale)
            set_variable_component(scalable_entity, "beeg_smol.scale", "value_float", scale)
        end

        --if math.abs(enemy_values.scale_throttled - enemy_values.scale) < scale_throttle then
        if scale_throttled ~= scale then
            if math.abs(scale_throttled - scale) < scale_throttle then
                scale_throttled = scale
                set_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float", scale_throttled)
            else
                local new_scale_throttled = scale_throttled - math.clamp(scale_throttled - scale, -scale_throttle, scale_throttle)
                if new_scale_throttled ~= scale_throttled then
                    set_variable_component(scalable_entity, "beeg_smol.scale_throttled", "value_float", new_scale_throttled)
                    scale_throttled = new_scale_throttled
                end
            end
        end

        if scale_throttled == scale then
            return nil
        end

        return scale_throttled
    end
end

function update_entity_scale_from_juice(entity, change)
    local parent = EntityGetParent(entity)
	local px, py = EntityGetTransform( parent )
    local children = EntityGetAllChildren(parent)
    local scalable_entity = nil
    for _, child in ipairs(children) do
        local entity_name = EntityGetName(child)
        if entity_name == "scalable_player" or entity_name == "scalable_enemy" then
            scalable_entity = child
        end
    end

    if scalable_entity == nil then return end
    local scale = get_variable_component(scalable_entity, "beeg_smol.scale", "value_float")

    --player
    if EntityHasTag(parent, "player_unit") then    
        local last_reported_scale = get_variable_component(scalable_entity, "beeg_smol.last_reported_scale", "value_float")
        local last_reported_scale_frame = get_variable_component(scalable_entity, "beeg_smol.last_reported_scale_frame", "value_int")
        local frame_num = GameGetFrameNum()
        if scale ~= last_reported_scale and (frame_num - scale_report_cooldown > last_reported_scale_frame) then
            -- GamePrint(tostring(math.round(scale, 2)) .. 'x scale')
            set_variable_component(scalable_entity, "beeg_smol.last_reported_scale", "value_float", scale)
            set_variable_component(scalable_entity, "beeg_smol.last_reported_scale_frame", "value_int", frame_num)
        end
		
		local new_scale = scale

		local change_dir
		if change == "grow" then
		  change_dir = 1
		else
		  change_dir = -1
		end
		
		local change_amount = 0.001
		new_scale = math.max(min_scale, math.min(max_scale, new_scale + change_dir * change_amount))
		local next_frame_sound = GetValueInteger("next_frame_sound", 0)
		if next_frame_sound == GameGetFrameNum() then
		  GamePlaySound("data/audio/Desktop/misc.bank", "misc/telekinesis/no_object", px, py)
		  next_frame_sound = 0
		  SetValueInteger("next_frame_sound", next_frame_sound)
		end
		if next_frame_sound == 0 and new_scale > min_scale and new_scale < max_scale then
			local min_delay = 5
			local max_delay = 30
			local diff = max_delay - min_delay
			local delay = diff - math.floor(new_scale / 1 * diff) + min_delay
			SetValueInteger("next_frame_sound", GameGetFrameNum() + delay)
		end

		set_variable_component(scalable_entity, "beeg_smol.scale", "value_float", new_scale)
		GameSetPostFxParameter("zoom_in_amount", scale, 0, 0, 0)

    --enemies
    else
        if scale ~= nil then
            local new_scale = scale
            if change == "grow" then
                new_scale = math.min(max_scale, scale + enemy_juice_increment)
            elseif change == "shrink" then
                new_scale = math.max(min_scale, scale - enemy_juice_increment)
            end
            if new_scale ~= scale then
                set_variable_component(scalable_entity, "beeg_smol.scale", "value_float", new_scale)
            end
        end
    end
end

function update_player(scalable_player, scale)
    local player = EntityGetWithTag("player_unit")[1]

    --reload the verlet_chain_cape is it disappears
    local verlet_chain_cape = EntityGetWithTag("verlet_chain_cape")[1]
    if verlet_chain_cape == nil then
		local children = EntityGetAllChildren(player)
		for _, child in ipairs(children) do
			--kill the verlet chain cape
			if EntityHasTag(child, "mock_verlet") == true then
				EntityKill(child)
			end
		end
        verlet_chain_cape = EntityLoad("mods/new_enemies/files/resize_entity_lib/verlet_chain_cape/verlet_chain_cape.xml")
        EntityAddChild(player, verlet_chain_cape)
    end

    --if scale has changed, new scale is already known from updates
    --otherwise look it up
    local scale_the_player = false
    if scale ~= nil then
        scale_the_player = true
    else
        scale = get_variable_component(scalable_player, "beeg_smol.scale_throttled", "value_float")
    end

    --damage scaling
    --four different linear functions, since max and min limits aren't symmetrical (0.5 and 0.667)
    local damage_scale = 1
	
	damage_scale = (scale / 1.5) - (1/1.5)
	if scale >= 1 then    
		damage_scale = (scale / 6) - (1/6)
	end

    --if scale has changed, scale the player
    if scale_the_player == true then
        scale_player(scalable_player, player, scale, damage_scale)
    end

    --stuff that has to get updated every frame regardless of scale
    update_player_every_frame(scalable_player, player, scale, damage_scale)
end

function scale_player(scalable_player, player, scale, damage_scale)
    --retrieve original values
    local original_values = nil
    local original_values_key = 'beeg_smol.original_values'
    local original_values_string = get_variable_component(scalable_player, original_values_key, "value_string")
    if original_values_string ~= nil then
        original_values = string_to_table(original_values_string)
    else
        return
    end

    --character data
    local character_data_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterDataComponent")
    local scaled_buoyancy_check_offset_y = original_values.buoyancy_check_offset_y * scale
    local scaled_climb_over_y = math.max(2, original_values.climb_over_y * scale)
    local scaled_check_collision_max_size_x = math.max(3, original_values.check_collision_max_size_x * scale)
    local scaled_check_collision_max_size_y = math.max(3, original_values.check_collision_max_size_y * scale)
    local scaled_collision_aabb_min_x = math.min(-0.5, original_values.collision_aabb_min_x * scale)
    local scaled_collision_aabb_max_x = math.max(0.5, original_values.collision_aabb_max_x * scale)
    local scaled_collision_aabb_min_y = math.min(-0.5, original_values.collision_aabb_min_y * scale)
    local scaled_collision_aabb_max_y = math.max(0.5, original_values.collision_aabb_max_y * scale)
    local scaled_eff_hg_size_x = original_values.eff_hg_size_x * scale
    local scaled_eff_hg_size_y = original_values.eff_hg_size_y * scale
    local scaled_eff_hg_damage_min = original_values.eff_hg_damage_min * scale
    local scaled_eff_hg_damage_max = original_values.eff_hg_damage_max * scale
    -- local scaled_destroy_ground = original_values.destroy_ground * scale * 10
    ComponentSetValue2(character_data_component, "buoyancy_check_offset_y", scaled_buoyancy_check_offset_y)
    ComponentSetValue2(character_data_component, "climb_over_y", scaled_climb_over_y)
    ComponentSetValue2(character_data_component, "check_collision_max_size_x", scaled_check_collision_max_size_x)
    ComponentSetValue2(character_data_component, "check_collision_max_size_y", scaled_check_collision_max_size_y)
    ComponentSetValue2(character_data_component, "collision_aabb_min_x", scaled_collision_aabb_min_x)
    ComponentSetValue2(character_data_component, "collision_aabb_max_x", scaled_collision_aabb_max_x)
    ComponentSetValue2(character_data_component, "collision_aabb_min_y", scaled_collision_aabb_min_y)
    ComponentSetValue2(character_data_component, "collision_aabb_max_y", scaled_collision_aabb_max_y)
    ComponentSetValue2(character_data_component, "eff_hg_size_x", scaled_eff_hg_size_x)
    ComponentSetValue2(character_data_component, "eff_hg_size_y", scaled_eff_hg_size_y)
    ComponentSetValue2(character_data_component, "eff_hg_damage_min", scaled_eff_hg_damage_min)
    ComponentSetValue2(character_data_component, "eff_hg_damage_max", scaled_eff_hg_damage_max)
    -- ComponentSetValue2(character_data_component, "destroy_ground", scaled_destroy_ground)

    --character platforming
    local character_platforming_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterPlatformingComponent")
    local velocity_scale = 1
    if ModSettingGet("beeg_smol.flip_movement_scaling") ~= true then
        velocity_scale = 0.5 + (scale * 0.5)
    else
        if scale < 1 then
            velocity_scale = 2 - scale
        else
            velocity_scale = (9/8) - (scale * (1/8))
        end
    end
    local scaled_fly_speed_mult = original_values.fly_speed_mult * velocity_scale
    local scaled_velocity_min_x = original_values.velocity_min_x * velocity_scale
    local scaled_velocity_max_x = original_values.velocity_max_x * velocity_scale
    ComponentSetValue2(character_platforming_component, "fly_speed_mult", scaled_fly_speed_mult)
    ComponentSetValue2(character_platforming_component, "velocity_min_x", scaled_velocity_min_x)
    ComponentSetValue2(character_platforming_component, "velocity_max_x", scaled_velocity_max_x)

    --platform shooter player
    local platform_shooter_player_component = EntityGetFirstComponentIncludingDisabled(player, "PlatformShooterPlayerComponent")
    local scaled_eating_area_min_x = original_values.eating_area_min_x * scale
    local scaled_eating_area_max_x = original_values.eating_area_max_x * scale
    local scaled_eating_area_min_y = original_values.eating_area_min_y * scale
    local scaled_eating_area_max_y = original_values.eating_area_max_y * scale
    local scaled_eating_cells_per_frame = original_values.eating_cells_per_frame * scale
    ComponentSetValue2(platform_shooter_player_component, "eating_area_min", scaled_eating_area_min_x, scaled_eating_area_min_y)
    ComponentSetValue2(platform_shooter_player_component, "eating_area_max", scaled_eating_area_max_x, scaled_eating_area_max_y)
    ComponentSetValue2(platform_shooter_player_component, "eating_cells_per_frame", scaled_eating_cells_per_frame)

    --damage model
    local damage_model_component = EntityGetFirstComponentIncludingDisabled(player, "DamageModelComponent")
    --being smol turns on environment physics damage
    ComponentSetValue2(damage_model_component, "physics_objects_damage", false)
	
    local scaled_damage_multiplier_curse = math.max(0.25, original_values.damage_multiplier_curse + damage_scale)
    local scaled_damage_multiplier_drill = math.max(0.25, original_values.damage_multiplier_drill + damage_scale)
    local scaled_damage_multiplier_electricity = math.max(0.25, original_values.damage_multiplier_electricity + damage_scale)
    local scaled_damage_multiplier_explosion = math.max(0.25, original_values.damage_multiplier_explosion + damage_scale)
    local scaled_damage_multiplier_fire = math.max(0.25, original_values.damage_multiplier_fire + damage_scale)
    local scaled_damage_multiplier_healing = math.max(0.25, original_values.damage_multiplier_healing + damage_scale)
    local scaled_damage_multiplier_ice = math.max(0.25, original_values.damage_multiplier_ice + damage_scale)
    local scaled_damage_multiplier_melee = math.max(0.25, original_values.damage_multiplier_melee + damage_scale)
    local scaled_damage_multiplier_overeating = math.max(0.25, original_values.damage_multiplier_overeating + damage_scale)
    --local scaled_damage_multiplier_physics_hit = math.max(0.25, original_values.damage_multiplier_physics_hit + damage_scale)
    local scaled_damage_multiplier_poison = math.max(0.25, original_values.damage_multiplier_poison + damage_scale)
    local scaled_damage_multiplier_projectile = math.max(0.25, original_values.damage_multiplier_projectile + damage_scale)
    local scaled_damage_multiplier_radioactive = math.max(0.25, original_values.damage_multiplier_radioactive + damage_scale)
    local scaled_damage_multiplier_slice = math.max(0.25, original_values.damage_multiplier_slice + damage_scale)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "curse", scaled_damage_multiplier_curse)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "drill", scaled_damage_multiplier_drill)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "electricity", scaled_damage_multiplier_electricity)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "explosion", scaled_damage_multiplier_explosion)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "fire", scaled_damage_multiplier_fire)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "healing", scaled_damage_multiplier_healing)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "ice", scaled_damage_multiplier_ice)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "melee", scaled_damage_multiplier_melee)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "overeating", scaled_damage_multiplier_overeating)
    --ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "physics_hit", scaled_damage_multiplier_physics_hit)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "poison", scaled_damage_multiplier_poison)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "projectile", scaled_damage_multiplier_projectile)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "radioactive", scaled_damage_multiplier_radioactive)
    ComponentObjectSetValue2(damage_model_component, "damage_multipliers", "slice", scaled_damage_multiplier_slice)

    --kick
    local kick_component = EntityGetFirstComponentIncludingDisabled(player, "KickComponent")
    local scaled_kick_radius = original_values.kick_radius * scale
    ComponentSetValue2(kick_component, "kick_radius", scaled_kick_radius)

    --hitbox
    local hitbox_component = EntityGetFirstComponentIncludingDisabled(player, "HitboxComponent")
    local scaled_hitbox_aabb_min_x = math.min(-0.5, original_values.hitbox_aabb_min_x * scale)
    local scaled_hitbox_aabb_max_x = math.max(0.5, original_values.hitbox_aabb_max_x * scale)
    local scaled_hitbox_aabb_min_y = math.min(-0.5, original_values.hitbox_aabb_min_y * scale)
    local scaled_hitbox_aabb_max_y = math.max(0.5, original_values.hitbox_aabb_max_y * scale)
    ComponentSetValue2(hitbox_component, "aabb_min_x", scaled_hitbox_aabb_min_x)
    ComponentSetValue2(hitbox_component, "aabb_max_x", scaled_hitbox_aabb_max_x)
    ComponentSetValue2(hitbox_component, "aabb_min_y", scaled_hitbox_aabb_min_y)
    ComponentSetValue2(hitbox_component, "aabb_max_y", scaled_hitbox_aabb_max_y)
end

function update_player_every_frame(scalable_player, player, scale, damage_scale)
    local frame_num = GameGetFrameNum()

    --x scale flip
    local controls_component = EntityGetComponent(player, "ControlsComponent")[1]
    local x = ComponentGetValue2(controls_component, "mAimingVector")
    local sign_x = 1
    if x < 0 then
        sign_x =  -1
    end

    --player root
    local world_player_x, world_player_y, world_player_rotation, world_player_scale_x, world_player_scale_y = EntityGetTransform(player)
    EntitySetTransform(player, world_player_x, world_player_y, world_player_rotation, scale * sign_x, scale)

    --body sprite
    local body_sprite = EntityGetFirstComponentIncludingDisabled(player, "SpriteComponent", "lukki_disable")
    local body_dummy_sprite = EntityGetFirstComponentIncludingDisabled(player, "SpriteComponent", "dummy_body")
    local body_rect_animation = ComponentGetValue2(body_sprite, "rect_animation")
    --the game sets all walk animations to "backwards" if the player is scaled, so I correct them on the dummy sprite body
    if body_rect_animation == "walk_backwards" or body_rect_animation == "walk_backwards_crouched" then
        local velocity_component = EntityGetFirstComponentIncludingDisabled(player, "VelocityComponent")
        local velocity_x = ComponentGetValue2(velocity_component, "mVelocity")
        --if velocity and look direction have the same signs then they'll multiply to a positive number
        if velocity_x * sign_x >= 0 then
            if body_rect_animation == "walk_backwards" then body_rect_animation = "walk" end
            if body_rect_animation == "walk_backwards_crouched" then body_rect_animation = "walk_crouched" end
        end
    end
	if (body_dummy_sprite ~= nil ) and (body_dummy_sprite > 0 ) then
		ComponentSetValue2(body_dummy_sprite, "rect_animation", body_rect_animation)
	end
    --"real" arm for rotation reference
    local arm = EntityGetWithTag("player_arm_r")[1]
    local arm_x, arm_y, arm_rotation, arm_scale_x, arm_scale_y = EntityGetTransform(arm)

    --our hotspot tracker for scaled arm position
    local arm_hotspot_tracker = EntityGetWithTag("arm_hotspot_tracker")[1]
    local arm_hotspot_x, arm_hotspot_y = EntityGetTransform(arm_hotspot_tracker)

    --adjust our visible dummy arm
    local dummy_arm = EntityGetWithTag("dummy_arm")[1]
    EntitySetTransform(dummy_arm, arm_hotspot_x, arm_hotspot_y, arm_rotation, scale, scale * sign_x)

    --our hotspot tracker for scaled hand position
    local hand_hotspot_tracker = EntityGetWithTag("hand_hotspot_tracker")[1]
    local hand_hotspot_x, hand_hotspot_y = EntityGetTransform(hand_hotspot_tracker)

    --adjust original arm position to track the hand hotspot
    EntitySetTransform(arm, hand_hotspot_x, hand_hotspot_y, arm_rotation, arm_scale_x, arm_scale_y)

    --fake an animated hotspot for our verlet chain cape using cos()
    --since the actual hotspot jumps around too fast at large scales,
    --tweaking out the verlet physics
    local cape_hotspot_cycle = (frame_num % 72) / 72
    local cape_hotspot_offset_x = 0
    local cape_hotspot_offset_y = math.cos(cape_hotspot_cycle * math.pi * 2)
    --lower the cape hotspot offset if crouched animation
    if string.find(body_rect_animation, 'crouched') ~= nil then
        cape_hotspot_offset_y = cape_hotspot_offset_y - 1
        cape_hotspot_offset_x = cape_hotspot_offset_x + sign_x
    --adjust offset x when idle
    elseif string.find(body_rect_animation, 'stand') ~= nil then
        cape_hotspot_offset_x = cape_hotspot_offset_x - sign_x
    end

    local verlet_chain_cape = EntityGetWithTag("verlet_chain_cape")[1]
    if verlet_chain_cape == nil then
		local children = EntityGetAllChildren(player)
		for _, child in ipairs(children) do
			--kill the verlet chain cape
			if EntityHasTag(child, "mock_verlet") == true then
				EntityKill(child)
			end
		end
        verlet_chain_cape = EntityLoad("mods/new_enemies/files/resize_entity_lib/verlet_chain_cape/verlet_chain_cape.xml")
        EntityAddChild(player, verlet_chain_cape)
    end

    local verlet_cape_x, verlet_cape_y, verlet_cape_rotation, verlet_cape_scale_x, verlet_cape_scale_y = EntityGetTransform(verlet_chain_cape)
    local cape_height = (7 + cape_hotspot_offset_y) * scale
    local cape_side_shift = cape_hotspot_offset_x * scale
    EntitySetTransform(verlet_chain_cape, world_player_x + cape_side_shift, world_player_y - cape_height, verlet_cape_rotation, verlet_cape_scale_x, verlet_cape_scale_y)

    local mock_verlet = update_mock_verlet(verlet_chain_cape, scale, sign_x)

    --cape position delay shows a bit, especially at high speeds / small scales
    --use current velocity to adjust mock cape's position to compensate
    --this section is *pure fudge*
    local velocity_component = EntityGetFirstComponentIncludingDisabled(player, "VelocityComponent")
    local velocity_x, velocity_y = ComponentGetValue2(velocity_component, "mVelocity")
    if math.abs(velocity_x) > 0.1 or math.abs(velocity_y) > 0.1 then
        --local mock_verlet = EntityGetWithName("scalable_verlet_" .. tostring(verlet_chain_cape))
        if mock_verlet ~= 0 and mock_verlet ~= nil then
            local mock_x, mock_y, mock_rotation, mock_scale_x, mock_scale_y = EntityGetTransform(mock_verlet)
            --local mock_offset_x, mock_offset_y = 0, 0
            if scale < 1.0 then
                velocity_y = velocity_y * (1.0 / scale)
                velocity_x = velocity_x * (1.0 / scale)
            end
            EntitySetTransform(mock_verlet, mock_x + velocity_x, mock_y + velocity_y, mock_rotation, mock_scale_x, mock_scale_y)
        end
    end

    local children = EntityGetAllChildren(player)

    local currently_invisible = false
    local beeg_id, smol_id = nil, nil
    local beegsofty_id, smolscrapper_id = nil, nil

    for _, child in ipairs(children) do
        --game effects (invisibility, etc)
        local entity_name = EntityGetName(child)
        if entity_name == "" then
            local game_effect = EntityGetFirstComponentIncludingDisabled(child, "GameEffectComponent")
            if game_effect ~= nil then
                scale_particle_effects_on_entity(child, scale)
                local effect_name = ComponentGetValue2(game_effect, "effect")
                if effect_name == "INVISIBILITY" then
                    currently_invisible = ComponentGetValue2(game_effect, "mInvisible")
                --elseif effect_name == "ON_FIRE" then
                --    scale_particle_effects_on_entity(child, scale)
                end

                if effect_name == "CUSTOM" then
                    local custom_effect_id = ComponentGetValue2(game_effect, "custom_effect_id")
                    if custom_effect_id == "BEEG" then
                        beeg_id = child
                    elseif custom_effect_id == "NEW_ENEMIES_SHRINK" then
                        smol_id = child
                    elseif custom_effect_id == "BEEGSOFTY" then
                        beegsofty_id = child
                    elseif custom_effect_id == "SMOLSCRAPPER" then
                        smolscrapper_id = child
                    end
                end
            end
        --why is the vanilla cape back?? die!
        elseif entity_name == "cape" then
            EntityKill(child)
        end

        --player-attached perks
        if EntityHasTag(child, "perk_entity") then
            scale_perk(child, scale)
        end
    end

    --stomp attack
    if scale > 1 and ModSettingGet("beeg_smol.stomp") == true then
        local character_data_component = EntityGetFirstComponentIncludingDisabled(player, "CharacterDataComponent")
        if ComponentGetValue2(character_data_component, "mFramesOnGround") == 1 then
            local eff_hg_size_y = ComponentGetValue2(character_data_component, "eff_hg_size_x")
            local eff_hg_damage_min = ComponentGetValue2(character_data_component, "eff_hg_damage_min")
            local eff_hg_damage_max = ComponentGetValue2(character_data_component, "eff_hg_damage_max")
            local velocity_x, velocity_y = ComponentGetValue2(character_data_component, "mVelocity")
            local nearby_entities = EntityGetInRadius(world_player_x, world_player_y, eff_hg_size_y * 0.5)
            for _, entity in ipairs(nearby_entities) do
                if not EntityHasTag(entity, "player_unit") then
                    local damage = Random(eff_hg_damage_min, eff_hg_damage_max)
                    EntityInflictDamage(entity, damage, "DAMAGE_MELEE", "Minä's stomp", "NONE", velocity_x * 2,0, velocity_y * 2.0)
                end
            end
        end
    end

    --play scaling scound effect, with cooldown on playback
    if (beeg_id ~= nil or smol_id ~= nil) and not (beeg_id ~= nil and smol_id ~= nil) then --my shitty knockoff xor()
        local last_sfx_frame = get_variable_component(scalable_player, "beeg_smol.last_sfx_frame", "value_int")
        local last_sfx_name = get_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string")
        if last_sfx_name == "beeg" and smol_id ~= nil and last_sfx_frame < frame_num - 100 then
            GameEntityPlaySound(player, "shrink_2")
            set_variable_component(scalable_player, "beeg_smol.last_sfx_frame", "value_int", frame_num)
            set_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string", "smol")
        elseif last_sfx_name == "smol" and beeg_id ~= nil and last_sfx_frame < frame_num - 100 then
            GameEntityPlaySound(player, "grow_2")
            set_variable_component(scalable_player, "beeg_smol.last_sfx_frame", "value_int", frame_num)
            set_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string", "beeg")
        elseif last_sfx_frame < frame_num - 500 then
            if beeg_id ~= nil then
                GameEntityPlaySound(player, "grow_2")
                set_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string", "beeg")
            else
                GameEntityPlaySound(player, "shrink_2")
                set_variable_component(scalable_player, "beeg_smol.last_sfx_name", "value_string", "smol")
            end
            set_variable_component(scalable_player, "beeg_smol.last_sfx_frame", "value_int", frame_num)
        end
    end

    --assign beeg softy status effect
    -- if scale > 1 then
        -- if smolscrapper_id ~= nil then
            -- EntityKill(smolscrapper_id)
        -- end
        -- if beegsofty_id == nil then
            -- beegsofty_id = EntityLoad("mods/beeg_smol/files/beeg/effect_beegsofty.xml", player_x, player_y)
            -- EntityAddChild(player, beegsofty_id)
        -- end
        -- local beegsofty_ui_icon = EntityGetFirstComponentIncludingDisabled(beegsofty_id, "UIIconComponent")
        -- local and_or_but = ", but"
        -- if ModSettingGet("beeg_smol.flip_armor_scaling") == true then
            -- and_or_but = ", and"
            -- ComponentSetValue2(beegsofty_ui_icon, "icon_sprite_file", "mods/beeg_smol/files/beeg/beegsofty_status_indicator_flipped.png")
        -- end
        -- local can_stomp = ""
        -- if ModSettingGet("beeg_smol.stomp") == true then
            -- can_stomp = and_or_but .. " you can stomp things"
        -- end
        -- ComponentSetValue2(beegsofty_ui_icon, "description", tostring(math.round(1 + damage_scale, 2)) .. "x damage taken" .. can_stomp)
    -- end

    --assign smol scrapper status effect
    -- if scale < 1 then
        -- if beegsofty_id ~= nil then
            -- EntityKill(beegsofty_id)
        -- end
        -- if smolscrapper_id == nil then
            -- smolscrapper_id = EntityLoad("mods/beeg_smol/files/smol/effect_smolscrapper.xml", player_x, player_y)
            -- EntityAddChild(player, smolscrapper_id)
        -- end
        -- local smolscrapper_ui_icon = EntityGetFirstComponentIncludingDisabled(smolscrapper_id, "UIIconComponent")
        -- local and_or_but = ", but"
        -- if ModSettingGet("beeg_smol.flip_armor_scaling") == true then
            -- and_or_but = ", and"
            -- ComponentSetValue2(smolscrapper_ui_icon, "icon_sprite_file", "mods/beeg_smol/files/smol/smolscrapper_status_indicator_flipped.png")
        -- end
        -- local takes_physics_damage = ""
        -- if ModSettingGet("beeg_smol.physics_damage") == true then
            -- takes_physics_damage = and_or_but .. " you take physics damage"
        -- end
        -- ComponentSetValue2(smolscrapper_ui_icon, "description", tostring(math.round(1 + damage_scale, 2)) .. "x damage taken" .. takes_physics_damage)
    -- end

    --the held item
    local held_item = get_held_item(player)
    local tags = EntityGetTags(held_item)
	
	-- Returns true if entity is a wand
	local function entity_is_wand(entity_id)
		local ability_component = EntityGetFirstComponentIncludingDisabled(entity_id, "AbilityComponent")
		return ComponentGetValue2(ability_component, "use_gun_script") == true
	end

	for _, child in ipairs(children) do
        local entity_name = EntityGetName(child)
		if entity_name == "inventory_quick" then
			for _, child2 in ipairs(EntityGetAllChildren(child)) do
				if entity_is_wand(child2) then
					scale_wand(child2, scale)
					--put a script on the wand or item to make them scale back to normal
					local var_comp = get_variable_storage_component( child2, "set_up_original_scale_adjuster" )
					if var_comp == nil then
						EntityAddComponent2(child2, "VariableStorageComponent", {
							name = "set_up_original_scale_adjuster"
						})
						EntityAddComponent2(child2, "LuaComponent", {
							script_throw_item = "mods/new_enemies/files/resize_entity_lib/restore_item_scale.lua"
						})
					end
				end
				if EntityHasTag(child2, "item_pickup") or EntityHasTag(child2, "item_physics") then
					local sprite_components = EntityGetComponentIncludingDisabled(child2, "SpriteComponent")
					if sprite_components ~= nil then
						for _, sprite_component in ipairs(sprite_components) do
							ComponentSetValue2(sprite_component, "has_special_scale", true)
							ComponentSetValue2(sprite_component, "special_scale_x", scale)
							ComponentSetValue2(sprite_component, "special_scale_y", scale * sign_x)
						end
					end
					--handle any particle effects on the held item
					scale_particle_effects_on_entity(child2, scale)
					
					local var_comp = get_variable_storage_component( child2, "set_up_original_scale_adjuster" )
					if var_comp == nil then
						EntityAddComponent2(child2, "VariableStorageComponent", {
							name = "set_up_original_scale_adjuster"
						})
						EntityAddComponent2(child2, "LuaComponent", {
							script_throw_item = "mods/new_enemies/files/resize_entity_lib/restore_item_scale.lua"
						})
					end
				end
			end
		end
	end
	
    -- if string.find(tags, "wand") ~= nil then

        -- scale_wand(held_item, scale)

    -- --general items
    -- elseif string.find(tags, "item_pickup") ~= nil or string.find(tags, "item_physics") ~= nil then
        -- --scale the sprite (assuming a default scale of 1 always, for now)
        -- local sprite_components = EntityGetComponent(held_item, "SpriteComponent")
        -- if sprite_components ~= nil then
            -- for _, sprite_component in ipairs(sprite_components) do
                -- ComponentSetValue2(sprite_component, "has_special_scale", true)
                -- ComponentSetValue2(sprite_component, "special_scale_x", scale)
                -- ComponentSetValue2(sprite_component, "special_scale_y", scale * sign_x)
            -- end
        -- end
        -- --handle any particle effects on the held item
        -- scale_particle_effects_on_entity(held_item, scale)
    -- end

    --similar fudge as the cape position adjustment per velocity, but for the held item
    if scale <= 1 then
        local item_x, item_y, item_rotation, item_scale_x, item_scale_y = EntityGetTransform(held_item)
        item_x = item_x + (velocity_x * scale * 1.4)
        item_y = item_y + (velocity_y * scale * 1.4)
        EntitySetTransform(held_item, item_x, item_y, item_rotation, item_scale_x, item_scale_y)
    end

    --invisibility visual fix for dummy arm and cape
    local status_invisible = get_variable_component(player, "beeg_smol.status_invisible", "value_bool")
    if status_invisible == true and currently_invisible == false then
        --GlobalsSetValue("beeg_smol.status_invisible", "false")
        set_variable_component(player, "beeg_smol.status_invisible", "value_bool", false)
        local dummy_arm_sprite = EntityGetFirstComponentIncludingDisabled(dummy_arm, "SpriteComponent")
        ComponentSetValue2(dummy_arm_sprite, "alpha", 1.0)
        local cape_segments = EntityGetAllChildren(mock_verlet)
        for _, cape_segment in ipairs(cape_segments) do
            local scalable_sprite = EntityGetFirstComponentIncludingDisabled(cape_segment, "SpriteComponent")
            ComponentSetValue2(scalable_sprite, "alpha", 1.0)
        end
    --remove fixes when no longer needed
    elseif status_invisible ~= true and currently_invisible == true then
        set_variable_component(player, "beeg_smol.status_invisible", "value_bool", true)
        local dummy_arm_sprite = EntityGetFirstComponentIncludingDisabled(dummy_arm, "SpriteComponent")
        ComponentSetValue2(dummy_arm_sprite, "alpha", 0.25)
        local cape_segments = EntityGetAllChildren(mock_verlet)
        for _, cape_segment in ipairs(cape_segments) do
            local scalable_sprite = EntityGetFirstComponentIncludingDisabled(cape_segment, "SpriteComponent")
            ComponentSetValue2(scalable_sprite, "alpha", 0.25)
        end
    end
end

function scale_particle_effects_on_entity(entity_id, scale)
    --handle any particle effects on the item
    local particle_emitter_components = EntityGetComponent(entity_id, "ParticleEmitterComponent")
    if particle_emitter_components ~= nil then
        for _, particle_emitter_component in ipairs(particle_emitter_components) do
            --fetch the original particle effect values, or save them out on first pass
            local original_values_key = 'beeg_smol.original_values_' .. particle_emitter_component
            local original_values_string = get_variable_component(entity_id, original_values_key, "value_string")
            if original_values_string == nil then
                area_circle_radius_min, area_circle_radius_max = ComponentGetValue2(particle_emitter_component, "area_circle_radius")
                local new_original_values = {
                    count_min = ComponentGetValue2(particle_emitter_component, "count_min"),
                    count_max = ComponentGetValue2(particle_emitter_component, "count_max"),
                    x_pos_offset_min = ComponentGetValue2(particle_emitter_component, "x_pos_offset_min"),
                    x_pos_offset_max = ComponentGetValue2(particle_emitter_component, "x_pos_offset_max"),
                    y_pos_offset_min = ComponentGetValue2(particle_emitter_component, "y_pos_offset_min"),
                    y_pos_offset_max = ComponentGetValue2(particle_emitter_component, "y_pos_offset_max"),
                    area_circle_radius_min = area_circle_radius_min,
                    area_circle_radius_max = area_circle_radius_max
                }
                original_values_string = table_to_string(new_original_values)
                set_variable_component(entity_id, original_values_key, "value_string", original_values_string)
            end
            local original_values = string_to_table(original_values_string)
            --set current values
            ComponentSetValue2(particle_emitter_component, "count_min", original_values.count_min * scale)
            ComponentSetValue2(particle_emitter_component, "count_max", original_values.count_max * scale)
            ComponentSetValue2(particle_emitter_component, "x_pos_offset_min", original_values.x_pos_offset_min * scale)
            ComponentSetValue2(particle_emitter_component, "x_pos_offset_max", original_values.x_pos_offset_max * scale)
            ComponentSetValue2(particle_emitter_component, "y_pos_offset_min", original_values.y_pos_offset_min * scale)
            ComponentSetValue2(particle_emitter_component, "y_pos_offset_max", original_values.y_pos_offset_max * scale)
            ComponentSetValue2(particle_emitter_component, "area_circle_radius", 
                original_values.area_circle_radius_min * scale, original_values.area_circle_radius_max * scale)
        end
    end
end

function scale_convert_material_fields_on_entity(entity_id, scale)
    local convert_comps = EntityGetComponent(entity_id, "MagicConvertMaterialComponent")
    if convert_comps ~= nil then
        for _, comp in ipairs(convert_comps) do
            --fetch the original particle effect values, or save them out on first pass
            local original_radius_key = 'beeg_smol.original_convert_radius_' .. comp
            local original_radius = get_variable_component(entity_id, original_radius_key, "value_float")
            if original_radius == nil then
                original_radius = ComponentGetValue2(comp, "radius")
                set_variable_component(entity_id, original_radius_key, "value_float", original_radius)
            end
            ComponentSetValue2(comp, "radius", original_radius * scale)
        end
    end
end

function scale_cell_eaters_on_entity(entity_id, scale)
    local cell_eater_comps = EntityGetComponent(entity_id, "CellEaterComponent")
    if cell_eater_comps ~= nil then
        for _, comp in ipairs(cell_eater_comps) do
            --fetch the original particle effect values, or save them out on first pass
            local original_radius_key = 'beeg_smol.original_eater_radius_' .. comp
            local original_radius = get_variable_component(entity_id, original_radius_key, "value_float")
            if original_radius == nil then
                original_radius = ComponentGetValue2(comp, "radius")
                set_variable_component(entity_id, original_radius_key, "value_float", original_radius)
            end
            ComponentSetValue2(comp, "radius", original_radius * scale)
        end
    end
end

function scale_energy_shields_on_entity(entity_id, scale)
    local energy_shield_comps = EntityGetComponent(entity_id, "EnergyShieldComponent")
    if energy_shield_comps ~= nil then
        for _, comp in ipairs(energy_shield_comps) do
            --fetch the original particle effect values, or save them out on first pass
            local original_radius_key = 'beeg_smol.original_shield_radius_' .. comp
            local original_radius = get_variable_component(entity_id, original_radius_key, "value_float")
            if original_radius == nil then
                original_radius = ComponentGetValue2(comp, "radius")
                set_variable_component(entity_id, original_radius_key, "value_float", original_radius)
            end
            ComponentSetValue2(comp, "radius", original_radius * scale)
        end
    end
end

function scale_area_damage_on_entity(entity_id, scale)
    local area_damage_comps = EntityGetComponent(entity_id, "AreaDamageComponent")
    if area_damage_comps ~= nil then
        for _, comp in ipairs(area_damage_comps) do
            --fetch the original particle effect values, or save them out on first pass
            local original_aabb_key = 'beeg_smol.original_area_dmg_radius_' .. comp
            local original_aabb = get_variable_component(entity_id, original_aabb_key, "value_float")
            if original_aabb == nil then
                original_aabb = ComponentGetValue2(comp, "aabb_max")
                set_variable_component(entity_id, original_aabb_key, "value_float", original_aabb)
            end
            ComponentSetValue2(comp, "aabb_min", -original_aabb * scale, -original_aabb * scale)
            ComponentSetValue2(comp, "aabb_max", original_aabb * scale, original_aabb * scale)
        end
    end
end

function scale_wand(wand_id, scale)
	local player = EntityGetWithTag("player_unit")[1]
    local wand_sprite_component = EntityGetFirstComponentIncludingDisabled(wand_id, "SpriteComponent")
    local original_values_key = 'beeg_smol.original_values'
    local original_values_string = get_variable_component(wand_id, original_values_key, "value_string")
    if original_values_string == nil then
        local offset_x, offset_y = 3, 3
        local shoot_pos_x, shoot_pos_y = 8, 0
		local hotspotscomponent = EntityGetFirstComponentIncludingDisabled( wand_id, "HotspotComponent", "shoot_pos" )
		if hotspotscomponent > 0 then
			shoot_pos_x, shoot_pos_y = ComponentGetValueVector2( hotspotscomponent, "offset" )
		end
		
        local wand_image_file = ComponentGetValue2(wand_sprite_component, "image_file")
        for _, wand in ipairs(wands) do
            if wand.file == wand_image_file then
                offset_x = wand.grip_x
                offset_y = wand.grip_y
                shoot_pos_x = wand.tip_x
                shoot_pos_y = wand.tip_y
                break
            end
        end
        local new_original_values = {
            offset_x = offset_x,
            offset_y = offset_y,
            shoot_pos_x = shoot_pos_x,
            shoot_pos_y = 0 --uh nevermind, apparently y shoot position hotspot is always 0
        }
        original_values_string = table_to_string(new_original_values)
        set_variable_component(wand_id, original_values_key, "value_string", original_values_string)
    end
    local original_values = string_to_table(original_values_string)
    ComponentSetValue2(wand_sprite_component, "has_special_scale", true)
    ComponentSetValue2(wand_sprite_component, "special_scale_x", scale)
   --x scale flip
    local controls_component = EntityGetComponent(player, "ControlsComponent")[1]
    local x = ComponentGetValue2(controls_component, "mAimingVector")
    local sign_x = 1
    if x < 0 then
        sign_x =  -1
    end
	
    ComponentSetValue2(wand_sprite_component, "special_scale_y", scale * sign_x)
    ComponentSetValue2(wand_sprite_component, "offset_x", tonumber(original_values.offset_x))
    ComponentSetValue2(wand_sprite_component, "offset_y", tonumber(original_values.offset_y))

    local wand_hotspot_component = EntityGetFirstComponentIncludingDisabled(wand_id, "HotspotComponent", "shoot_pos")
    ComponentSetValue2(wand_hotspot_component, "offset", original_values.shoot_pos_x * scale, original_values.shoot_pos_y * scale)
end

function update_enemy_every_frame(scalable_entity, scale)
    local enemy_id = EntityGetParent(scalable_entity)
    local enemy_x, enemy_y, enemy_rotation, enemy_scale_x, enemy_scale_y = EntityGetTransform(enemy_id)

    local children = EntityGetAllChildren(enemy_id)
    --no children probably means the entity just died, RIP
    if children == nil then return end

    for _, child in ipairs(children) do
        --verlet chain scaling
        local verlet_physics_component = EntityGetFirstComponentIncludingDisabled(child, "VerletPhysicsComponent")
        if verlet_physics_component ~= nil then
            update_mock_verlet(child, scale)
        end
    end

    if EntityGetFirstComponentIncludingDisabled(enemy_id, "WormComponent") ~= nil then
        local hitbox_num = 1
        local hitbox_coords = {}
        local hitbox_components = EntityGetComponentIncludingDisabled(enemy_id, "HitboxComponent")
        for _, hitbox in ipairs(hitbox_components) do
            local x = (ComponentGetValue(hitbox, "aabb_min_x") + ComponentGetValue(hitbox, "aabb_max_x")) * 0.5
            local y = (ComponentGetValue(hitbox, "aabb_min_y") + ComponentGetValue(hitbox, "aabb_max_y")) * 0.5
            hitbox_coords[hitbox_num] = { x = x, y = y }
            hitbox_num = hitbox_num + 1
        end

        local scalable_sprites_entity = EntityGetWithName("scalable_sprites_" .. tostring(enemy_id))
        --entities get new ids when they leave area and come back
        --rename stuff to relink if that happens
        if scalable_sprites_entity == 0 then
            scalable_sprites_entity = setup_scalable_worm_sprites(enemy_id)
        end

        local worm_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "WormComponent")
        local scalable_sprite_entities = EntityGetAllChildren(scalable_sprites_entity)
        hitbox_num = 1
        for _, sprite_entity in ipairs(scalable_sprite_entities) do
            local sprite_rotation = 0
            if hitbox_num == 1 then
                local look_x, look_y = ComponentGetValue2(worm_component, "mTargetVec")
                sprite_rotation = math.atan2(look_y, look_x) - math.pi
                if sprite_rotation > math.pi then sprite_rotation = sprite_rotation - (2 * math.pi) end
                if sprite_rotation < -math.pi then sprite_rotation = sprite_rotation + (2 * math.pi) end
            else
                local look_x = hitbox_coords[hitbox_num].x - hitbox_coords[hitbox_num - 1].x
                local look_y = hitbox_coords[hitbox_num].y - hitbox_coords[hitbox_num - 1].y
                sprite_rotation = math.atan2(look_y, look_x)
                if sprite_rotation > math.pi then sprite_rotation = sprite_rotation - (2 * math.pi) end
                if sprite_rotation < -math.pi then sprite_rotation = sprite_rotation + (2 * math.pi) end
            end
            EntitySetTransform(sprite_entity, enemy_x + hitbox_coords[hitbox_num].x, enemy_y + hitbox_coords[hitbox_num].y, sprite_rotation, scale, scale)
            hitbox_num = hitbox_num + 1
        end
    end

    if EntityGetFirstComponentIncludingDisabled(enemy_id, "BossDragonComponent") ~= nil then
        local hitbox_num = 1
        local hitbox_coords = {}
        local hitbox_components = EntityGetComponentIncludingDisabled(enemy_id, "HitboxComponent")
        for _, hitbox in ipairs(hitbox_components) do
            local x = (ComponentGetValue(hitbox, "aabb_min_x") + ComponentGetValue(hitbox, "aabb_max_x")) * 0.5
            local y = (ComponentGetValue(hitbox, "aabb_min_y") + ComponentGetValue(hitbox, "aabb_max_y")) * 0.5
            hitbox_coords[hitbox_num] = { x = x, y = y }
            hitbox_num = hitbox_num + 1
        end

        local scalable_sprites_entity = EntityGetWithName("scalable_sprites_" .. tostring(enemy_id))
        --entities get new ids when they leave area and come back
        --rename stuff to relink if that happens
        if scalable_sprites_entity == 0 then
            scalable_sprites_entity = setup_scalable_dragon_sprites(enemy_id)
        end

        local boss_dragon_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "BossDragonComponent")
        local scalable_sprite_entities = EntityGetAllChildren(scalable_sprites_entity)
        hitbox_num = 1
        for _, sprite_entity in ipairs(scalable_sprite_entities) do
            local sprite_rotation = 0
            if hitbox_num == 1 then
                local look_x, look_y = ComponentGetValue2(boss_dragon_component, "mTargetVec")
                sprite_rotation = math.atan2(look_y, look_x) - math.pi
                if sprite_rotation > math.pi then sprite_rotation = sprite_rotation - (2 * math.pi) end
                if sprite_rotation < -math.pi then sprite_rotation = sprite_rotation + (2 * math.pi) end
            else
                local look_x = hitbox_coords[hitbox_num].x - hitbox_coords[hitbox_num - 1].x
                local look_y = hitbox_coords[hitbox_num].y - hitbox_coords[hitbox_num - 1].y
                sprite_rotation = math.atan2(look_y, look_x)
                if sprite_rotation > math.pi then sprite_rotation = sprite_rotation - (2 * math.pi) end
                if sprite_rotation < -math.pi then sprite_rotation = sprite_rotation + (2 * math.pi) end
            end
            EntitySetTransform(sprite_entity, enemy_x + hitbox_coords[hitbox_num].x, enemy_y + hitbox_coords[hitbox_num].y, sprite_rotation, scale, scale)
            hitbox_num = hitbox_num + 1
        end

        local children = EntityGetAllChildren(enemy_id)
        if children ~= nil then
            for _, child in ipairs(children) do
                local sprite = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
                if sprite ~= nil then
                    ComponentSetValue2(sprite, "has_special_scale", true)
                    ComponentSetValue2(sprite, "special_scale_x", scale)
                    ComponentSetValue2(sprite, "special_scale_y", scale)
                end
            end
        end
    end
end

function scale_enemy(scalable_entity, scale)
    local enemy_values_string = get_variable_component(scalable_entity, "beeg_smol.enemy_values", "value_string")
    if enemy_values_string == nil then return end
    local enemy_values = string_to_table(enemy_values_string)
    local enemy_id = EntityGetParent(scalable_entity)
    local enemy_x, enemy_y, enemy_rotation, enemy_scale_x, enemy_scale_y = EntityGetTransform(enemy_id)
    EntitySetTransform(enemy_id, enemy_x, enemy_y, enemy_rotation, scale, scale)

    if enemy_values.has_character_platforming == "true" then
        local character_platforming_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "CharacterPlatformingComponent")
        ComponentSetValue2(character_platforming_component, "fly_speed_mult", enemy_values.fly_speed_mult * scale)
        ComponentSetValue2(character_platforming_component, "velocity_min_x", enemy_values.velocity_min_x * scale)
        ComponentSetValue2(character_platforming_component, "velocity_max_x", enemy_values.velocity_max_x * scale)
    end

    if enemy_values.has_character_data == "true" then
        local character_data_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "CharacterDataComponent")
        ComponentSetValue2(character_data_component, "buoyancy_check_offset_y", enemy_values.buoyancy_check_offset_y * scale)
        ComponentSetValue2(character_data_component, "climb_over_y",  math.max(2, enemy_values.climb_over_y * scale))
        ComponentSetValue2(character_data_component, "check_collision_max_size_x", math.max(3, enemy_values.check_collision_max_size_x * scale))
        ComponentSetValue2(character_data_component, "check_collision_max_size_y", math.max(3, enemy_values.check_collision_max_size_y * scale))
        ComponentSetValue2(character_data_component, "collision_aabb_min_x",  math.min(-0.5, enemy_values.collision_aabb_min_x * scale))
        ComponentSetValue2(character_data_component, "collision_aabb_max_x",  math.max(0.5, enemy_values.collision_aabb_max_x * scale))
        ComponentSetValue2(character_data_component, "collision_aabb_min_y", math.min(-0.5, enemy_values.collision_aabb_min_y * scale))
        ComponentSetValue2(character_data_component, "collision_aabb_max_y", math.max(0.5, enemy_values.collision_aabb_max_y * scale))
    end

    if enemy_values.has_boss_dragon_component == "true" then
        local dragon_part_distance = tonumber(enemy_values.dragon_part_distance)
        local dragon_hitbox_radius = tonumber(enemy_values.dragon_hitbox_radius)
        local dragon_ground_check_offset = tonumber(enemy_values.dragon_ground_check_offset)
        local dragon_speed = tonumber(enemy_values.dragon_speed)
        local dragon_speed_hunt = tonumber(enemy_values.dragon_speed_hunt)

        local half_dampened_scale = 1 + ((scale - 1) / 2)
        if scale < 1 then
            half_dampened_scale = 1 - ((1 - scale) / 2)
        end

        local boss_dragon_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "BossDragonComponent")
        ComponentSetValue2(boss_dragon_component, "part_distance", dragon_part_distance * scale)
        ComponentSetValue2(boss_dragon_component, "hitbox_radius", dragon_hitbox_radius * scale)
        ComponentSetValue2(boss_dragon_component, "ground_check_offset", dragon_ground_check_offset * scale)
        ComponentSetValue2(boss_dragon_component, "speed", dragon_speed * half_dampened_scale)
        ComponentSetValue2(boss_dragon_component, "speed_hunt", dragon_speed_hunt * half_dampened_scale)
    end
    
    if enemy_values.has_worm_component == "true" then
        local worm_part_distance = tonumber(enemy_values.worm_part_distance)
        local worm_hitbox_radius = tonumber(enemy_values.worm_hitbox_radius)
        local worm_ground_check_offset = tonumber(enemy_values.worm_ground_check_offset)
        local worm_target_kill_radius = tonumber(enemy_values.worm_target_kill_radius)
        local worm_speed = tonumber(enemy_values.worm_speed)
        local worm_gravity = tonumber(enemy_values.worm_gravity)
        local worm_tail_gravity = tonumber(enemy_values.worm_tail_gravity)

        --after dampening speed scaling half, 5x size worms are just barely escapable if you're lucky
		local half_dampened_scale = 1
        if scale < 1 then
            half_dampened_scale = 1 - ((1 - scale) / 2)
		elseif scale ~= nil then
			half_dampened_scale = 1 + ((scale - 1) / 2)
        end

        local worm_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "WormComponent")
        ComponentSetValue2(worm_component, "part_distance", worm_part_distance * scale)
        ComponentSetValue2(worm_component, "hitbox_radius", worm_hitbox_radius * scale)
        ComponentSetValue2(worm_component, "ground_check_offset", worm_ground_check_offset * scale)
        ComponentSetValue2(worm_component, "target_kill_radius", worm_target_kill_radius * scale)
        ComponentSetValue2(worm_component, "speed", worm_speed * half_dampened_scale)

        --gravity seems to overwhelm big worms and drag them down
        --not sure how to counteract this beyond nearly turning off gravity when they're big
        local new_gravity = worm_gravity
        if scale > 1 then
            new_gravity = math.max(0.1, worm_gravity - ((worm_gravity * scale) / 4))
        end
        ComponentSetValue2(worm_component, "gravity",new_gravity)
        local new_tail_gravity = worm_tail_gravity
        if scale > 1 then
            new_tail_gravity = math.max(0.1, worm_tail_gravity - ((worm_tail_gravity * scale) / 4))
        end
        ComponentSetValue2(worm_component, "tail_gravity", new_tail_gravity)
    end

    if enemy_values.has_worm_ai_component == "true" then
        local worm_ai_give_up_area_radius = tonumber(enemy_values.worm_ai_give_up_area_radius)
        local worm_ai_speed = tonumber(enemy_values.worm_ai_speed)
        local worm_ai_speed_hunt = tonumber(enemy_values.worm_ai_speed_hunt)

        local worm_ai_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "WormAIComponent")
        ComponentSetValue2(worm_ai_component, "give_up_area_radius", worm_ai_give_up_area_radius * scale)
        ComponentSetValue2(worm_ai_component, "speed", worm_ai_speed * half_dampened_scale)
        ComponentSetValue2(worm_ai_component, "speed_hunt", worm_ai_speed_hunt * half_dampened_scale)
    end

    if enemy_values.has_cell_eater_component == "true" then
        local cell_eater_component = EntityGetFirstComponentIncludingDisabled(enemy_id, "CellEaterComponent")
        ComponentSetValue2(cell_eater_component, "radius", enemy_values.cell_eater_radius * scale)
    end

    local hitbox_components = EntityGetComponentIncludingDisabled(enemy_id, "HitboxComponent")
    if hitbox_components ~= nil then
        local hitbox_num = 1
        for _, hitbox_component in ipairs(hitbox_components) do
            ComponentSetValue(hitbox_component, "aabb_min_x", enemy_values["hitbox_aabb_min_x_" .. hitbox_num] * scale)
            ComponentSetValue(hitbox_component, "aabb_max_x", enemy_values["hitbox_aabb_max_x_" .. hitbox_num] * scale)
            ComponentSetValue(hitbox_component, "aabb_min_y", enemy_values["hitbox_aabb_min_y_" .. hitbox_num] * scale)
            ComponentSetValue(hitbox_component, "aabb_max_y", enemy_values["hitbox_aabb_max_y_" .. hitbox_num] * scale)
            hitbox_num = hitbox_num + 1
        end
    end

    --kolmi's shield visuals
    if EntityHasTag(enemy_id, "boss_centipede") then
        local children = EntityGetAllChildren(enemy_id)
        if children ~= nil then
            for _, child in ipairs(children) do
                if EntityGetName(child) == "shield_entity" then
                    scale_energy_shields_on_entity(child, scale)
                    scale_particle_effects_on_entity(child, scale)
                    local sprite_component = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
                    if sprite_component ~= nil then
                        ComponentSetValue2(sprite_component, "has_special_scale", true)
                        ComponentSetValue2(sprite_component, "special_scale_x", scale)
                        ComponentSetValue2(sprite_component, "special_scale_y", scale)
                    end
                end
            end
        end
    end

    --levi's eye and tentacles
    if EntityHasTag(enemy_id, "boss_fish") then
        local children = EntityGetAllChildren(enemy_id)
        if children ~= nil then
            for _, child in ipairs(children) do
                local sprite_component = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
                if sprite_component ~= nil then
                    ComponentSetValue2(sprite_component, "has_special_scale", false)
                end
                local child_x, child_y, child_rotation, child_scale_x, child_scale_y = EntityGetTransform(child)
                EntitySetTransform(child, child_x, child_y, child_rotation, scale, scale)
            end
        end
    end

    scale_particle_effects_on_entity(enemy_id, scale)
    local children = EntityGetAllChildren(enemy_id)
    for _, child in ipairs(children) do
        scale_particle_effects_on_entity(child, scale)

        --ik limb scaling
        local ik_limb_component = EntityGetFirstComponentIncludingDisabled(child, "IKLimbComponent")
        if ik_limb_component ~= nil then
            scale_ik_entity(child, scale)
        end
    end
end

function scale_perk(perk_entity, scale)
    --lukki legs / leggy
    local sprite_components = EntityGetComponentIncludingDisabled(perk_entity, "SpriteComponent")
    if sprite_components ~= nil then
        for _, sprite_component in ipairs(sprite_components) do
            local image_file = ComponentGetValue2(sprite_component, "image_file")
            if string.find(image_file, "data/entities/misc/perks/attack_foot") ~= nil or
            string.find(image_file, "data/entities/misc/perks/attack_leggy") ~= nil then
                scale_ik_entity(perk_entity, scale)
                return
            end
        end
    end

    --vomit rats, freeze field, fire gas
    local convert_material_components = EntityGetComponentIncludingDisabled(perk_entity, "MagicConvertMaterialComponent")
    if convert_material_components ~= nil then
        scale_convert_material_fields_on_entity(perk_entity, scale)
    end

    --dissolve powders
    local cell_eater_component = EntityGetComponentIncludingDisabled(perk_entity, "MagicConvertMaterialComponent")
    if cell_eater_component ~= nil then
        scale_cell_eaters_on_entity(perk_entity, scale)
    end

    --particle effects on fungal disease, fungal, lukki minion, mana from kills, 
    --plague rats, projectile sector, projectile repulsion, risky crits, energy shield, 
    --spore pods, teleportitis dodge perks, also the player halos
    local particle_emitter_components = EntityGetComponentIncludingDisabled(perk_entity, "ParticleEmitterComponent")
    if particle_emitter_components ~= nil then
        scale_particle_effects_on_entity(perk_entity, scale)
    end

    --energy shield radius
    local energy_shield_component = EntityGetComponentIncludingDisabled(perk_entity, "EnergyShieldComponent")
    if energy_shield_component ~= nil then
        scale_energy_shields_on_entity(perk_entity, scale)
    end

    --contact damage
    local area_damage_component = EntityGetComponentIncludingDisabled(perk_entity, "AreaDamageComponent")
    if area_damage_component ~= nil then
        scale_area_damage_on_entity(perk_entity, scale)
    end
end

function scale_ik_entity(ik_entity, scale)
    local sprite_components = EntityGetComponentIncludingDisabled(ik_entity, "SpriteComponent")
    if sprite_components ~= nil then
        local update_last_scale = false
        for _, sprite_component in ipairs(sprite_components) do
            --free sprite scale from IK control for 1 frame to update x/y scale. filthy hack.
            if GameGetFrameNum() % 20 == 0 then
                local last_scale = get_variable_component(ik_entity, "last_scale", "value_float")
                if last_scale ~= scale then
                    update_last_scale = true
                    ComponentSetValue2(sprite_component, "update_transform", true)
                    ComponentSetValue2(sprite_component, "visible", false)
                    ComponentSetValue2(sprite_component, "has_special_scale", true)
                    ComponentSetValue2(sprite_component, "special_scale_x", scale)
                    ComponentSetValue2(sprite_component, "special_scale_y", scale)
                end
            else
                ComponentSetValue2(sprite_component, "update_transform", false)
                ComponentSetValue2(sprite_component, "visible", true)
            end
        end
        if update_last_scale then
            set_variable_component(ik_entity, "last_scale", "value_float", scale)
        end
    end

    local ik_limb_component = EntityGetFirstComponentIncludingDisabled(ik_entity, "IKLimbComponent")
    if ik_limb_component ~= nil then
        local original_ik_limb_length = get_variable_component(ik_entity, "original_ik_limb_length", "value_float")
        if original_ik_limb_length == nil then 
            original_ik_limb_length = ComponentGetValue2(ik_limb_component, "length")
            set_variable_component(ik_entity, "original_ik_limb_length", "value_float", original_ik_limb_length)
        end
        ComponentSetValue(ik_limb_component, "length", original_ik_limb_length * scale)
    end

    local ik_limb_attacker_component = EntityGetFirstComponentIncludingDisabled(ik_entity, "IKLimbAttackerComponent")
    if ik_limb_attacker_component ~= nil then
        local original_ik_limb_radius = get_variable_component(ik_entity, "original_ik_limb_radius", "value_float")
        if original_ik_limb_radius == nil then
            original_ik_limb_radius = ComponentGetValue2(ik_limb_attacker_component, "radius")
            set_variable_component(ik_entity, "original_ik_limb_radius", "value_float", original_ik_limb_radius)
        end
        ComponentSetValue(ik_limb_attacker_component, "radius", original_ik_limb_radius * scale)
    end

    local ik_limb_walker_component = EntityGetFirstComponentIncludingDisabled(ik_entity, "IKLimbWalkerComponent")
    if ik_limb_walker_component ~= nil then
        local original_ik_limb_spread = get_variable_component(ik_entity, "original_ik_limb_spread", "value_float")
        if original_ik_limb_spread == nil then
            original_ik_limb_spread = ComponentGetValue2(ik_limb_walker_component, "ground_attachment_min_spread")
            original_ik_limb_velocity = ComponentGetValue2(ik_limb_walker_component, "leg_velocity_coeff")

            set_variable_component(ik_entity, "original_ik_limb_spread", "value_float", original_ik_limb_spread)
            set_variable_component(ik_entity, "original_ik_limb_velocity", "value_float", original_ik_limb_velocity)
        end
        ComponentSetValue(ik_limb_walker_component, "ground_attachment_min_spread", original_ik_limb_spread * scale)
        ComponentSetValue(ik_limb_walker_component, "leg_velocity_coeff", original_ik_limb_velocity * (1 / scale))
    end
end

function get_held_item(player)
    local inventory = EntityGetFirstComponentIncludingDisabled(player, "Inventory2Component")
    if inventory == nil then return nil end
    local item = ComponentGetValue2(inventory, "mActiveItem")
    return item
end