--https://noita.fandom.com/wiki/Modding:_Utilities#Internal_variable_manipulation
function get_variable_component(entity_id, variable_name, variable_type)
	local value = nil
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if(var_name == variable_name) then
				value = ComponentGetValue2(comp_id, variable_type)
			end
		end
	end
	return value
end

--https://noita.fandom.com/wiki/Modding:_Utilities#Internal_variable_manipulation
function set_variable_component(entity_id, variable_name, variable_type, new_value)
    local set = false
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if( var_name == variable_name) then
				ComponentSetValue2( comp_id, variable_type, new_value )
                set = true
			end
		end
	end
    if set == false then
        add_variable_component(entity_id, variable_name, variable_type, new_value)
    end
end

--https://noita.fandom.com/wiki/Modding:_Utilities#Internal_variable_manipulation
function add_variable_component(entity_id, variable_name, variable_type, initial_value)
	if(variable_type == "value_int") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_int=initial_value
		})
	elseif(variable_type == "value_string") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_string=initial_value
		})
	elseif(variable_type == "value_float") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_float=initial_value
		})
	elseif(variable_type == "value_bool") then
		EntityAddComponent2(entity_id, "VariableStorageComponent", {
			name=variable_name,
			value_bool=initial_value
		})
	end
end

function remove_variable_component(entity_id, variable_name)
	local removed = false
	local components = EntityGetComponentIncludingDisabled( entity_id, "VariableStorageComponent" )
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue2( comp_id, "name" )
			if( var_name == variable_name) then
				EntityRemoveComponent(entity_id, comp_id)
                removed = true
			end
		end
	end
    return removed
end

--flat (non-nested) lua table to string
function table_to_string(t)
    local result = ''
    for k, v in pairs(t) do
        result = result .. ' ' .. tostring(k) .. '=' .. tostring(v)
	end
	--result = result:sub(1, result:len()-1)
	return result
end

--simple string to flat lua table
function string_to_table(s)
	local t = {}
	for pair in string.gmatch(s, "[^%s]+") do
		local halves = {}
		for half in string.gmatch(pair, "[^=]+") do
			table.insert(halves, half)
		end
		t[halves[1]] = halves[2]
	end
	return t
end

--https://steamcommunity.com/app/573090/discussions/0/1639792569839228614/#c1636416951459602507
function math.round(x, decimals)
    local n = 10^(decimals or 0)
    x = x * n
    if x >= 0 then x = math.floor(x + 0.5) else x = math.ceil(x - 0.5) end
    return x / n
end

--https://stackoverflow.com/a/72776077
function math.clamp(x, min, max)
    return math.max(math.min(x, max), min)
end

--[[--https://stackoverflow.com/a/68840015
function table.merge(...)
	local result <const> = {}
	-- For each source table
	for _, t in ipairs{...} do
		-- For each pair in t
		for k, v in pairs(t) do
			result[k] = v
		end
	end
	return result
end]]