dofile_once("data/scripts/lib/utilities.lua")

local distance_full = 300

local entity_id = GetUpdatedEntityID()
local x, y, rot = EntityGetTransform( entity_id )

function calculate_force_at(body_x, body_y)
	local distance = math.sqrt( ( x - body_x ) ^ 2 + ( y - body_y ) ^ 2 )
	local direction = 0 - math.atan2( ( y - body_y ), ( x - body_x ) )

	local gravity_percent = ( distance_full - distance ) / distance_full
	local gravity_coeff = 100
	
	local fx = math.cos( direction ) * ( gravity_coeff * gravity_percent )
	local fy = -math.sin( direction ) * ( gravity_coeff * gravity_percent )

	return fx,fy
end

-- attract projectiles
local entities = EntityGetInRadiusWithTag(x, y, distance_full, "selective_hole")
for _,id in ipairs(entities) do	
	local physicscomp = EntityGetFirstComponent(id, "PhysicsBody2Component") or EntityGetFirstComponent( id, "PhysicsBodyComponent")
	if physicscomp == nil and not EntityHasTag(id,"black_hole") then
		local px, py = EntityGetTransform( id )

		local velocitycomp = EntityGetFirstComponent( id, "VelocityComponent" )
		if ( velocitycomp ~= nil ) then
			local fx, fy = calculate_force_at(px, py)
			edit_component( id, "VelocityComponent", function(comp,vars)
				local vel_x,vel_y = ComponentGetValue2( comp, "mVelocity")
				
				vel_x = vel_x + fx
				vel_y = vel_y + fy

				ComponentSetValue2( comp, "mVelocity", vel_x, vel_y)
			end)
		end
	end
end
	
for i=1, Random(1, 6) do
	local rand_dir = Randomf(-math.pi, math.pi)
	local inner_radius = 20
	local outer_radius = 300
	local x1 = x + math.cos(rand_dir) * inner_radius
	local y1 = y + math.sin(rand_dir) * inner_radius
	local x2 = x + math.cos(rand_dir) * outer_radius
	local y2 = y + math.sin(rand_dir) * outer_radius
	local did_hit, hit_x, hit_y = RaytraceSurfaces(x1, y1, x2, y2)
	if did_hit then
		local particle_checker = EntityLoad( "mods/new_enemies/files/selective_hole/selective_hole_material_checker.xml", hit_x, hit_y )
		EntityAddChild(entity_id, particle_checker)
	end
end