-- dofile_once("mods/koira/files/scripts/component_list.lua")

function clamp(value, min, max)
	if value > max then return max
	elseif value < min then return min
	else return value end
end
function sign(val)
	if val > 0 then return 1
	elseif val < 0 then return -1
	else return 0 end
end
function length(x, y)
  return math.sqrt(x * x + y * y)
end
function norVector(x, y)
  local len = length(x, y)
  x = x / len
  y = y / len
  return x, y
end
function dotProduct(x1, y1, x2, y2)
	return x1*x2 + y1*y2
end
function radToDeg(rad)
	local deg = (rad * 180) / math.pi
	if deg < 0 then deg = 360 + deg end
	return deg
end
function degToRad(deg)
	return deg * 0.01745329252
end
function angle(cx, cy, ex, ey)
	local dy = ey - cy
	local dx = ex - cx
	local theta = math.atan2(dy, dx)
	return radToDeg(theta)
end
function Distance(x1, y1, x2, y2)
	if x1 and x2 then return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
	else return 0 end
end
function Cross(x1, y1, x2, y2)
	return (x1*y2) - (y1*x2)
end
function boolToInt(value)
  return value and 1 or 0
end
function GlobalToLocal(x1, y1, x2, y2, x)
	local xs = math.sin(x)
	local xc = math.cos(x)
	return ((x2-x1)*xc)+((y2-y1)*xs), (-(x2-x1)*xs)+((y2-y1)*xc)
end
function LocalToGlobal(x1, y1, x2, y2, x)
	local xs = math.sin(x)
	local xc = math.cos(x)
	return (x2*xc)-(y2*xs)+x1, (x2*xs)+(y2*xc)+y1
end

--Applies force as if it was given on a sertain point on the entity
function ApplyForceAtPoint(entity, point_x, point_y, force_x, force_y, angular_drag)
	ApplyForcePlus(entity, force_x, force_y)
	ApplyTorqueFromPoint(entity, point_x, point_y, force_x, force_y, angular_drag)
end
--Applies angular force as if a force was given on a sertain point on the entity
function ApplyTorqueFromPoint(entity, point_x, point_y, force_x, force_y, drag)
	local x, y, rot = EntityGetTransform(entity)
	
	local torque = -Cross(x - point_x, y - point_y, force_x, force_y)

	if tostring(torque) ~= "nan" then
		PhysicsApplyTorque(entity, clamp((torque / 50) - drag, -20, 20))
	end
end
--Improved force function that works with creatures and other things that doesnt use "proper" physics
function ApplyForcePlus(entity, f_x, f_y)
	if tostring(f_x) ~= "nan" and tostring(f_y) ~= "nan" and not EntityHasTag( entity, "boss" ) and not EntityHasTag( entity, "boss_tag" ) and not EntityHasTag( entity, "miniboss" ) then

		local phyC = EntityGetFirstComponent(entity,"PhysicsBodyComponent") or EntityGetFirstComponent(entity,"PhysicsBody2Component")
		local charC = EntityGetFirstComponent(entity,"CharacterDataComponent")
		local velC = EntityGetFirstComponent(entity,"VelocityComponent")

		if phyC then
		    PhysicsApplyForce(entity, f_x, f_y)
		elseif charC then
			local posX, posY = EntityGetTransform(entity)
		    local vel_x, vel_y = ComponentGetValue2(charC,"mVelocity")
		    local mass = ComponentGetValue2(charC,"mass")

		    vel_x = vel_x + f_x / mass
		    vel_y = vel_y + f_y / mass
 
			-- if not EntityHasTag(entity, "player_unit") then CancelAccel(entity) end
			ComponentSetValue2(charC, "mVelocity", vel_x, vel_y)
		elseif velC then
		    local vel_x, vel_y = ComponentGetValue2(velC,"mVelocity")
		    local mass = ComponentGetValue2(velC,"mass")
		    vel_x = vel_x + f_x / (mass * 20)
		    vel_y = vel_y + f_y / (mass * 20)
		    ComponentSetValue2(velC, "mVelocity", vel_x, vel_y)
		end
	end
end
--used for specific stuff releted to items
function FakePickup(entity)
	local x, y = EntityGetTransform(entity)
	local tempInv = EntityLoad((filePath.."temp_inv.xml"), x, y)

	GamePickUpInventoryItem(tempInv, entity, false)
	GameDropAllItems(tempInv)
	EntitySetTransform(entity, x, y)
	EntityKill(tempInv)
end
--gets the item an entity is currently holding
function GetHeldItem(entity)
	local comp = EntityGetFirstComponent(entity, "Inventory2Component")
	return ComponentGetValue2(comp, "mActiveItem")
end
--set variable in a VariableStorageComponent
function EntitySetVariable(entity, tag, type, set)
	local storage = EntityGetFirstComponentIncludingDisabled(entity, "VariableStorageComponent", tag)
	if not storage then
		storage = EntityAddComponent2(entity, "VariableStorageComponent", {name=tag})
		ComponentAddTag(storage, tag)
	end
	ComponentSetValue2(storage, type, set)
	return set
end
--get variable in a VariableStorageComponent
function EntityGetVariable(entity, tag, type)
	local storage = EntityGetFirstComponentIncludingDisabled(entity, "VariableStorageComponent", tag)
	
	if not storage then
		local variablestorages = EntityGetComponent( entity, "VariableStorageComponent" )
		if variablestorages then
			for j,storage_id in ipairs(variablestorages) do
				local var_name = ComponentGetValue( storage_id, "name" )
				if ( var_name == tag ) then
					storage = storage_id
				end
			end
		end
	end

	if storage then return ComponentGetValue2(storage, type) end

	return nil
end
--set the acceleration of a creature to 0, and gives it back when they are on the ground.
function CancelAccel(entity)
	local char = EntityGetFirstComponent(entity, "CharacterPlatformingComponent")
	local charData = EntityGetFirstComponent(entity,"CharacterDataComponent")
	local currentAccel = ComponentGetValue2(char, "accel_x")
	local comp = EntityGetFirstComponent(entity, "VariableStorageComponent","accel")
	
	if not comp then
		local a = EntityAddComponent2(entity, "VariableStorageComponent", {name="accel", value_float=currentAccel})
		local la = EntityAddComponent2(entity, "LuaComponent", {script_source_file=(filePath.."add_accel.lua"),execute_every_n_frame =1})
		ComponentAddTag(a, "accel")
		ComponentAddTag(la, "accel_lua")
	end
	ComponentSetValue2(char, "accel_x", 0)
	ComponentSetValue2(charData, "mCollidedHorizontally", true)
end
--set the position of all verlet points, used when teleporting a verlet entity.
function SetVerletAllPositions(entity, x, y)
	component_readwrite(EntityGetFirstComponent(entity, "VerletPhysicsComponent"), {positions = {}, positions_prev={}}, function(verlet_parts)
		for i = 1, #verlet_parts.positions, 2 do
			verlet_parts.positions[i] = x
			verlet_parts.positions[i + 1] = y
			verlet_parts.positions_prev[i] = x
			verlet_parts.positions_prev[i + 1] = y
		end
	end)
end
--currently not much, will expand on later
function Drag(f, vel, drag)
	return f - (drag * vel)
end
--get the material of the pixel at the selected coordinates
--need to have coroutines in the script it is getting executed in
--"func" is the name of the function it should execute after getting the material, it should look something like this: [function name_of_function(mat)] from here you can reference "mat" to get the material
function GetMaterial(x, y, func)
	async(function()
		local sucker = EntityLoad(filePath.."sucker.xml", x, y)
		wait(1)
		local matInv = EntityGetFirstComponent(sucker, "MaterialInventoryComponent")
		local mats = ComponentGetValue2(matInv, "count_per_material_type")
	
		for i = 1, #mats, 1 do
			if mats[i] ~= 0 then
				EntityKill(sucker)
				local mat = CellFactory_GetName(i - 1)
				_G[func](mat)
				return
			end
		end
	end)
end
--set a pixel with selected material at the selected coordinates
function SetMaterial(material, x, y)
	ChangeMaterial("air", x, y)--incase there is a pixel occupying coordinates, turn it into air

	local spawner = EntityCreateNew("spawner")
	EntitySetTransform(spawner, x, y)
	local vars=
	{
	 emitted_material_name=material,
	 image_animation_file=(filePath.."spawner.png"),

	 create_real_particles=true,
	 count_min=1,
	 count_max=1,
	 render_on_grid=true,
	 fade_based_on_lifetime=true,
	 cosmetic_force_create=false,
	 emission_interval_min_frames=1,
	 emission_interval_max_frames=1,
	 emit_cosmetic_particles=false,
	 image_animation_speed=2,
	 image_animation_loop=false,
	 image_animation_raytrace_from_center=false,
	 collide_with_gas_and_fire=false,	
	 is_emitting=true
	}

	EntityAddComponent2(spawner, "ParticleEmitterComponent", vars)
	EntityAddComponent2(spawner, "LifetimeComponent", {lifetime=5})
end
--change the material of the pixel at the selected coordinates(cant change air)
function ChangeMaterial(material, x, y)
	local spawner = EntityCreateNew("spawner")
	EntitySetTransform(spawner, x, y)
	local vars=
	{
	 from_any_material=true,
	 to_material=CellFactory_GetType(material),
	 is_circle=false,
	 radius=1,
	 kill_when_finished=true
	}

	EntityAddComponent2(spawner, "MagicConvertMaterialComponent", vars)
	EntityAddComponent2(spawner, "LifetimeComponent", {lifetime=5})
end
--convert the material of pixels in a radius of selected coordinates with the "from" material to the "to" material
function ConvertMaterial(from, to, x, y, r)
	local spawner = EntityCreateNew("spawner")
	EntitySetTransform(spawner, x, y)
	local vars=
	{
	 from_material=CellFactory_GetType(from),
	 to_material=CellFactory_GetType(to),
	 is_circle=true,
	 radius=r,
	 kill_when_finished=true
	}

	EntityAddComponent2(spawner, "MagicConvertMaterialComponent", vars)
	EntityAddComponent2(spawner, "LifetimeComponent", {lifetime=5})
end

function GetWhoShot(entity)
	local comp = EntityGetFirstComponent(entity, "ProjectileComponent")
	return ComponentGetValue2(comp, "mWhoShot")
end

function ObstaclesInTheWay(x1, y1, x2, y2)
	local hit, hitX, hitY = Raytrace(x1, y1, x2, y2)
	if Distance(hitX, hitY, x2, y2) < 15 or not hit then return false end
	return true
end

function StickToEntity(entity, stick_to)
	--print("stick")
	local c =EntityAddComponent2(entity, "LuaComponent", {script_source_file=(filePath.."stick_to_entity.lua"),execute_every_n_frame =1})
	ComponentAddTag(c, "stick_to_entity")
	local x1, y1, rot1 = EntityGetTransform(entity)
	local x2, y2, rot2 = EntityGetTransform(stick_to)
	
	local dir_x, dir_y = x1 - x2, y1 - y2
	local angleAdd = math.atan2(dir_y, dir_x) - rot2
	
	EntitySetVariable(entity, "dist", "value_float", Distance(x1, y1, x2, y2))
	EntitySetVariable(entity, "angleAdd", "value_float",  angleAdd)
	EntitySetVariable(entity, "rotAdd", "value_float", rot1 - rot2)
	EntitySetVariable(entity, "stick_to", "value_int", stick_to)
end

function UnStickFromEntity(entity)
	local comp = EntityGetFirstComponent(entity, "VelocityComponent")
	if comp then ComponentSetValue2(comp, "mVelocity", 0, 0) end
	local c =EntityGetFirstComponent(entity, "LuaComponent", "stick_to_entity")
	local dist=EntityGetFirstComponent(entity, "VariableStorageComponent", "dist")
	local aAdd=EntityGetFirstComponent(entity, "VariableStorageComponent", "angleAdd")
	local rAdd=EntityGetFirstComponent(entity, "VariableStorageComponent", "rotAdd")
	local stick=EntityGetFirstComponent(entity, "VariableStorageComponent", "stick_to")
	
	EntityRemoveComponent(entity, c)
	EntityRemoveComponent(entity, dist)
	EntityRemoveComponent(entity, aAdd)
	EntityRemoveComponent(entity, rAdd)
	EntityRemoveComponent(entity, stick)
end
--to do: make this work with anything
function Velocity(entity)
	local comp = EntityGetFirstComponent(entity, "VelocityComponent")
	return ComponentGetValue2(comp, "mVelocity")
end

function DuplicateComponentsToEntity(from, entity, tag_to_ignore)
	local components = EntityGetAllComponents(from)
	for i, comp in ipairs(components) do
		
		if ComponentHasTag(comp, tag_to_ignore) then
			goto continue
		end
		
		local cType = ComponentGetTypeName(comp)
		local cMembers = comps[cType]
		
		local cID
		
		if cType == "SpriteComponent" then
			local file = ComponentGetValue2(comp, "image_file")
			cID= EntityAddComponent(entity, cType, {image_file=file})
		else
			cID= EntityAddComponent(entity, cType)
		end
		
		for i, memb in ipairs(cMembers) do
			local value = ComponentGetValue2(comp, memb)
			local v1, v2 = ComponentGetValue2(comp, memb)
			local vv1, vv2, vv3= ComponentGetValue2(comp, memb)
			local vvv1, vvv2, vvv3, vvv4 = ComponentGetValue2(comp, memb)
			local vvvv1, vvvv2, vvvv3, vvvv4, vvvv5 = ComponentGetValue2(comp, memb)
			if value and not v2 and not vv3 and not vvv4 and not vvvv5 then
				ComponentSetValue2(cID, memb, value)
			elseif v2 and not vv3 and not vvv4 and not vvvv5 then
				ComponentSetValue2(cID, memb, v1, v2)
			elseif vv3 and not vvv4 and not vvvv5 then
				ComponentSetValue2(cID, memb, vv1, vv2, vv3)
			elseif vvv4 and not vvvv5 then
				ComponentSetValue2(cID, memb, vvv1, vvv2, vvv3, vvv4)
			elseif vvvv5 then
				ComponentSetValue2(cID, memb, vvvv1, vvvv2, vvvv3, vvvv4, vvvv5)
			end
		end
		::continue::
	end
end

function Freeze(entity)
	if EntityGetIsAlive(entity) then
		local charC = EntityGetFirstComponentIncludingDisabled(entity,"CharacterDataComponent")
		local sPhy = EntityGetFirstComponentIncludingDisabled(entity,"SimplePhysicsComponent")

		if charC then EntitySetComponentIsEnabled(entity, charC, false) end
		if sPhy then EntitySetComponentIsEnabled(entity, sPhy, false) end
		PhysicsSetStatic( entity, true )
	end
end

function Unfreeze(entity)
	if EntityGetIsAlive(entity) then
		local charC = EntityGetFirstComponentIncludingDisabled(entity,"CharacterDataComponent")
		local sPhy = EntityGetFirstComponentIncludingDisabled(entity,"SimplePhysicsComponent")
	
		if charC then EntitySetComponentIsEnabled(entity, charC, true) end
		if sPhy then EntitySetComponentIsEnabled(entity, sPhy, true) end
		PhysicsSetStatic( entity, false )
	end
end

function DebugSquare(x, y)
	local square = EntityCreateNew("debug_square")
	EntitySetTransform(square, x, y)
	EntityAddComponent2(square, "SpriteComponent", {z_index=-5,alpha=0.8,offset_x=1, offset_y=1, image_file=(filePath.."debug_square.png")})
	EntityAddComponent2(square, "LifetimeComponent", {lifetime=1})
end

function DebugLine(x1, y1, x2, y2, mat)
	if mat == nil then mat = "spark_blue" end
	local line = EntityLoad(filePath.."debug_line.xml", x1, y1)
	local particles = EntityGetFirstComponent(line, "ParticleEmitterComponent")
	EntityAddComponent2(line, "LifetimeComponent", {lifetime=2})
	ComponentSetValue2(particles, "mExPosition", x2, y2)
	ComponentSetValue2(particles, "emitted_material_name", mat)
end

function GetClosestPointOnGround(x, y, rayLength, rayCount)
	local minX, minY= 0, 0
	local minDist = 10000
	local a = 0
	for i = 1, rayCount, 1 do
		local angle = (360/rayCount)*i
		local rad = degToRad(angle)
		local addX, addY = math.cos(rad)*rayLength, math.sin(rad)*rayLength
	
		local did_hit, hitX, hitY = RaytracePlatforms(x, y, x+addX, y+addY)
		local dist = Distance(hitX, hitY, x, y)
	
		if did_hit and dist < minDist then
			minX, minY = hitX, hitY
			minDist = dist
			a = angle
		end
	end
	return minX, minY, minDist, a
end

function EntityGetVariables(entity, name, type)	
	local variables = {}
	local variablesStorage = {}
	local variablestorages = EntityGetComponent( entity, "VariableStorageComponent" )or{}
	for i,storage_id in ipairs(variablestorages) do
		local compName = ComponentGetValue2( storage_id, "name" )
		if compName == name then
			local var = ComponentGetValue2(storage_id, type)
			table.insert(variablesStorage, storage_id)
			table.insert(variables, var)
		end
	end
	return variables, variablesStorage
end
--adds a outline to entity with the color of material
function AddOutline(entity, thickness, material, tag)
	local x, y, rot, scaleX, scaleY = EntityGetTransform(entity)
	local outline = EntityLoad(filePath.."outline.xml", x, y)

	local sprite = EntityGetFirstComponent(entity, "SpriteComponent")
	local file = ComponentGetValue2(sprite, "image_file")
	local z = ComponentGetValue2(sprite, "z_index")
	local oX, oY = ComponentGetValue2(sprite, "offset_x"), ComponentGetValue2(sprite, "offset_y")
	local ani = ComponentGetValue2(sprite, "rect_animation")
	
	EntityAddTag(outline, tag)
	EntityAddChild(entity, outline)
	
	EntityAddComponent2(outline, "SpriteComponent", {image_file=file, offset_x=oX, offset_y=oY, z_index =(z+0.1), rect_animation=ani})
	EntityAddComponent2(outline, "LuaComponent", {script_source_file=(filePath.."follow_parent.lua")})
	
	AddMaterialInventoryMaterial(outline, material, 1)
	EntitySetTransform(outline, x, y, rot, scaleX+thickness, scaleY+thickness)
end

function AddSprite(entity, settings, tag, mat)
	local x, y, rot, scaleX, scaleY = EntityGetTransform(entity)
	local sprite = EntityCreateNew("temp sprite")
	EntityAddTag(sprite,tag)
	EntityAddChild(entity, sprite)
	EntityAddComponent2(sprite, "SpriteComponent", settings)
	EntityAddComponent2(sprite, "LuaComponent", {script_source_file=(filePath.."follow_parent.lua")})
	if mat then
		EntityAddComponent2(sprite, "MaterialInventoryComponent")
		EntityAddComponent2(sprite, "PotionComponent")
		AddMaterialInventoryMaterial(sprite, mat, 1)
	end
end

function GetFirstChildWithTag(entity, tag)
	local children = EntityGetAllChildren(entity)or{}
	for i, child in ipairs(children) do
		if EntityHasTag(child, tag) then return child end
	end
end

function GetChildrenWithTag(entity, tag)
	local children = EntityGetAllChildren(entity)or{}
	local withTag = {}
	for i, child in ipairs(children) do
		if EntityHasTag(child, tag) then table.insert(withTag, child) end
	end
	return withTag
end

function Line(entity1, entity2, mat)
	if mat == nil then mat = "spark_blue" end
	local line = EntityLoad(filePath.."debug_line.xml")
	local particles = EntityGetFirstComponent(line, "ParticleEmitterComponent")
	ComponentSetValue2(particles, "emitted_material_name", mat)
	EntityAddComponent2(line, "LuaComponent", {script_source_file=(filePath.."setLine.lua")})
	EntitySetVariable(line, "line_entity1", "value_int", entity1)
	EntitySetVariable(line, "line_entity2", "value_int", entity2)
	return line
end